c ----------------------------------------------------------------------
c --- CPRAMMET is a 'CALPUFF' version of PCRAMMET that uses the
c --- subroutine RADSOL to estimate solar radiation when this is
c --- missing from the input data.    D.Strimaitis 11/3/99
c --- Also modified to write radiation and RH fields when using
c --- CD144 surface data format.      D.Strimaitis 11/21/00
c --- VERSION :   00326
c --- Syntax error picked up by LF95 compiler fixed (comma missing at end
c --- of a continuation line) (no update to version)
c ---                                 D.Strimaitis 08/14/02
c ----------------------------------------------------------------------
      PROGRAM PCRAMMET
C***********************************************************************
C*    PCRAMMET    A Meteorological Pre-processor for EPA Dispersion Models
C*
C*    VERSION :   99169
C*
C*    PURPOSE:    Reads raw meteorological data - surface, mixing height
C*                and precipitation data from a variety of NCDC/SCRAM
C*                formats and prepares unformatted or ASCII met data file
C*                for EPA dispersion models.
C*
C*    PROGRAMMER: Jayant Hardikar, Jim Paumier
C*                PES Inc.
C*
C*    DATE:       August 15, 1995
C*
C*    INPUTS:     Surface data       - CD144 or SAMSON or SCRAM
C*                Mixing Height data - SCRAM expanded format
C*                Precipitation data - SAMSON or TD3240
C*                Other user options
C*
C*    OUTPUTS:    Unformatted or ASCII meteorological data ready for
C*                EPA dispersion models.
C*
C*    ASSUMPTIONS:
C*                All hours are contiguous
C*                All input files have same date/time
C*                24 hours of data per day
C*
C*    REVISIONS:
C*      06/30/98  Added necessary code to process Hourly United States
C*                Weather observations (HUSWO) data retrieved from CD.
C*                NOTE: Since the data retrieved from CD can either be 
C*                      in English or metric units and since the
C*                      default is English and since there is no 
C*                      indication of the units in the data file, 
C*                      all processing in PCRAMMET will assume English 
C*                      units.  This assumption is consistent with 
C*                      the processing that is performed in the program
C*                      that computes the twice daily mixing heights.
C*
C*     06/18/99  Added code to write a message to log file when 
C*               processing ASOS data begins
C*
C*               Made corrections for Y2K (subr. HR0024); these
C*               corrections will work only through the year 2049
C*
C*               Removed all conditional compile statements (Lahey 
C*               Fortran 90 does not support conditional compiling)
C*
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      INTEGER ISAMWX(9), IHUSWX(4), IP5(5), NPROC

      DATA NPROC /0/

C*    Process the command line options
      CALL GETCMD (IFCNT)

C*    Get user options
      CALL SETUP
      WRITE(*,*) ' '

C*    Process Input and Output Files
      CALL FILES

C*    Begin Loop Over Days (Use the End-of-File in Surface
C*    Data as the indicator of the processing period)
      IDAY = 0
      EOFSFC = .FALSE.
      DO WHILE (.NOT. EOFSFC)
         IDAY = IDAY + 1

C*       Initialize data to missing
         CALL VARINI

C*       Read Surface Data For This Day (24 Hours)
         CALL READSF (NREAD)
         NPROC = NPROC + NREAD

         IF (NREAD .EQ. 24 )THEN
C*          24 hours of surface observations were read, continue

C*          Read Mixing Height Data For This Day (AM/PM)
            CALL READMX

C*          Read Hourly Precipitation Data For This Day (24 Hours)
            CALL READPP

C*          Fill Up The Appropriate Meteorological Arrays, 
C*          including the date and time
            CALL FILMET (NREAD)

            Write (*,500 ) ISYR(1),ISMO(1),ISDY(1)

C*          Determine the Julian day for the current date;
C*          use the year, month and day from the first hour of the day
            CALL JULIAN (ISYR(1),ISMO(1),ISDY(1),JDAY,0)

C*          Calculate Sunrise/Sunset Times For This Day as well as
C*          the solar elevation for all hours of this day
            CALL SUN(JDAY)

C*          Begin Loop over All Hours For This Day
            DO 100 IHR = 1, 24

C*             Calculate date-hr integer (YYJJJHH)
               KDATHR = ISYR(IHR)*100000 + JDAY*100 + IHR

C*             Check Date/Time Stamps Between All Types Of Data
               CALL CHKDAT(IHR)

C*             Determine If The Hour Is 'Calm'
               CALL CHKCLM(IHR)

C*             Calculate Flow Vector (Randomized and
C*             Non Randomized) from Wind Direction
               CALL FLOVEC(IHR,JDAY)

C*             Convert the precipitation codes if WETFLG is set
               IF( WETFLG )THEN
                  IF( SFCTYP .EQ. 'CD144' .OR. SFCTYP .EQ. 'SCRAM' )THEN
C*                   Convert the CD144 codes to the precip code for the hour
                     DO 40 I = 1,5
                        IP5(I) = IPREC(I,IHR)
   40                CONTINUE
                     CALL PCODES(IDIAG,KDATHR,IP5,IPCODE(IHR),
     &                           NOWARN,SFCTYP)

                  ELSEIF (SFCTYP .EQ. 'SAMSON') THEN
C*                   Convert SAMSON Precip Codes to CD144 Types
                     DO 50 JX = 1,9
                        ISAMWX(JX) = IWXSAM(JX,IHR)
   50                CONTINUE

                     CALL SAMWX(ISAMWX,IP5)
                     CALL PCODES(IDIAG,KDATHR,IP5,IPCODE(IHR),
     &                           NOWARN,SFCTYP)

                  ELSEIF (SFCTYP .EQ. 'HUSWO') THEN
C*                   Convert HUSWO Precip Codes to CD144 Types

                     DO 55 JX = 1,4
                        IHUSWX(JX) = IWXHUS(JX,IHR)
   55                CONTINUE
                     CALL HUSWX(IHUSWX,IP5)
                     CALL PCODES(IDIAG,KDATHR,IP5,IPCODE(IHR),
     &                           NOWARN,SFCTYP)

                  ENDIF

               ELSE
                  IPCODE(IHR) = 0

               ENDIF

C*             Convert all Data to Appropriate Units
               CALL UNITS(IHR)

C*             Calculate the Stability Class For This Hour
               CALL STABIT(IDAY,IHR)

C*             Call Interpolated Mixing Height For This Hour
               CALL MIXHIT(IHR)

C*             Calculate U* And L For This Hour When Applicable
               IF( DRYFLG  .OR.  WETFLG )THEN
                  CALL USTARL(IHR)
               ENDIF

C*          End Loop over All Hours For This Day
100         CONTINUE

C*          Write Out Data for All Hours of This Day
            CALL WRITIT (JDAY)

         ELSE
            IF( NREAD .GT. 0  )THEN
               WRITE(IDIAG, 600) NREAD, ISYR(1),ISMO(1),ISDY(1)
  600          FORMAT( ' WARNING: ONLY',I3,' HRS DATA ON THE',
     &                 ' DAY FOLLOWING ',3I3,
     &               /,'          NO DATA WERE PROCESSED FOR THE',
     &                 ' ENTIRE DAY')
            ENDIF
         ENDIF

C*    End Loop Over Days
      END DO

C*    Write the # of hours processed
      IF( NREAD .EQ. 24 )THEN
         WRITE( IDIAG,750 )NPROC,SFCTYP
      ELSE
         WRITE( IDIAG,750 )NPROC-NREAD,SFCTYP
      ENDIF

      IF( IFCNT .LT.2 )THEN
         WRITE ( *, 1100 )
      ENDIF

C---- NOWARN was initialized as TRUE; if any warning message was written
C     to the log file, then NOWARN was set to FALSE
      WRITE( * ,760 )
      IF( .NOT. NOWARN )THEN
         WRITE( *,765 )
      ENDIF

C*    End Program
      STOP

  500 FORMAT( '+ Now processing day (yy/mm/dd) ' , 3(i2.2:,'/') )
  750 FORMAT( /,' PCRAMMET processed ',i6,' hours of ',a6,' data'/)
  760 FORMAT(//,4X, ' See the file PCRAM.LOG for a review of this run ')
  765 FORMAT( /,4X, ' *** Warning messages were generated by PCRAMMET',
     &              ' and are in PCRAM.LOG' )
 1100 FORMAT( //,' Your responses to the program prompts have been',
     &           ' written to the file "INPUTS."' )
      END


      SUBROUTINE GETCMD (IFCNT)
C***********************************************************************
C*    GETCMD Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE: Controls Retrieving Input and Output File Names From
C*             the Command Line for PCs, and OPENs the Files
C*
C*    PROGRAMMER: Roger Brode
C*
C*    DATE:       March 2, 1992
C*
C*    MODIFIED:   February 13, 1995
C*                Adapted for PCRAMMET to allow for zero
C*                command line parameters
C*
C*    INPUTS:  Command Line
C*
C*    OUTPUTS: Input Runstream File Name
C*             Output Print File Name
C*
C*    CALLED FROM:   MAIN PROGRAM
C***********************************************************************
C
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

C*    Declare the COMLIN Variable to Hold Contents of Command Line for Lahey
      CHARACTER*120 COMLIN
      INTEGER*2 LOCB(120),LOCE(120)
      LOGICAL INFLD

C*    Initialize variables
      COMDLN = .FALSE.

C*    Use Lahey Function GETCL To Retrieve Contents of Command Line.
C*    Retrieve Input and Output File Names From the COMLIN Variable.
      CALL GETCL(COMLIN)
      INFLD = .FALSE.
      IFCNT = 0
      DO 100 I = 1, 120
         IF (.NOT.INFLD .AND. COMLIN(I:I) .NE. ' ') THEN
            INFLD = .TRUE.
            IFCNT = IFCNT + 1
            LOCB(IFCNT) = I
         ELSE IF (INFLD .AND. COMLIN(I:I) .EQ. ' ') THEN
            INFLD = .FALSE.
            LOCE(IFCNT) = I - 1
         END IF
 100  CONTINUE

C*    If no command line arguments specified, set the logical
C*    flag to false, else set it to true and pass back the
C*    file names

      IF (IFCNT .EQ. 0) THEN
         COMDLN = .FALSE.
         INPFIL = ' '
         OUTFIL = ' '
      ELSE IF (IFCNT .EQ. 2) THEN
         COMDLN = .TRUE.
         INPFIL = COMLIN(LOCB(1):LOCE(1))
         OUTFIL = COMLIN(LOCB(2):LOCE(2))
      ELSE

C*       Error on Command Line.  Write Error Message and STOP
         COMDLN = .FALSE.
         WRITE(*,660)
         STOP

      END IF

660   FORMAT(' COMMAND ERROR: PCRAMMET [input_control output_for_model]'
     &    ,/,'                          __________ optional __________')

      RETURN
      END


      SUBROUTINE SETUP
C***********************************************************************
C*    SETUP Module of PCRAMMET Meteorological Pre-processor  
C*
C*    PURPOSE:    Obtains user inputs needed for the PCRAMMET run - either
C*                from the file specified on the command line or the screen.
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 13, 1995
C*
C*    INPUTS:     Command Line Logical Flag and Filenames if Applicable
C*
C*    OUTPUTS:    Various input parameters
C*
C*    CALLED FROM:  MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

C*    Open Program Diagnostics Output File and Write Info
      OPEN (IDIAG,FILE='PCRAM.LOG')
      WRITE (IDIAG,7000) VERSON
7000  FORMAT (30X,'********************',/,
c dgs&        30X,'      PCRAMMET      ',/,
     &        30X,'      CPRAMMET      ',/,
     &        30X,'   VERSION: ',A5     ,/,
     &        30X,'********************',/)

C*    If the User Specified Files on the Command Line,
C*    Then Open the Input File and Read Data from it.
      IF (COMDLN) THEN
         OPEN (INOPT,FILE=INPFIL,STATUS='OLD',ERR=1001)
         CALL READIN(INOPT)

      ELSE
C*       Otherwise, Read Get Inputs from Screen
         CALL READIN(-9)
      ENDIF
      GO TO 999

C*    Error Processing
1001  CALL ERRHDL('E','1001',0)

999   RETURN
      END


      SUBROUTINE READIN(INFROM)
C***********************************************************************
C*    READIN Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Reads user inputs needed for the PCRAMMET run - either
C*                from the file specified on the command line or the
C*                screen.
C*
C*    PROGRAMMER: Jayant Hardikar, Jim Paumier
C*                PES Inc.
C*
C*    DATE:       August 15, 1995
C*
C*    INPUTS:     Unit Number of File to Read From. A '-9' Means Read
C*                from Screen
C*
C*    OUTPUTS:    Various input parameters
C*
C*    CALLED FROM: SETUP
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      CHARACTER*40 ANSWER
      LOGICAL      XFILE

      IF (INFROM .EQ. -9) THEN

C*       Open the file to capture the user's responses
         OPEN (INEW,FILE='INPUTS',STATUS='unknown',ERR=1002)

C*       Read Inputs From Screen

C*----------------------------------------------------------------------
    1    WRITE (*,*) 'Will you be making any dry or wet ',
     &               'deposition calculations? '
         WRITE (*,*) 'None / Dry / Wet : '
         READ  (*,'(A)',ERR=1) ANSWER
         CALL LWRUPR (ANSWER)

         IF (ANSWER .EQ. 'DRY'  .OR.  ANSWER .EQ. 'D' )THEN
            DRYFLG = .TRUE.
            WETFLG = .FALSE.
         ELSE IF (ANSWER .EQ. 'WET'  .OR.  ANSWER .EQ. 'W' )THEN
            DRYFLG = .FALSE.
            WETFLG = .TRUE.
         ELSE IF (ANSWER .EQ. 'NONE'  .OR.  ANSWER .EQ. 'N' )THEN
            DRYFLG = .FALSE.
            WETFLG = .FALSE.
         ELSE
            WRITE (*,*) 'Invalid response, please answer appropriately'
            WRITE (*,*)
            GO TO 1
         ENDIF
         WRITE (INEW,110) ANSWER
  110    FORMAT( A,T41,' No/Dry/Wet Deposition calculations')

C*----------------------------------------------------------------------
    2    WRITE (*,*) 'Enter the OUTPUT filename: '
         READ  (*,'(A)',IOSTAT=IOS) OUTFIL
         IF( IOS .NE. 0 )THEN
            WRITE( *,* ) '  ERROR ON INPUT'
            GO TO 2
         ENDIF

         IF (OUTFIL(1:1) .EQ. ' ') THEN
            WRITE (*,*) 'Invalid response, please answer appropriately'
            WRITE (*,*)
            GO TO 2
         ENDIF

C*----------------------------------------------------------------------
C*       Force The Output File Type to be ASCII if DRY or WET
C*       Depletion is Desired
         IF (DRYFLG .OR. WETFLG) THEN
            OUTTYP = 'ASCII'
            WRITE (*,*) '    For Dry and Wet Deposition Calculations,'
            WRITE (*,*) '   the Output File Type Is Forced to be ASCII'
            WRITE (*,*)
         ELSE
3           WRITE (*,*)
            WRITE (*,*) 'Enter the output file type: '
            WRITE (*,*) 'Unform / Ascii : '
            READ  (*,'(A)',ERR=3) ANSWER
            CALL LWRUPR (ANSWER)
            IF (ANSWER(1:1) .EQ. 'U' ) THEN
               OUTTYP = 'UNFORM'
            ELSE
               OUTTYP = 'ASCII'
            ENDIF
            WRITE (INEW,310) ANSWER
  310       FORMAT( A, T41, ' Unform/Ascii output type')
         ENDIF

C*----------------------------------------------------------------------
    6    WRITE (*,*) 'Enter MIXING HEIGHT data filename: '
  600    READ  (*,'(A)',ERR=6) MIXNAM
         IF (MIXNAM(1:1) .EQ. ' ') THEN
            WRITE (*,*) 'Invalid response, please answer appropriately'
            WRITE (*,*)
            GO TO 6
         ENDIF
C*       Determine if the mixing height data file exists
         XFILE = .false.
         INQUIRE( FILE=MIXNAM, EXIST=XFILE )
         IF( .NOT. XFILE )THEN
            WRITE( *,* ) ' Mixing height data file does not exist -'
            WRITE( *,* ) '     re-enter the mixing height filename: '
            GO TO 600
         ENDIF
         WRITE (INEW,610) MIXNAM
  610    FORMAT( A, T41, ' Mixing height data file')

C*----------------------------------------------------------------------
    4    WRITE (*,*) 'Enter the HOURLY SURFACE DATA filename: '
  400    READ  (*,'(A)',ERR=4) SFCNAM
         IF (SFCNAM(1:1) .EQ. ' ') THEN
            WRITE (*,*) 'Invalid response, please answer appropriately'
            WRITE (*,*)
            GO TO 4
         ENDIF

C*       Determine if the input surface data file exists
         XFILE = .false.
         INQUIRE( FILE=SFCNAM, EXIST=XFILE )
         IF( .NOT. XFILE )THEN
            WRITE( *,* ) ' Surface data file does not exist -'
            WRITE( *,* ) '   re-enter the surface data filename: '
            GO TO 400
         ENDIF
         WRITE (INEW,410) SFCNAM
  410    FORMAT(A,T41,' Hourly surface data file')

C*----------------------------------------------------------------------
    5    WRITE (*,*) 'Enter surface data format: '
         WRITE (*,*) 'CD144 or SAMSON or HUSWO or SCRAM : '
         READ (*,'(A)',ERR=5) ANSWER
         CALL LWRUPR (ANSWER)

         IF (ANSWER .NE. 'CD144' .AND.  ANSWER .NE. 'SCRAM'  .AND.
     &       ANSWER .NE. 'SAMSON' .AND. ANSWER .NE. 'HUSWO') THEN
             WRITE (*,*) 'Invalid response, please answer appropriately'
             WRITE (*,*)
             GO TO 5
         ENDIF
         SFCTYP = ANSWER

         IF( SFCTYP .EQ. 'SCRAM'  .AND.  WETFLG )THEN
            CALL ERRHDL('E','1043',0)
         ENDIF

         WRITE (INEW,510) ANSWER
  510    FORMAT( A, T41, ' Surface data format')

C*----------------------------------------------------------------------
         IF( SFCTYP .EQ. 'CD144'  .OR.  SFCTYP .EQ. 'SCRAM'  .OR.
     &       SFCTYP .EQ. 'HUSWO' )THEN

            WRITE( *, 4000 )
 4000       FORMAT( ' For CD-144, SCRAM, and HUSWO data, additional',
     &              ' station information is needed'/)

   20       WRITE (*,2000)
 2000       FORMAT(3x, ' Enter the latitude of the surface station ',
     &                 'in decimal degrees',/,
     &         4x, '   - positive for stations north of the equator: ')

            READ (*,*,IOSTAT=IOS) XLAT

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 20
            ENDIF

            IF( ABS(XLAT) .GT. 90.0 )THEN
               WRITE( *,2015 )
               GO TO 20
            ENDIF

            WRITE (INEW,2010) XLAT
 2010       FORMAT( f10.3, T41, ' Station latitude (decimal degrees)' )
 2015       FORMAT('   INVALID VALUE: |Latitude| must be <= 90.0')

C*----------------------------------------------------------------------
   21       WRITE (*,2100)
 2100       FORMAT(3x, ' Enter the longitude of the surface station ',
     &                 'in decimal degrees',/,
     &           4x,'   - positive for stations WEST of Greenwich: ')
            READ (*,*,IOSTAT=IOS) XLON

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 21
            ENDIF

            IF( ABS(XLON) .GT. 180.0 )THEN
               WRITE( *,2106 )
               GO TO 21
            ENDIF

            WRITE (INEW, 2105) XLON
 2105       FORMAT( f10.3, T41, ' Station longitude (decimal degrees)' )
 2106       FORMAT('   INVALID VALUE: |Longitude| must be <= 180.0')

C*----------------------------------------------------------------------
  210       WRITE (*,2110)
 2110       FORMAT(3x, ' Enter the time zone of the surface station ',
     &         /,4x,'   - positive for stations WEST of Greenwich: ')
            READ (*,*,IOSTAT=IOS) ITZONE
            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 210
            ENDIF

            IF( ITZONE .GT. 12 )THEN
               WRITE( *,2120 )
               GO TO 210
            ENDIF

            WRITE (INEW,2115) ITZONE
 2115       FORMAT( I10, T41, ' Station time zone' )
 2120       FORMAT( '  INVALID VALUE: |Time Zone| must be <= 12')
         ENDIF

C*----------------------------------------------------------------------
         IF (WETFLG) THEN
            IF (SFCTYP .EQ. 'SAMSON' .OR. SFCTYP .EQ. 'HUSWO' ) THEN
7             WRITE (*,*)'SAMSON/HUSWO data may already contain hourly',
     &                   ' precip'
              WRITE (*,*)'Do you want to supplement with TD-3240 data? '
              WRITE (*,*) 'Yes / No : '
              READ (*,'(A)',ERR=7) ANSWER
              CALL LWRUPR (ANSWER)

              IF (ANSWER .EQ. 'YES'  .OR.  ANSWER .EQ. 'Y') THEN
                 PPTFLG = .TRUE.
                 WRITE (INEW,710) ANSWER
              ELSEIF( ANSWER .EQ. 'NO'  .OR.  ANSWER .EQ. 'N' )THEN
                 PPTFLG = .FALSE.
                 WRITE (INEW,710) ANSWER
                 GO TO 1000
              ELSE
                 WRITE (*,*) 'Invalid response, please answer',
     &                       ' appropriately'
                 WRITE (*,*)
                 GO TO 7
              ENDIF
            ELSE
                 PPTFLG = .TRUE.
            ENDIF
  710       FORMAT(A,T41,' Supplement SAMSON/HUSWO precip with TD3240?')

C*----------------------------------------------------------------------
    8       WRITE (*,*) 'Enter HOURLY PRECIP DATA filename: '
  800       READ  (*,'(A)',ERR=8) PPTNAM
            IF (PPTNAM(1:1) .EQ. ' ') THEN
               WRITE (*,*) 'Invalid response, please answer',
     &                     ' appropriately'
               WRITE (*,*)
               GO TO 8
            ENDIF
C*          Determine if the precipitation data file exists
            XFILE = .false.
            INQUIRE( FILE=PPTNAM, EXIST=XFILE )
            IF( .NOT. XFILE )THEN
               WRITE( *,* ) ' Precipitation data file does not exist -'
               WRITE( *,* ) '     re-enter the precipitation filename: '
               GO TO 800
            ENDIF
            WRITE (INEW,810) PPTNAM
  810       FORMAT( A,T41, ' Precipitation data file')

C*----------------------------------------------------------------------
    9       WRITE (*,*) 'Enter precipitation file format'
            WRITE (*,*) 'Variable or Fixed: '
            READ (*,'(A)',ERR=9) ANSWER
            CALL LWRUPR (ANSWER)

            IF ((ANSWER .NE. 'VARIABLE' .AND.  ANSWER .NE. 'V')  .AND.
     &          (ANSWER .NE. 'FIXED'    .AND.  ANSWER .NE. 'F' ))THEN
               WRITE (*,*) 'Invalid response, please answer',
     &                     ' appropriately'
               WRITE (*,*)
               GO TO 9
            ENDIF
            PPTFMT = ANSWER
            WRITE (INEW,910) ANSWER
  910       FORMAT( A,T41,' Precip data format: Variable / Fixed')
         ENDIF

C*----------------------------------------------------------------------
 1000    IF( DRYFLG  .OR.  WETFLG )THEN

C---------- Enter the site characteristics for deposition processes

            WRITE (*,4100)
 4100       FORMAT (' For deposition, site characteristics are needed'/)

C---------- Minimum Monin-Obukhov length

   22       WRITE (*,2200)
 2200       FORMAT(4x,'Enter the minimum Monin-Obukhov length ',
     &                'for stable conditions (meters)',/
     &          ,4x,'  2m for open land to 100-150m for city centers: ')
            READ (*,*,IOSTAT=IOS) ELMINM

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 22
            ENDIF

            IF( ELMINM .LE. 0.0 )THEN
               WRITE( *,2215 )
               GO TO 22
            ENDIF

            WRITE (INEW,2210) ELMINM
 2210       FORMAT(f10.3, T41, ' Min. Obukhov length (m)')
 2215       FORMAT('   INVALID VALUE: Minimum L for a stable',
     &             ' atmosphere must be > 0.0')

C---------- Minimum Monin-Obukhov length - application site: NOT ACTIVATED

C   42       WRITE (*,2250)
C 2250       FORMAT(4x,'Enter the minimum Monin-Obukhov length ',
C     &                'for stable conditions (meters)',/
C     &             4x,'AT THE APPLICATION SITE',
C     &         /,4x,'  2m for open land to 100-150m for city centers: ')
C            READ (*,*,IOSTAT=IOS) ELMINA
C            IF( IOS .NE. 0 )THEN
C               WRITE( *,* ) '  ERROR ON INPUT'
C               GO TO 42
C            ENDIF
C            WRITE (INEW,2260) ELMINA
C 2260       FORMAT(f10.3, t41, ' Min. Obukhov length (m) - application')

C---------- Anemometer height

   23       WRITE (*,2300)
 2300       FORMAT(4x, 'Enter the anemometer height (meters) ', /
     &           4x,'    (measurement height of the wind): ' )
            READ (*,*,IOSTAT=IOS) ANEMHT

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 23
            ENDIF

            IF( ANEMHT .LE. 0.0 )THEN
               WRITE( *,* ) '  INVALID VALUE: Height must be > 0.0'
               GO TO 23
            ENDIF

            WRITE (INEW,2310) ANEMHT
 2310       FORMAT(F10.3, T41, ' Anemometer height (m)')

C---------- Surface roughness length for the
C           site where the measurements were taken.

   29       WRITE (*,2900)
 2900       FORMAT(4x, 'Enter the surface roughness length (meters) ',
     &                 'AT THE ** MEASUREMENT SITE **',/,
     &             4x, '   0.0001 for open water to 1.3 for forests: ')
            READ (*,*,IOSTAT=IOS) Z0MEAS

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 29
            ENDIF

            IF( Z0MEAS .LE. 0.0 )THEN
               WRITE( *,* ) '  INVALID VALUE: Sfc. rough. must be > 0.0'
               GO TO 29
            ENDIF

            IF( ANEMHT .LE. Z0MEAS )THEN
               WRITE( *,2905 )
 2905          FORMAT('   WARNING: Roughness length must be < '
     &                'anemometer ht')
            ENDIF

            WRITE (INEW,2910) Z0MEAS
 2910       FORMAT( f10.4,t41,' Roughness length (m), measurement site')

C---------- For this version of PCRAMMET, set the displacement height
C           to 0.0 meters; the computations below will still 'use' the
C           displacement height
            ZDMEAS = 0.0

C  24       WRITE (*,2400)
C2400       FORMAT(4x,'Enter the displacement height (meters)', /,
C      &             4x,' at the measurement site',/,
C      &             4x,'   - must be less than the anemometer height: ')
C           READ (*,*,IOSTAT=IOS) ZDMEAS
C           IF( IOS .NE. 0 )THEN
C              WRITE( *,* ) 'ERROR ON INPUT'
C              GO TO 24
C           ENDIF
C           WRITE (INEW,2410) ZDMEAS
C2410       FORMAT(f10.3,t41,' Displacement ht (m), measurement site')

C           IF( ANEMHT .LE. ZDMEAS )THEN
C              CALL ERRHDL( 'E','1110',0 )
C           ENDIF

C---------- Surface roughness length and displacement height at the
C           site where the output data are to be applied.

   39       WRITE (*,3900)
 3900       FORMAT(4x, 'Enter the surface roughness length (meters) ',
     &                 'AT THE >> APPLICATION SITE <<',/,
     &             4x, '   0.0001 for open water to 1.3 for forests: ')
            READ (*,*,IOSTAT=IOS) Z0APPL

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 39
            ENDIF

            IF( Z0APPL .LE. 0.0 )THEN
               WRITE( *,* ) '  INVALID VALUE: Sfc. rough. must be > 0.0'
               GO TO 39
            ENDIF

            IF( ANEMHT .LE. Z0MEAS )THEN
               WRITE( *,2905 )
            ENDIF

            WRITE (INEW,3910) Z0APPL
 3910       FORMAT( f10.4,t41,' Roughness length (m), application site')

C34         WRITE (*,3400)
C3400       FORMAT(4x,'Enter the displacement height (meters)', /,
C     &            4x,' at the application site',/,
C     &            4x,'   - must be less than the anemometer height: ')
C           READ (*,*,IOSTAT=IOS) ZDAPPL
C           IF( IOS .NE. 0 )THEN
C              WRITE( *,* ) 'ERROR ON INPUT'
C              GO TO 24
C           ENDIF
C           WRITE (INEW,3410) ZDAPPL
C0          FORMAT(f10.3,t41,' Displacement ht (m), application site')
C
C           IF( ANEMHT .LE. ZDAPPL )THEN
C              CALL ERRHDL( 'E','1110',0 )
C           ENDIF

C---------- Noon-time albedo

   25       WRITE (*,2500)
 2500       FORMAT(4x, 'Enter the noon time albedo (as a fraction) ', /,
     &             4x, '       0.1 for water to 0.9 for fresh snow: ')
            READ (*,*,IOSTAT=IOS) ALBEDO

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 25
            ENDIF

            IF( ALBEDO .LT. 0.0  .OR.  ALBEDO .GT. 1.0 )THEN
               WRITE( *,2515 )
               GO TO 25
            ENDIF

            WRITE (INEW,2510) ALBEDO
 2510       FORMAT(f10.3,T41,' Noon time albedo')
 2515       FORMAT('   INVALID VALUE: Allowable values for albedo',
     &             ' are 0.0 to 1.0')
C---------- Bowen ratio

   26       WRITE (*,2600)
 2600       FORMAT(4x, 'Enter the Bowen ratio ', /,
     &           4x,'  0.1 for very moist to 10.0 for desert: ')
            READ (*,*,IOSTAT=IOS) BOWEN

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 26
            ENDIF

            IF( BOWEN .LT. 0.1  .OR.  BOWEN .GT. 10.0 )THEN
               WRITE( *,2615 )
            ENDIF

            WRITE (INEW,2610) BOWEN
 2610       FORMAT(f10.3,t41,' Bowen ratio')
 2615       FORMAT('   WARNING: Bowen ratio outside reasonable',
     &             ' range: 0.1 to 10.0'/)

C---------- Anthropogenic heat flux

   27       WRITE (*,2700)
 2700       FORMAT(4x,'Enter the anthopogenic heat flux (W/m^2)', /,
     &          4x,'  0.0 for rural to 100.0 for large cities: ')
            READ (*,*,IOSTAT=IOS) QF

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 27
            ENDIF

            IF( QF .LT. 0.0 )THEN
               WRITE( *,2715 )
            ENDIF

            WRITE (INEW,2710) QF
 2710       FORMAT( f10.3,t41,' Anthropogenic heat flux (W/m^2)')
 2715       FORMAT('   WARNING: Anthropogenic heat flux should be',
     &             ' greater than or equal to zero'/)

C---------- Fraction of net radiation absorbed by the ground

   28       WRITE (*,2800)
 2800       FORMAT(4x,'Enter the fraction of net radiation absorbed by',
     &             ' the ground' ,/,29x,
     &             ' 0.15 for rural to 0.27 for urban: ')
            READ (*,*,IOSTAT=IOS) CSUBG

            IF( IOS .NE. 0 )THEN
               WRITE( *,* ) '  ERROR ON INPUT'
               GO TO 28
            ENDIF

            IF( CSUBG .LT. 0.0  .OR.  CSUBG .GT. 1.0 )THEN
               WRITE( *,2815 )
               GO TO 28
            ENDIF

            WRITE (INEW,2810) CSUBG
 2810       FORMAT( f10.3,t41,' Fraction net rad''n absorbed by ground')
 2815       FORMAT('   INVALID VALUE: Allowable values for the'
     &             ' fraction are 0.0 to 1.0')

         ENDIF

C*       Close the New Inputs file
         CLOSE (INEW)

      ELSE
C*       Read the Inputs From the Response File

         READ  (INOPT,'(A)',ERR=1003) ANSWER
         CALL LWRUPR (ANSWER)

         IF (ANSWER .EQ. 'DRY'  .OR.  ANSWER .EQ. 'D' ) THEN
            DRYFLG = .TRUE.
            WETFLG = .FALSE.
         ELSE IF (ANSWER .EQ. 'WET'  .OR.  ANSWER .EQ. 'W') THEN
            DRYFLG = .FALSE.
            WETFLG = .TRUE.
         ELSE IF (ANSWER .EQ. 'NONE' .OR.  ANSWER .EQ. 'N') THEN
            DRYFLG = .FALSE.
            WETFLG = .FALSE.
         ELSE
            GO TO 1004
         ENDIF

         IF (DRYFLG .OR. WETFLG) THEN
            OUTTYP = 'ASCII'
         ELSE
            READ  (INOPT,'(A)',ERR=1006) ANSWER
            CALL LWRUPR (ANSWER)
            IF (ANSWER(1:1) .EQ. 'U') THEN
               OUTTYP = 'UNFORM'
            ELSE
               OUTTYP = 'ASCII'
            ENDIF
         ENDIF

         READ  (INOPT,'(A)',ERR=1009) MIXNAM

         READ  (INOPT,'(A)',ERR=1007) SFCNAM
         READ (INOPT,'(A)',ERR=1008) ANSWER
         CALL LWRUPR (ANSWER)
         SFCTYP = ANSWER
         IF( SFCTYP .EQ. 'SCRAM'  .AND.  WETFLG )THEN
            CALL ERRHDL('E','1043',0)
         ENDIF

         IF( SFCTYP .EQ. 'CD144'  .OR.  SFCTYP .EQ. 'SCRAM'  .OR.
     &       SFCTYP .EQ. 'HUSWO' )THEN
            READ (INOPT,*,ERR=1013) XLAT
            READ (INOPT,*,ERR=1014) XLON
            READ (INOPT,*,ERR=1101) ITZONE
         ENDIF

         IF (WETFLG) THEN
            IF (SFCTYP .EQ. 'SAMSON'  .OR.  SFCTYP .EQ. 'HUSWO') THEN
              READ (INOPT,'(A)',ERR=1010) ANSWER
              CALL LWRUPR (ANSWER)

              IF (ANSWER .EQ. 'YES'  .OR.  ANSWER .EQ. 'Y') THEN
                 PPTFLG = .TRUE.
              ELSE
                 PPTFLG = .FALSE.
                 GO TO 40
              ENDIF
            ELSE
                 PPTFLG = .TRUE.
            ENDIF

            READ  (INOPT,'(A)',ERR=1011) PPTNAM
            READ  (INOPT,'(A)',ERR=1012) ANSWER
            CALL LWRUPR (ANSWER)
            PPTFMT = ANSWER
         ENDIF

   40    IF( DRYFLG  .OR.  WETFLG )THEN
            READ (INOPT,*,ERR=1015) ELMINM
            READ (INOPT,*,ERR=1016) ANEMHT
            IF( ANEMHT .LE. 0.0 )THEN
               CALL ERRHDL('E', '1115', 0)
            ENDIF

            READ (INOPT,*,ERR=1022) Z0MEAS
            IF( Z0MEAS .LE. 0.0 )THEN
               CALL ERRHDL('E', '1110', 0)
            ENDIF

            READ (INOPT,*,ERR=1022) Z0APPL
            IF( Z0APPL .LE. 0.0 )THEN
               CALL ERRHDL('E', '1110', 0)
            ENDIF

            READ (INOPT,*,ERR=1018) ALBEDO
            IF( ALBEDO .LT. 0.0  .OR.  ALBEDO .GT. 1.0)THEN
               CALL ERRHDL('E', '1130', 0)
            ENDIF

            READ (INOPT,*,ERR=1019) BOWEN
            IF( BOWEN .LT. 0.1 .OR.  BOWEN .GT. 10.0 )THEN
               WRITE( IDIAG,2615 )
               WRITE( *,2615 )
            ENDIF

            READ (INOPT,*,ERR=1020) QF
            IF( QF .LT. 0.0 )THEN
               WRITE( IDIAG,2715 )
               WRITE( *,2715)
            ENDIF

            READ (INOPT,*,ERR=1021) CSUBG
            IF( CSUBG .LE. 0.0 )THEN
               CALL ERRHDL('E', '1135', 0)
            ENDIF
         ENDIF

      ENDIF

C*    Write some of the responses to the LOG file, PCRAM.LOG

      WRITE (IDIAG, 55) OUTFIL
  55  FORMAT ( ' The following data were used for output file: ',A40/ )
      IF( DRYFLG )THEN
         WRITE( IDIAG,155 )
      ELSEIF( WETFLG )THEN
         WRITE( IDIAG,156 )
      ELSE
         WRITE( IDIAG,157 )
      ENDIF
 155  FORMAT ( 'DRY',T41, ' Dry deposition calculations')
 156  FORMAT ( 'WET',T41, ' Wet deposition calculations')
 157  FORMAT ( 'NONE',T41, ' No deposition calculations')

      WRITE (IDIAG,610) MIXNAM
      WRITE (IDIAG,410) SFCNAM
      WRITE (IDIAG,510) SFCTYP

      IF( SFCTYP .EQ. 'CD144'  .OR.  SFCTYP .EQ. 'SCRAM'  .OR.
     &    SFCTYP .EQ. 'HUSWO' )THEN
         WRITE (IDIAG,2010) XLAT
         WRITE (IDIAG, 2105) XLON
         WRITE (IDIAG,2115) ITZONE
      ENDIF
      IF ( PPTFLG )THEN
         WRITE (IDIAG,810) PPTNAM
      ENDIF
      IF( DRYFLG .OR. WETFLG )THEN
         WRITE (IDIAG,2210) ELMINM
         WRITE (IDIAG,2310) ANEMHT
         WRITE (IDIAG,2910) Z0MEAS
         WRITE (IDIAG,3910) Z0APPL
         WRITE (IDIAG,2510) ALBEDO
         WRITE (IDIAG,2610) BOWEN
         WRITE (IDIAG,2710) QF
         WRITE (IDIAG,2810) CSUBG
      ENDIF
      WRITE( IDIAG, 3000 )
 3000 FORMAT( //,' The following messages were written by PCRAMMET:'/)



      GO TO 999

C*    Error Handling - the error handler stops PCRAMMET
1002  CALL ERRHDL('E','1002',0)
1003  CALL ERRHDL('E','1003',0)
1004  CALL ERRHDL('E','1004',0)
1006  CALL ERRHDL('E','1006',0)
1007  CALL ERRHDL('E','1007',0)
1008  CALL ERRHDL('E','1008',0)
1009  CALL ERRHDL('E','1009',0)
1010  CALL ERRHDL('E','1010',0)
1011  CALL ERRHDL('E','1011',0)
1012  CALL ERRHDL('E','1012',0)
1013  CALL ERRHDL('E','1013',0)
1014  CALL ERRHDL('E','1014',0)
1015  CALL ERRHDL('E','1015',0)
1016  CALL ERRHDL('E','1016',0)
1017  CALL ERRHDL('E','1017',0)
1018  CALL ERRHDL('E','1018',0)
1019  CALL ERRHDL('E','1019',0)
1020  CALL ERRHDL('E','1020',0)
1021  CALL ERRHDL('E','1021',0)
1022  CALL ERRHDL('E','1022',0)
1101  CALL ERRHDL('E','1101',0)

999   RETURN
      END


      SUBROUTINE FILES
C***********************************************************************
C*    FILES Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Checks to See If All the Required Files Are Present
C*                and Opens Them.
C*
C*    PROGRAMMER: Jayant Hardikar, Jim Paumier
C*                PES Inc.
C*
C*    DATE:       August 15, 1995
C*
C*    INPUTS:     Filenames
C*
C*    OUTPUTS:    ----
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      LOGICAL EXISTS
      CHARACTER ATEST*1, BTEST*3, SCTEST*80

C*    First the Surface File
      EXISTS = .FALSE.
      INQUIRE (FILE=SFCNAM,EXIST=EXISTS)
      IF (EXISTS) THEN
         OPEN (INSFC,FILE=SFCNAM,ERR=1023)
      ELSE
         GO TO 1023
      ENDIF

C*    The Mixing Height File
      EXISTS = .FALSE.
      INQUIRE (FILE=MIXNAM,EXIST=EXISTS)
      IF (EXISTS) THEN
         OPEN (INMIX,FILE=MIXNAM,ERR=1024)
      ELSE
         GO TO 1024
      ENDIF

C*    The Precip File - Open Only if Wet Deposition is Desired
C*    And Either the Surface Data is Not from Samson or When
C*    a TD3240 File Supplements the SAMSON Precip.

      IF (WETFLG)THEN
         IF (PPTFLG) THEN
            EXISTS = .FALSE.
            INQUIRE (FILE=PPTNAM,EXIST=EXISTS)
            IF (EXISTS) THEN
               OPEN (INPPT,FILE=PPTNAM,ERR=1025)

C*             IF THE TD3240 File Is A Variable Format
C*             File, Then First Rewrite All Precip Data To
C*             A Scratch Formatted File (VAR2FIX logic)
               IF (PPTFMT .EQ. 'VARIABLE'  .OR.  PPTFMT .EQ. 'V') THEN

C*                Check To See If The File Is REALLY A Variable FMT
                  READ (INPPT,'(T28,A3)',ERR=1026) BTEST
                  IF (BTEST .EQ. '   ') GO TO 1026
                  REWIND (INPPT)

C*****            OPEN (INPPT+50,file='azbycxdw', ERR=1027)
                  OPEN (INPPT+50,STATUS='SCRATCH', ERR=1027)
C                 write (*,*) 'pptfile opened - temporary'
                  CALL VAR2FIX(INPPT)
                  INPPT = INPPT + 50
                  REWIND (INPPT)

               ELSE
C*                Check To See If The File Is REALLY A Fixed FMT
                  READ (INPPT,'(T28,A3)',ERR=1028) BTEST
                  IF (BTEST .NE. '   ') GO TO 1028
                  REWIND (INPPT)

               ENDIF

            ELSE
               GO TO 1029
            ENDIF
         ENDIF
      ENDIF

C*    The Output File
      IF (OUTTYP .EQ. 'UNFORM') THEN
         OPEN (IOUT,FILE=OUTFIL,FORM='UNFORMATTED',
     &         ACCESS='SEQUENTIAL',ERR=1030)

      ELSE IF (OUTTYP .EQ. 'ASCII') THEN
         OPEN (IOUT,FILE=OUTFIL,ERR=1031)

      ENDIF

C*    Check to See If The File Are Of The Type That The User Says
C*    They Are; a difficulty may arise if the first record in the
C     CD-144 file has 28 characters or less - the program will
C     determine the file is not of type CD144 when, in fact, it is.

      IF (SFCTYP .EQ. 'CD144') THEN
         READ(INSFC, '(A80)', ERR=1032 ) SCTEST
         NCH = 80
         DO WHILE( NCH .GT. 0  .AND.  SCTEST(NCH:NCH) .EQ. ' ')
            NCH = NCH -1
         ENDDO

         IF( NCH .LE. 28 )THEN
C---------- Likely not a CD144 file; records should be > 28 characters
            GO TO 1032
         ENDIF


      ELSEIF( SFCTYP .EQ. 'SCRAM' )THEN
         READ(INSFC, '(A80)', ERR=1034 ) SCTEST
         NCH = 80
         DO WHILE( NCH .GT. 0  .AND.  SCTEST(NCH:NCH) .EQ. ' ')
            NCH = NCH -1
         ENDDO

         IF( NCH .GT. 28 )THEN
C---------- Not a SCRAM file, which should have <= 28-character records
            GO TO 1034
         ENDIF

      ELSE IF (SFCTYP .EQ. 'SAMSON') THEN
         READ (INSFC,'(A1)',ERR=1033)ATEST
         IF (ATEST .NE. '~') THEN
C---------- Not a SAMSON file
            GO TO 1033
         ENDIF

      ELSE IF (SFCTYP .EQ. 'HUSWO') THEN
         READ (INSFC,'(A80)',ERR=1033)SCTEST
         IF (INDEX(SCTEST,'1     2') .EQ. 0 ) THEN
C---------- Not likely a HUSWO file
            GO TO 1044
         ENDIF

      ENDIF

      REWIND (INSFC)
      GO TO 999

1023  CALL ERRHDL('E','1023',0)
1024  CALL ERRHDL('E','1024',0)
1025  CALL ERRHDL('E','1025',0)
1026  CALL ERRHDL('E','1026',0)
1027  CALL ERRHDL('E','1027',0)
1028  CALL ERRHDL('E','1028',0)
1029  CALL ERRHDL('E','1029',0)
1030  CALL ERRHDL('E','1030',0)
1031  CALL ERRHDL('E','1031',0)
1032  CALL ERRHDL('E','1032',0)
1033  CALL ERRHDL('E','1033',0)
1034  CALL ERRHDL('E','1034',0)
1044  CALL ERRHDL('E','1044',0)

999   RETURN
      END


      SUBROUTINE VARINI
C***********************************************************************
C*    VARINI Module of the PCRAMMET Meteorological Preprocessor
C*
C*    PURPOSE:    Initializes the input and output data
C*
C*    PROGRAMMER: J. Paumier
C*                PES Inc.
C*
C*    DATE:       April 28, 1995
C*
C*    INPUTS:     None
C*
C*    OUTPUTS:    Initialized data
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

      DO 100 IH = 1,24
         KST(IH)    = 9
         ICOVER(IH) = 99
         IDIR(IH)   = 999
         ICEIL(IH)  = 9999
         IGRAD(IH)  = 9999
         IRHUM(IH) = -99

         RSOL(IH)   = 0.0
         SHF(IH)    = 0.0
         FCOVER(IH) = 99.0
         WSPEED(IH) = 99.0
         AFV(IH)    = 999.0
         FVR(IH)    = 999.0
         HLH(1,IH)  = 9999.0
         HLH(2,IH)  = 9999.0
         TEMP(IH)   = 9999.0
         PRESS(IH)  = 9999.0
         PRECIP(IH) = 9999.0
         P3240(IH)  = 9999.0

         USTAR(IH)  = -9.0
         XMONIN(IH) = -999.0

  100 CONTINUE

      RETURN
      END


      SUBROUTINE WRITIT(JDAY)
C***********************************************************************
C*    USTARL Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    This Routine Writes Out 24 Hours Worth of Data
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:     Atmospheric boundary layer variables for ISCST-type
C*                models
C*
C*    OUTPUTS:    ---
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
C*    (IWRI is Used to Check If This Routine is Being Called
C*    for the First Time or Not)
      INTEGER ISAVWX, ISAVMIX
      SAVE IWRI
      INCLUDE 'PCRAM.INC'

C*    Save the year to another variable so it can be converted to a 
C     4-digit year

      ISAVWX = ISYR(1)
      ISAVMIX = IYRMIX(2)

      IF (OUTTYP .EQ. 'ASCII') THEN

C*       Write Header Info the First Time
C*       Recall that the second record of mixing height data is for
C*       the current day and the first data record is for the
C*       previous day, so write the year corresponding to the second
C*       data record.
         IF (IWRI .EQ. 0) THEN
C           Convert the years in the header record from 2-digit to 4-digit

            IF( ISAVWX .GE. 0  .AND.  ISAVWX .LE. 49 ) THEN
               ISAVWX = 2000 + ISAVWX
            ELSEIF( ISAVWX .GE. 50  .AND.  ISAVWX .LE. 99 ) THEN
               ISAVWX = 1900 + ISAVWX
            ENDIF

            IF( ISAVMIX .GE. 0  .AND.  ISAVMIX .LE. 49 ) THEN
               ISAVMIX = 2000 + ISAVMIX
            ELSEIF( ISAVMIX .GE. 50  .AND.  ISAVMIX .LE. 99 ) THEN
               ISAVMIX = 1900 + ISAVMIX
            ENDIF

            WRITE(IOUT,200) IDSFC, ISAVWX, IDMIX(1), ISAVMIX
            IWRI = IWRI + 1
         ENDIF

C*       Loop Over 24 Hours
         DO 20 I = 1,24

c -dgs-     Add section to recompute solar radiation as a real
            if(igrad(i).EQ.9999) then
c ---          Replace missing value with estimate
               call RADSOL(i)
               qswrad=AMAX1(0.0,rsol(i))
            else
c ---          Use reported value as real watts/m^2
               qswrad=FLOAT(igrad(i))
            endif

C*          If the Hour is Calm, then set the direction to 0 and the
C*          speed to 0.0 ( for ASCII output only); this is the method
C           used in ISC2 to identify calm winds in an ASCII file.
            IF (CALM(I)) THEN
               WSPEED(I) = 0.0
            ENDIF

C*          Write Hourly Met Data
            IF( WETFLG )THEN

c dgs          Use qswrad (either measured or estimated from clouds)
c dgs          and include in output for all data file formats
               IF( SFCTYP .EQ. 'SAMSON'  .OR. SFCTYP .EQ. 'HUSWO' )THEN
                  WRITE(IOUT,250) ISYR(I),ISMO(I),ISDY(I),ISHR(I),
     &                            FVR(I),WSPEED(I),TEMP(I),KST(I),
     &                            HLH(1,I),HLH(2,I),USTAR(I),XMONIN(I),
     &                            Z0APPL,IPCODE(I),PRECIP(I), 
     &                            qswrad, IRHUM(I)
               ELSE
                  WRITE(IOUT,250) ISYR(I),ISMO(I),ISDY(I),ISHR(I),
     &                            FVR(I),WSPEED(I),TEMP(I),KST(I),
     &                            HLH(1,I),HLH(2,I),USTAR(I),XMONIN(I),
     &                            Z0APPL,IPCODE(I),PRECIP(I),
     &                            qswrad, IRHUM(I)
               ENDIF

            ELSEIF( DRYFLG )THEN
c dgs          Use qswrad (either measured or estimated from clouds)
c dgs          and include in output for all data file formats
               IF( SFCTYP .EQ. 'SAMSON'  .OR. SFCTYP .EQ. 'HUSWO' )THEN
                  WRITE(IOUT,255) ISYR(I),ISMO(I),ISDY(I),ISHR(I),
     &                            FVR(I),WSPEED(I),TEMP(I),KST(I),
     &                            HLH(1,I),HLH(2,I),USTAR(I),XMONIN(I),
     &                            Z0APPL, qswrad, IRHUM(I)
               ELSE
                  WRITE(IOUT,255) ISYR(I),ISMO(I),ISDY(I),ISHR(I),
     &                            FVR(I),WSPEED(I),TEMP(I),KST(I),
     &                            HLH(1,I),HLH(2,I),USTAR(I),XMONIN(I),
     &                            Z0APPL, qswrad, IRHUM(I)
               ENDIF

            ELSE
               WRITE(IOUT,250) ISYR(I),ISMO(I),ISDY(I),ISHR(I),FVR(I),
     &                      WSPEED(I),TEMP(I),KST(I),HLH(1,I),HLH(2,I)
            ENDIF

20       CONTINUE

      ELSE IF (OUTTYP .EQ. 'UNFORM') THEN

C*       Write Header Info the First Time
         IF (IWRI .EQ. 0) THEN
C           Convert the year in the header record from 2-digit to 4-digit

            IF( ISAVWX .GE. 0  .AND.  ISAVWX .LE. 49 ) THEN
               ISAVWX = 2000 + ISAVWX
            ELSEIF( ISAVWX .GE. 50  .AND.  ISAVWX .LE. 99 ) THEN
               ISAVWX = 1900 + ISAVWX
            ENDIF

            IF( ISAVMIX .GE. 0  .AND.  ISAVMIX .LE. 49 ) THEN
               ISAVMIX = 2000 + ISAVMIX
            ELSEIF( ISAVMIX .GE. 50  .AND.  ISAVMIX .LE. 99 ) THEN
               ISAVMIX = 1900 + ISAVMIX
            ENDIF

            WRITE(IOUT) IDSFC, ISAVWX, IDMIX(1), ISAVMIX
            IWRI = IWRI + 1
         ENDIF


         WRITE (IOUT) ISYR(1),ISMO(1),FLOAT(JDAY),KST,WSPEED,TEMP,
     &                AFV,FVR,HLH

      ENDIF

C     NOTE: The colons in the format statement below is an edit descriptor 
C           that tells the program to terminate format control when the
C           output list associated with the WRITE statement is exhausted.

200   FORMAT(4(I6,1X))
250   FORMAT(I2.2, 3I2,2F9.4,F6.1,I2,2F7.1:,F9.4,F10.1,F8.4:,I4,F7.2:,
     &       T102,F9.0,T111,I3)
255   FORMAT(I2.2, 3I2,2F9.4,F6.1,I2,2F7.1,F9.4,F10.1,F8.4:,
     &       T102,F9.0,T111,I3)

      RETURN
      END


      BLOCK DATA
C***********************************************************************
C*    BLOCK Block Data for the PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Initializes Block Data
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 13, 1995
C*
C*    INPUTS:
C*
C*
C*    OUTPUTS:
C*
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

      INTEGER IRAND,KRAND(8784)
      COMMON /BDATA/ IRAND(24,366)
      EQUIVALENCE (IRAND(1,1), KRAND(1))

      DATA CONST /57.29578/                                                                                                             MET40800
      DATA INSFC/11/, INOPT/12/, INMIX/13/, INPPT/14/, IOUT/15/
      DATA INEW /77/, IDIAG/88/
      DATA NOWARN /.TRUE./
c dgs DATA VERSON/'99169'/
      DATA VERSON/'00326'/

      DATA (KRAND(I),I=1,456)
     1/5,2,8,7,7,6,9,7,1,5,8,0,7,3,6,8,5,1,8,1,4,6,4,4,
     20,6,6,4,0,1,3,0,4,5,9,5,3,1,4,8,2,5,6,9,8,9,1,2,
     34,0,8,4,1,3,0,3,2,8,6,2,5,6,1,4,3,5,1,2,1,3,5,1,
     40,0,2,3,5,6,0,2,0,1,1,7,9,6,5,4,7,3,1,3,1,3,6,2,
     59,8,1,4,6,9,6,9,3,2,2,3,2,4,9,2,3,0,1,1,6,4,2,6,
     60,7,9,7,6,0,2,6,2,2,9,0,5,9,6,6,6,9,2,5,6,0,6,0,
     72,7,4,9,7,0,3,8,4,8,7,6,5,0,1,9,8,8,5,6,2,6,5,9,
     83,5,2,9,1,9,9,5,4,4,3,1,3,8,6,3,2,4,7,6,7,2,4,8,
     99,0,0,8,0,3,7,8,4,8,5,1,8,9,4,7,5,8,0,5,3,4,8,9,
     15,7,4,9,2,9,8,8,0,6,5,7,9,0,0,2,2,1,7,3,9,6,2,5,
     24,8,7,8,7,8,6,5,6,9,1,4,9,8,1,5,7,0,3,8,0,0,5,3,
     32,3,8,0,1,2,2,6,7,5,2,3,2,2,5,3,0,3,6,9,0,1,9,4,
     43,2,7,3,8,4,1,7,5,7,6,5,4,1,5,0,1,8,4,0,2,5,4,2,
     58,7,2,4,6,0,6,8,4,4,2,1,7,5,1,0,6,0,9,7,0,1,1,4,
     64,6,2,4,9,4,3,6,1,1,6,4,8,4,7,0,7,1,5,6,8,7,3,4,
     70,3,1,3,8,6,6,2,6,1,5,6,6,4,5,9,0,5,3,4,0,0,3,6,
     81,2,3,8,7,1,4,5,8,6,0,9,0,2,8,6,2,3,3,1,2,2,5,3,
     94,3,3,7,9,6,3,2,7,6,1,1,5,9,5,9,3,3,9,4,3,5,7,0,
     11,1,9,0,7,2,6,6,1,5,8,3,4,4,5,7,8,2,4,8,1,5,2,0/
      DATA (KRAND(I),I=457, 912)
     */3,7,1,8,1,7,9,2,4,9,2,0,7,4,6,7,5,3,1,9,3,8,3,2,
     *3,4,3,4,1,5,3,1,9,4,6,9,3,0,4,2,2,8,0,3,0,1,2,3,
     *6,5,4,8,9,6,1,6,2,8,2,7,1,1,9,8,3,0,9,1,4,9,5,7,
     *4,1,4,4,3,5,3,1,9,3,0,8,4,3,4,7,6,9,8,5,1,1,1,4,
     *0,3,1,2,3,6,1,7,2,8,1,2,3,1,7,2,2,6,6,6,9,9,7,3,
     *4,6,8,9,5,9,0,5,8,6,5,2,7,6,6,4,0,5,5,4,2,4,1,4,
     *6,9,2,2,8,0,7,0,7,8,6,0,0,6,0,4,9,5,8,2,0,0,8,2,
     *5,6,5,5,2,5,1,4,5,6,7,4,0,1,8,0,2,5,0,0,2,5,0,0,
     *1,4,1,8,6,1,0,3,3,8,8,0,2,9,5,6,5,2,2,4,1,5,2,2,
     *1,6,9,5,2,8,7,8,7,3,7,4,7,5,9,8,4,1,7,1,3,6,8,2,
     *9,6,1,7,4,2,6,9,1,6,6,3,8,4,2,7,6,5,8,7,2,5,5,2,
     *8,2,6,5,7,1,9,6,0,8,2,0,1,8,3,0,2,3,0,3,5,2,0,7,
     *7,2,1,3,1,0,3,0,4,8,0,0,8,3,0,0,2,6,0,3,9,6,5,6,
     *6,9,7,9,3,7,0,0,9,0,5,0,7,0,6,4,2,8,2,1,7,4,3,7,
     *5,9,0,2,0,0,4,6,8,7,9,4,2,2,6,1,1,9,0,6,7,2,7,9,
     *1,1,0,9,6,2,1,3,8,0,9,7,7,1,9,6,0,6,9,8,0,8,6,2,
     *0,3,6,6,3,0,3,2,4,9,6,6,9,0,8,9,7,1,9,6,3,6,8,1,
     *2,7,4,0,2,1,8,3,8,2,2,0,8,3,9,6,6,6,2,5,3,6,1,1,
     *2,9,8,2,5,4,6,8,8,7,2,5,6,4,5,5,8,7,2,9,4,9,7,8/
      DATA (KRAND(I),I=913, 1368)
     */4,5,9,5,4,6,5,9,5,8,0,7,8,6,6,7,2,3,9,1,7,3,0,6,
     *3,8,0,1,5,2,0,1,4,5,5,8,3,0,1,2,6,1,1,3,0,9,9,5,
     *9,3,4,5,0,9,5,3,1,4,5,6,8,3,7,9,2,8,4,1,9,8,1,5,
     *0,7,8,6,1,0,7,2,7,9,5,5,2,6,9,9,8,1,5,7,5,0,0,8,
     *3,3,8,5,3,6,7,6,0,6,9,1,2,5,2,8,2,6,9,4,6,9,2,9,
     *8,9,3,1,8,6,1,4,3,5,8,5,3,1,4,3,4,9,8,0,3,3,2,0,
     *4,6,0,0,2,9,4,8,7,7,2,8,6,1,9,7,7,0,3,1,5,2,2,7,
     *7,6,6,4,2,1,5,8,0,2,0,9,2,6,2,0,8,0,0,9,5,8,4,8,
     *9,5,6,8,5,3,9,4,2,8,9,6,7,9,0,4,6,3,6,8,8,3,5,8,
     *0,6,5,8,5,1,9,5,2,5,9,8,2,8,8,6,3,3,8,2,4,6,7,0,
     *7,7,7,8,1,4,4,8,2,2,3,1,0,1,5,0,3,7,4,8,2,7,6,9,
     *3,3,2,0,3,2,4,1,8,5,4,7,6,6,1,1,8,1,6,4,2,3,1,2,
     *2,7,8,5,2,7,6,9,6,1,7,6,2,4,8,0,2,1,9,5,9,0,0,4,
     *4,8,2,2,8,2,8,0,8,0,6,8,5,7,2,2,2,5,6,8,3,7,6,0,
     *0,4,4,0,8,4,9,4,8,1,8,0,3,8,8,6,7,5,2,0,0,3,9,1,
     *0,2,4,3,5,7,9,5,6,5,3,0,0,9,0,1,0,0,9,6,6,4,2,7,
     *7,6,6,5,8,1,0,3,5,6,4,3,5,9,9,7,6,8,8,8,5,9,1,1,
     *8,9,2,1,9,7,6,7,6,0,7,0,0,4,9,5,2,5,2,1,4,0,8,8,
     *5,2,4,6,9,4,7,6,3,5,4,4,2,5,0,6,1,8,7,8,9,5,9,8/
      DATA (KRAND(I),I=1369, 1824)
     */2,1,7,6,9,8,7,7,9,3,5,3,2,9,9,9,0,5,2,3,0,9,2,1,
     *3,3,0,1,4,3,0,4,1,0,3,3,4,9,6,3,6,7,1,4,3,9,6,9,
     *3,3,2,1,8,2,9,6,2,8,8,1,9,9,8,9,5,9,3,3,3,3,7,4,
     *9,3,3,4,4,9,0,9,7,1,1,2,5,0,0,2,4,0,7,2,9,8,4,3,
     *3,2,3,7,9,8,3,2,7,8,1,9,0,0,7,7,6,3,7,0,6,2,5,9,
     *4,9,7,0,2,0,1,3,1,8,4,0,7,2,7,3,8,8,7,2,3,4,3,1,
     *4,4,0,1,4,1,9,5,0,8,3,5,9,8,4,5,3,7,3,8,0,2,8,1,
     *0,4,4,1,4,5,4,9,2,5,2,4,6,1,5,3,5,3,0,9,1,4,6,4,
     *8,6,5,0,1,5,0,5,0,2,0,7,3,6,2,4,6,6,5,3,2,4,9,4,
     *6,2,2,0,6,8,2,4,4,8,5,4,7,9,5,6,3,8,8,4,5,0,1,8,
     *3,7,2,6,4,6,9,4,6,0,9,0,6,8,4,8,9,9,0,0,2,1,1,2,
     *3,4,3,4,1,5,3,1,9,4,6,9,3,0,4,2,2,8,0,3,0,1,2,3,
     *6,5,4,8,9,6,1,6,2,8,2,7,1,1,9,8,3,0,9,1,4,9,5,7,
     *4,1,4,4,3,5,3,1,9,3,0,8,4,3,4,7,6,9,8,5,1,1,1,4,
     *0,3,1,2,3,6,1,7,2,8,1,2,3,1,7,2,2,6,6,6,9,9,7,3,
     *4,6,8,9,5,9,0,5,8,6,5,2,7,6,6,4,0,5,5,4,2,4,1,4,
     *6,9,2,2,8,0,7,0,7,8,6,0,0,6,0,4,9,5,8,2,0,0,8,2,
     *5,6,5,5,2,5,1,4,5,6,7,4,0,1,8,0,2,5,0,0,2,5,0,0,
     *1,4,1,8,6,1,0,3,3,8,8,0,2,9,5,6,5,2,2,4,1,5,2,2/
      DATA (KRAND(I),I=1825, 2280)
     */1,6,9,5,2,8,7,8,7,3,7,4,7,5,9,8,4,1,7,1,3,6,8,2,
     *9,6,1,7,4,2,6,9,1,6,6,3,8,4,2,7,6,5,8,7,2,5,5,2,
     *8,2,6,5,7,1,9,6,0,8,2,0,1,8,3,0,2,3,0,3,5,2,0,7,
     *7,2,1,3,1,0,3,0,4,8,0,0,8,3,0,0,2,6,0,3,9,6,5,6,
     *6,9,7,9,3,7,0,0,9,0,5,0,7,0,6,4,2,8,2,1,7,4,3,7,
     *5,9,0,2,0,0,4,6,8,7,9,4,2,2,6,1,1,9,0,6,7,2,7,9,
     *1,1,0,9,6,2,1,3,8,0,9,7,7,1,9,6,0,6,9,8,0,8,6,2,
     *0,3,6,6,3,0,3,2,4,9,6,6,9,0,8,9,7,1,9,6,3,6,8,1,
     *2,7,4,0,2,1,8,3,8,2,2,0,8,3,9,6,6,6,2,5,3,6,1,1,
     *2,9,8,2,5,4,6,8,8,7,2,5,6,4,5,5,8,7,2,9,4,9,7,8,
     *4,5,9,5,4,6,5,9,5,8,0,7,8,6,6,7,2,3,9,1,7,3,0,6,
     *3,8,0,1,5,2,0,1,4,5,5,8,3,0,1,2,6,1,1,3,0,9,9,5,
     *9,3,4,5,0,9,5,3,1,4,5,6,8,3,7,9,2,8,4,1,9,8,1,5,
     *0,7,8,6,1,0,7,2,7,9,5,5,2,6,9,9,8,1,5,7,5,0,0,8,
     *3,3,8,5,3,6,7,6,0,6,9,1,2,5,2,8,2,6,9,4,6,9,2,9,
     *8,9,3,1,8,6,1,4,3,5,8,5,3,1,4,3,4,9,8,0,3,3,2,0,
     *4,6,0,0,2,9,4,8,7,7,2,8,6,1,9,7,7,0,3,1,5,2,2,7,
     *7,6,6,4,2,1,5,8,0,2,0,9,2,6,2,0,8,0,0,9,5,8,4,8,
     *9,5,6,8,5,3,9,4,2,8,9,6,7,9,0,4,6,3,6,8,8,3,5,8/
      DATA (KRAND(I),I=2281, 2736)
     */0,6,5,8,5,1,9,5,2,5,9,8,2,8,8,6,3,3,8,2,4,6,7,0,
     *7,7,7,8,1,4,4,8,2,2,3,1,0,1,5,0,3,7,4,8,2,7,6,9,
     *3,3,2,0,3,2,4,1,8,5,4,7,6,6,1,1,8,1,6,4,2,3,1,2,
     *2,7,8,5,2,7,6,9,6,1,7,6,2,4,8,0,2,1,9,5,9,0,0,4,
     *4,8,2,2,8,2,8,0,8,0,6,8,5,7,2,2,2,5,6,8,3,7,6,0,
     *0,4,4,0,8,4,9,4,8,1,8,0,3,8,8,6,7,5,2,0,0,3,9,1,
     *0,2,4,3,5,7,9,5,6,5,3,0,0,9,0,1,0,0,9,6,6,4,2,7,
     *7,6,6,5,8,1,0,3,5,6,4,3,5,9,9,7,6,8,8,8,5,9,1,1,
     *8,9,2,1,9,7,6,7,6,0,7,0,0,4,9,5,2,5,2,1,4,0,8,8,
     *5,2,4,6,9,4,7,6,3,5,4,4,2,5,0,6,1,8,7,8,9,5,9,8,
     *2,1,7,6,9,8,7,7,9,3,5,3,2,9,9,9,0,5,2,3,0,9,2,1,
     *3,3,0,1,4,3,0,4,1,0,3,3,4,9,6,3,6,7,1,4,3,9,6,9,
     *3,3,2,1,8,2,9,6,2,8,8,1,9,9,8,9,5,9,3,3,3,3,7,4,
     *9,3,3,4,4,9,0,9,7,1,1,2,5,0,0,2,4,0,7,2,9,8,4,3,
     *3,2,3,7,9,8,3,2,7,8,1,9,0,0,7,7,6,3,7,0,6,2,5,9,
     *4,9,7,0,2,0,1,3,1,8,4,0,7,2,7,3,8,8,7,2,3,4,3,1,
     *4,4,0,1,4,1,9,5,0,8,3,5,9,8,4,5,3,7,3,8,0,2,8,1,
     *0,4,4,1,4,5,4,9,2,5,2,4,6,1,5,3,5,3,0,9,1,4,6,4,
     *8,6,5,0,1,5,0,5,0,2,0,7,3,6,2,4,6,6,5,3,2,4,9,4/
      DATA (KRAND(I),I=2737, 3192)
     */6,2,2,0,6,8,2,4,4,8,5,4,7,9,5,6,3,8,8,4,5,0,1,8,
     *3,7,2,6,4,6,9,4,6,0,9,0,6,8,4,8,9,9,0,0,2,1,1,2,
     *3,4,3,4,1,5,3,1,9,4,6,9,3,0,4,2,2,8,0,3,0,1,2,3,
     *6,5,4,8,9,6,1,6,2,8,2,7,1,1,9,8,3,0,9,1,4,9,5,7,
     *4,1,4,4,3,5,3,1,9,3,0,8,4,3,4,7,6,9,8,5,1,1,1,4,
     *0,3,1,2,3,6,1,7,2,8,1,2,3,1,7,2,2,6,6,6,9,9,7,3,
     *4,6,8,9,5,9,0,5,8,6,5,2,7,6,6,4,0,5,5,4,2,4,1,4,
     *6,9,2,2,8,0,7,0,7,8,6,0,0,6,0,4,9,5,8,2,0,0,8,2,
     *5,6,5,5,2,5,1,4,5,6,7,4,0,1,8,0,2,5,0,0,2,5,0,0,
     *1,4,1,8,6,1,0,3,3,8,8,0,2,9,5,6,5,2,2,4,1,5,2,2,
     *1,6,9,5,2,8,7,8,7,3,7,4,7,5,9,8,4,1,7,1,3,6,8,2,
     *9,6,1,7,4,2,6,9,1,6,6,3,8,4,2,7,6,5,8,7,2,5,5,2,
     *8,2,6,5,7,1,9,6,0,8,2,0,1,8,3,0,2,3,0,3,5,2,0,7,
     *7,2,1,3,1,0,3,0,4,8,0,0,8,3,0,0,2,6,0,3,9,6,5,6,
     *6,9,7,9,3,7,0,0,9,0,5,0,7,0,6,4,2,8,2,1,7,4,3,7,
     *5,9,0,2,0,0,4,6,8,7,9,4,2,2,6,1,1,9,0,6,7,2,7,9,
     *1,1,0,9,6,2,1,3,8,0,9,7,7,1,9,6,0,6,9,8,0,8,6,2,
     *0,3,6,6,3,0,3,2,4,9,6,6,9,0,8,9,7,1,9,6,3,6,8,1,
     *2,7,4,0,2,1,8,3,8,2,2,0,8,3,9,6,6,6,2,5,3,6,1,1/
      DATA (KRAND(I),I=3193, 3648)
     */2,9,8,2,5,4,6,8,8,7,2,5,6,4,5,5,8,7,2,9,4,9,7,8,
     *4,5,9,5,4,6,5,9,5,8,0,7,8,6,6,7,2,3,9,1,7,3,0,6,
     *3,8,0,1,5,2,0,1,4,5,5,8,3,0,1,2,6,1,1,3,0,9,9,5,
     *9,3,4,5,0,9,5,3,1,4,5,6,8,3,7,9,2,8,4,1,9,8,1,5,
     *0,7,8,6,1,0,7,2,7,9,5,5,2,6,9,9,8,1,5,7,5,0,0,8,
     *3,3,8,5,3,6,7,6,0,6,9,1,2,5,2,8,2,6,9,4,6,9,2,9,
     *8,9,3,1,8,6,1,4,3,5,8,5,3,1,4,3,4,9,8,0,3,3,2,0,
     *4,6,0,0,2,9,4,8,7,7,2,8,6,1,9,7,7,0,3,1,5,2,2,7,
     *7,6,6,4,2,1,5,8,0,2,0,9,2,6,2,0,8,0,0,9,5,8,4,8,
     *9,5,6,8,5,3,9,4,2,8,9,6,7,9,0,4,6,3,6,8,8,3,5,8,
     *0,6,5,8,5,1,9,5,2,5,9,8,2,8,8,6,3,3,8,2,4,6,7,0,
     *7,7,7,8,1,4,4,8,2,2,3,1,0,1,5,0,3,7,4,8,2,7,6,9,
     *3,3,2,0,3,2,4,1,8,5,4,7,6,6,1,1,8,1,6,4,2,3,1,2,
     *2,7,8,5,2,7,6,9,6,1,7,6,2,4,8,0,2,1,9,5,9,0,0,4,
     *4,8,2,2,8,2,8,0,8,0,6,8,5,7,2,2,2,5,6,8,3,7,6,0,
     *0,4,4,0,8,4,9,4,8,1,8,0,3,8,8,6,7,5,2,0,0,3,9,1,
     *0,2,4,3,5,7,9,5,6,5,3,0,0,9,0,1,0,0,9,6,6,4,2,7,
     *7,6,6,5,8,1,0,3,5,6,4,3,5,9,9,7,6,8,8,8,5,9,1,1,
     *8,9,2,1,9,7,6,7,6,0,7,0,0,4,9,5,2,5,2,1,4,0,8,8/
      DATA (KRAND(I),I=3649, 4104)
     */5,2,4,6,9,4,7,6,3,5,4,4,2,5,0,6,1,8,7,8,9,5,9,8,
     *2,1,7,6,9,8,7,7,9,3,5,3,2,9,9,9,0,5,2,3,0,9,2,1,
     *3,3,0,1,4,3,0,4,1,0,3,3,4,9,6,3,6,7,1,4,3,9,6,9,
     *3,3,2,1,8,2,9,6,2,8,8,1,9,9,8,9,5,9,3,3,3,3,7,4,
     *9,3,3,4,4,9,0,9,7,1,1,2,5,0,0,2,4,0,7,2,9,8,4,3,
     *3,2,3,7,9,8,3,2,7,8,1,9,0,0,7,7,6,3,7,0,6,2,5,9,
     *4,9,7,0,2,0,1,3,1,8,4,0,7,2,7,3,8,8,7,2,3,4,3,1,
     *4,4,0,1,4,1,9,5,0,8,3,5,9,8,4,5,3,7,3,8,0,2,8,1,
     *0,4,4,1,4,5,4,9,2,5,2,4,6,1,5,3,5,3,0,9,1,4,6,4,
     *8,6,5,0,1,5,0,5,0,2,0,7,3,6,2,4,6,6,5,3,2,4,9,4,
     *6,2,2,0,6,8,2,4,4,8,5,4,7,9,5,6,3,8,8,4,5,0,1,8,
     *3,7,2,6,4,6,9,4,6,0,9,0,6,8,4,8,9,9,0,0,2,1,1,2,
     *3,4,3,4,1,5,3,1,9,4,6,9,3,0,4,2,2,8,0,3,0,1,2,3,
     *6,5,4,8,9,6,1,6,2,8,2,7,1,1,9,8,3,0,9,1,4,9,5,7,
     *4,1,4,4,3,5,3,1,9,3,0,8,4,3,4,7,6,9,8,5,1,1,1,4,
     *0,3,1,2,3,6,1,7,2,8,1,2,3,1,7,2,2,6,6,6,9,9,7,3,
     *4,6,8,9,5,9,0,5,8,6,5,2,7,6,6,4,0,5,5,4,2,4,1,4,
     *6,9,2,2,8,0,7,0,7,8,6,0,0,6,0,4,9,5,8,2,0,0,8,2,
     *5,6,5,5,2,5,1,4,5,6,7,4,0,1,8,0,2,5,0,0,2,5,0,0/
      DATA (KRAND(I),I=4105, 4560)
     */1,4,1,8,6,1,0,3,3,8,8,0,2,9,5,6,5,2,2,4,1,5,2,2,
     *1,6,9,5,2,8,7,8,7,3,7,4,7,5,9,8,4,1,7,1,3,6,8,2,
     *9,6,1,7,4,2,6,9,1,6,6,3,8,4,2,7,6,5,8,7,2,5,5,2,
     *8,2,6,5,7,1,9,6,0,8,2,0,1,8,3,0,2,3,0,3,5,2,0,7,
     *7,2,1,3,1,0,3,0,4,8,0,0,8,3,0,0,2,6,0,3,9,6,5,6,
     *6,9,7,9,3,7,0,0,9,0,5,0,7,0,6,4,2,8,2,1,7,4,3,7,
     *5,9,0,2,0,0,4,6,8,7,9,4,2,2,6,1,1,9,0,6,7,2,7,9,
     *1,1,0,9,6,2,1,3,8,0,9,7,7,1,9,6,0,6,9,8,0,8,6,2,
     *0,3,6,6,3,0,3,2,4,9,6,6,9,0,8,9,7,1,9,6,3,6,8,1,
     *2,7,4,0,2,1,8,3,8,2,2,0,8,3,9,6,6,6,2,5,3,6,1,1,
     *2,9,8,2,5,4,6,8,8,7,2,5,6,4,5,5,8,7,2,9,4,9,7,8,
     *4,5,9,5,4,6,5,9,5,8,0,7,8,6,6,7,2,3,9,1,7,3,0,6,
     *3,8,0,1,5,2,0,1,4,5,5,8,3,0,1,2,6,1,1,3,0,9,9,5,
     *9,3,4,5,0,9,5,3,1,4,5,6,8,3,7,9,2,8,4,1,9,8,1,5,
     *0,7,8,6,1,0,7,2,7,9,5,5,2,6,9,9,8,1,5,7,5,0,0,8,
     *3,3,8,5,3,6,7,6,0,6,9,1,2,5,2,8,2,6,9,4,6,9,2,9,
     *8,9,3,1,8,6,1,4,3,5,8,5,3,1,4,3,4,9,8,0,3,3,2,0,
     *4,6,0,0,2,9,4,8,7,7,2,8,6,1,9,7,7,0,3,1,5,2,2,7,
     *7,6,6,4,2,1,5,8,0,2,0,9,2,6,2,0,8,0,0,9,5,8,4,8/
      DATA (KRAND(I),I=4561, 5016)
     */9,5,6,8,5,3,9,4,2,8,9,6,7,9,0,4,6,3,6,8,8,3,5,8,
     *0,6,5,8,5,1,9,5,2,5,9,8,2,8,8,6,3,3,8,2,4,6,7,0,
     *7,7,7,8,1,4,4,8,2,2,3,1,0,1,5,0,3,7,4,8,2,7,6,9,
     *3,3,2,0,3,2,4,1,8,5,4,7,6,6,1,1,8,1,6,4,2,3,1,2,
     *2,7,8,5,2,7,6,9,6,1,7,6,2,4,8,0,2,1,9,5,9,0,0,4,
     *4,8,2,2,8,2,8,0,8,0,6,8,5,7,2,2,2,5,6,8,3,7,6,0,
     *0,4,4,0,8,4,9,4,8,1,8,0,3,8,8,6,7,5,2,0,0,3,9,1,
     *0,2,4,3,5,7,9,5,6,5,3,0,0,9,0,1,0,0,9,6,6,4,2,7,
     *7,6,6,5,8,1,0,3,5,6,4,3,5,9,9,7,6,8,8,8,5,9,1,1,
     *8,9,2,1,9,7,6,7,6,0,7,0,0,4,9,5,2,5,2,1,4,0,8,8,
     *5,2,4,6,9,4,7,6,3,5,4,4,2,5,0,6,1,8,7,8,9,5,9,8,
     *2,1,7,6,9,8,7,7,9,3,5,3,2,9,9,9,0,5,2,3,0,9,2,1,
     *3,3,0,1,4,3,0,4,1,0,3,3,4,9,6,3,6,7,1,4,3,9,6,9,
     *3,3,2,1,8,2,9,6,2,8,8,1,9,9,8,9,5,9,3,3,3,3,7,4,
     *9,3,3,4,4,9,0,9,7,1,1,2,5,0,0,2,4,0,7,2,9,8,4,3,
     *3,2,3,7,9,8,3,2,7,8,1,9,0,0,7,7,6,3,7,0,6,2,5,9,
     *4,9,7,0,2,0,1,3,1,8,4,0,7,2,7,3,8,8,7,2,3,4,3,1,
     *4,4,0,1,4,1,9,5,0,8,3,5,9,8,4,5,3,7,3,8,0,2,8,1,
     *0,4,4,1,4,5,4,9,2,5,2,4,6,1,5,3,5,3,0,9,1,4,6,4/
      DATA (KRAND(I),I=5017, 5472)
     */8,6,5,0,1,5,0,5,0,2,0,7,3,6,2,4,6,6,5,3,2,4,9,4,
     *6,2,2,0,6,8,2,4,4,8,5,4,7,9,5,6,3,8,8,4,5,0,1,8,
     *3,7,2,6,4,6,9,4,6,0,9,0,6,8,4,8,9,9,0,0,2,1,1,2,
     *3,4,3,4,1,5,3,1,9,4,6,9,3,0,4,2,2,8,0,3,0,1,2,3,
     *6,5,4,8,9,6,1,6,2,8,2,7,1,1,9,8,3,0,9,1,4,9,5,7,
     *4,1,4,4,3,5,3,1,9,3,0,8,4,3,4,7,6,9,8,5,1,1,1,4,
     *0,3,1,2,3,6,1,7,2,8,1,2,3,1,7,2,2,6,6,6,9,9,7,3,
     *4,6,8,9,5,9,0,5,8,6,5,2,7,6,6,4,0,5,5,4,2,4,1,4,
     *6,9,2,2,8,0,7,0,7,8,6,0,0,6,0,4,9,5,8,2,0,0,8,2,
     *5,6,5,5,2,5,1,4,5,6,7,4,0,1,8,0,2,5,0,0,2,5,0,0,
     *1,4,1,8,6,1,0,3,3,8,8,0,2,9,5,6,5,2,2,4,1,5,2,2,
     *1,6,9,5,2,8,7,8,7,3,7,4,7,5,9,8,4,1,7,1,3,6,8,2,
     *9,6,1,7,4,2,6,9,1,6,6,3,8,4,2,7,6,5,8,7,2,5,5,2,
     *8,2,6,5,7,1,9,6,0,8,2,0,1,8,3,0,2,3,0,3,5,2,0,7,
     *7,2,1,3,1,0,3,0,4,8,0,0,8,3,0,0,2,6,0,3,9,6,5,6,
     *6,9,7,9,3,7,0,0,9,0,5,0,7,0,6,4,2,8,2,1,7,4,3,7,
     *5,9,0,2,0,0,4,6,8,7,9,4,2,2,6,1,1,9,0,6,7,2,7,9,
     *1,1,0,9,6,2,1,3,8,0,9,7,7,1,9,6,0,6,9,8,0,8,6,2,
     *0,3,6,6,3,0,3,2,4,9,6,6,9,0,8,9,7,1,9,6,3,6,8,1/
      DATA (KRAND(I),I=5473, 5928)
     */2,7,4,0,2,1,8,3,8,2,2,0,8,3,9,6,6,6,2,5,3,6,1,1,
     *2,9,8,2,5,4,6,8,8,7,2,5,6,4,5,5,8,7,2,9,4,9,7,8,
     *4,5,9,5,4,6,5,9,5,8,0,7,8,6,6,7,2,3,9,1,7,3,0,6,
     *3,8,0,1,5,2,0,1,4,5,5,8,3,0,1,2,6,1,1,3,0,9,9,5,
     *9,3,4,5,0,9,5,3,1,4,5,6,8,3,7,9,2,8,4,1,9,8,1,5,
     *0,7,8,6,1,0,7,2,7,9,5,5,2,6,9,9,8,1,5,7,5,0,0,8,
     *3,3,8,5,3,6,7,6,0,6,9,1,2,5,2,8,2,6,9,4,6,9,2,9,
     *8,9,3,1,8,6,1,4,3,5,8,5,3,1,4,3,4,9,8,0,3,3,2,0,
     *4,6,0,0,2,9,4,8,7,7,2,8,6,1,9,7,7,0,3,1,5,2,2,7,
     *7,6,6,4,2,1,5,8,0,2,0,9,2,6,2,0,8,0,0,9,5,8,4,8,
     *9,5,6,8,5,3,9,4,2,8,9,6,7,9,0,4,6,3,6,8,8,3,5,8,
     *0,6,5,8,5,1,9,5,2,5,9,8,2,8,8,6,3,3,8,2,4,6,7,0,
     *7,7,7,8,1,4,4,8,2,2,3,1,0,1,5,0,3,7,4,8,2,7,6,9,
     *3,3,2,0,3,2,4,1,8,5,4,7,6,6,1,1,8,1,6,4,2,3,1,2,
     *2,7,8,5,2,7,6,9,6,1,7,6,2,4,8,0,2,1,9,5,9,0,0,4,
     *4,8,2,2,8,2,8,0,8,0,6,8,5,7,2,2,2,5,6,8,3,7,6,0,
     *0,4,4,0,8,4,9,4,8,1,8,0,3,8,8,6,7,5,2,0,0,3,9,1,
     *0,2,4,3,5,7,9,5,6,5,3,0,0,9,0,1,0,0,9,6,6,4,2,7,
     *7,6,6,5,8,1,0,3,5,6,4,3,5,9,9,7,6,8,8,8,5,9,1,1/
      DATA (KRAND(I),I=5929, 6384)
     */8,9,2,1,9,7,6,7,6,0,7,0,0,4,9,5,2,5,2,1,4,0,8,8,
     *5,2,4,6,9,4,7,6,3,5,4,4,2,5,0,6,1,8,7,8,9,5,9,8,
     *2,1,7,6,9,8,7,7,9,3,5,3,2,9,9,9,0,5,2,3,0,9,2,1,
     *3,3,0,1,4,3,0,4,1,0,3,3,4,9,6,3,6,7,1,4,3,9,6,9,
     *3,3,2,1,8,2,9,6,2,8,8,1,9,9,8,9,5,9,3,3,3,3,7,4,
     *9,3,3,4,4,9,0,9,7,1,1,2,5,0,0,2,4,0,7,2,9,8,4,3,
     *3,2,3,7,9,8,3,2,7,8,1,9,0,0,7,7,6,3,7,0,6,2,5,9,
     *4,9,7,0,2,0,1,3,1,8,4,0,7,2,7,3,8,8,7,2,3,4,3,1,
     *4,4,0,1,4,1,9,5,0,8,3,5,9,8,4,5,3,7,3,8,0,2,8,1,
     *0,4,4,1,4,5,4,9,2,5,2,4,6,1,5,3,5,3,0,9,1,4,6,4,
     *8,6,5,0,1,5,0,5,0,2,0,7,3,6,2,4,6,6,5,3,2,4,9,4,
     *6,2,2,0,6,8,2,4,4,8,5,4,7,9,5,6,3,8,8,4,5,0,1,8,
     *3,7,2,6,4,6,9,4,6,0,9,0,6,8,4,8,9,9,0,0,2,1,1,2,
     *3,4,3,4,1,5,3,1,9,4,6,9,3,0,4,2,2,8,0,3,0,1,2,3,
     *6,5,4,8,9,6,1,6,2,8,2,7,1,1,9,8,3,0,9,1,4,9,5,7,
     *4,1,4,4,3,5,3,1,9,3,0,8,4,3,4,7,6,9,8,5,1,1,1,4,
     *0,3,1,2,3,6,1,7,2,8,1,2,3,1,7,2,2,6,6,6,9,9,7,3,
     *4,6,8,9,5,9,0,5,8,6,5,2,7,6,6,4,0,5,5,4,2,4,1,4,
     *6,9,2,2,8,0,7,0,7,8,6,0,0,6,0,4,9,5,8,2,0,0,8,2/
      DATA (KRAND(I),I=6385, 6840)
     */5,6,5,5,2,5,1,4,5,6,7,4,0,1,8,0,2,5,0,0,2,5,0,0,
     *1,4,1,8,6,1,0,3,3,8,8,0,2,9,5,6,5,2,2,4,1,5,2,2,
     *1,6,9,5,2,8,7,8,7,3,7,4,7,5,9,8,4,1,7,1,3,6,8,2,
     *9,6,1,7,4,2,6,9,1,6,6,3,8,4,2,7,6,5,8,7,2,5,5,2,
     *8,2,6,5,7,1,9,6,0,8,2,0,1,8,3,0,2,3,0,3,5,2,0,7,
     *7,2,1,3,1,0,3,0,4,8,0,0,8,3,0,0,2,6,0,3,9,6,5,6,
     *6,9,7,9,3,7,0,0,9,0,5,0,7,0,6,4,2,8,2,1,7,4,3,7,
     *5,9,0,2,0,0,4,6,8,7,9,4,2,2,6,1,1,9,0,6,7,2,7,9,
     *1,1,0,9,6,2,1,3,8,0,9,7,7,1,9,6,0,6,9,8,0,8,6,2,
     *0,3,6,6,3,0,3,2,4,9,6,6,9,0,8,9,7,1,9,6,3,6,8,1,
     *2,7,4,0,2,1,8,3,8,2,2,0,8,3,9,6,6,6,2,5,3,6,1,1,
     *2,9,8,2,5,4,6,8,8,7,2,5,6,4,5,5,8,7,2,9,4,9,7,8,
     *4,5,9,5,4,6,5,9,5,8,0,7,8,6,6,7,2,3,9,1,7,3,0,6,
     *3,8,0,1,5,2,0,1,4,5,5,8,3,0,1,2,6,1,1,3,0,9,9,5,
     *9,3,4,5,0,9,5,3,1,4,5,6,8,3,7,9,2,8,4,1,9,8,1,5,
     *0,7,8,6,1,0,7,2,7,9,5,5,2,6,9,9,8,1,5,7,5,0,0,8,
     *3,3,8,5,3,6,7,6,0,6,9,1,2,5,2,8,2,6,9,4,6,9,2,9,
     *8,9,3,1,8,6,1,4,3,5,8,5,3,1,4,3,4,9,8,0,3,3,2,0,
     *4,6,0,0,2,9,4,8,7,7,2,8,6,1,9,7,7,0,3,1,5,2,2,7/
      DATA (KRAND(I),I=6841, 7296)
     */7,6,6,4,2,1,5,8,0,2,0,9,2,6,2,0,8,0,0,9,5,8,4,8,
     *9,5,6,8,5,3,9,4,2,8,9,6,7,9,0,4,6,3,6,8,8,3,5,8,
     *0,6,5,8,5,1,9,5,2,5,9,8,2,8,8,6,3,3,8,2,4,6,7,0,
     *7,7,7,8,1,4,4,8,2,2,3,1,0,1,5,0,3,7,4,8,2,7,6,9,
     *3,3,2,0,3,2,4,1,8,5,4,7,6,6,1,1,8,1,6,4,2,3,1,2,
     *2,7,8,5,2,7,6,9,6,1,7,6,2,4,8,0,2,1,9,5,9,0,0,4,
     *4,8,2,2,8,2,8,0,8,0,6,8,5,7,2,2,2,5,6,8,3,7,6,0,
     *0,4,4,0,8,4,9,4,8,1,8,0,3,8,8,6,7,5,2,0,0,3,9,1,
     *0,2,4,3,5,7,9,5,6,5,3,0,0,9,0,1,0,0,9,6,6,4,2,7,
     *7,6,6,5,8,1,0,3,5,6,4,3,5,9,9,7,6,8,8,8,5,9,1,1,
     *8,9,2,1,9,7,6,7,6,0,7,0,0,4,9,5,2,5,2,1,4,0,8,8,
     *5,2,4,6,9,4,7,6,3,5,4,4,2,5,0,6,1,8,7,8,9,5,9,8,
     *2,1,7,6,9,8,7,7,9,3,5,3,2,9,9,9,0,5,2,3,0,9,2,1,
     *3,3,0,1,4,3,0,4,1,0,3,3,4,9,6,3,6,7,1,4,3,9,6,9,
     *3,3,2,1,8,2,9,6,2,8,8,1,9,9,8,9,5,9,3,3,3,3,7,4,
     *9,3,3,4,4,9,0,9,7,1,1,2,5,0,0,2,4,0,7,2,9,8,4,3,
     *3,2,3,7,9,8,3,2,7,8,1,9,0,0,7,7,6,3,7,0,6,2,5,9,
     *4,9,7,0,2,0,1,3,1,8,4,0,7,2,7,3,8,8,7,2,3,4,3,1,
     *4,4,0,1,4,1,9,5,0,8,3,5,9,8,4,5,3,7,3,8,0,2,8,1/
      DATA (KRAND(I),I=7297, 7752)
     */0,4,4,1,4,5,4,9,2,5,2,4,6,1,5,3,5,3,0,9,1,4,6,4,
     *8,6,5,0,1,5,0,5,0,2,0,7,3,6,2,4,6,6,5,3,2,4,9,4,
     *6,2,2,0,6,8,2,4,4,8,5,4,7,9,5,6,3,8,8,4,5,0,1,8,
     *3,7,2,6,4,6,9,4,6,0,9,0,6,8,4,8,9,9,0,0,2,1,1,2,
     *3,4,3,4,1,5,3,1,9,4,6,9,3,0,4,2,2,8,0,3,0,1,2,3,
     *6,5,4,8,9,6,1,6,2,8,2,7,1,1,9,8,3,0,9,1,4,9,5,7,
     *4,1,4,4,3,5,3,1,9,3,0,8,4,3,4,7,6,9,8,5,1,1,1,4,
     *0,3,1,2,3,6,1,7,2,8,1,2,3,1,7,2,2,6,6,6,9,9,7,3,
     *4,6,8,9,5,9,0,5,8,6,5,2,7,6,6,4,0,5,5,4,2,4,1,4,
     *6,9,2,2,8,0,7,0,7,8,6,0,0,6,0,4,9,5,8,2,0,0,8,2,
     *5,6,5,5,2,5,1,4,5,6,7,4,0,1,8,0,2,5,0,0,2,5,0,0,
     *1,4,1,8,6,1,0,3,3,8,8,0,2,9,5,6,5,2,2,4,1,5,2,2,
     *1,6,9,5,2,8,7,8,7,3,7,4,7,5,9,8,4,1,7,1,3,6,8,2,
     *9,6,1,7,4,2,6,9,1,6,6,3,8,4,2,7,6,5,8,7,2,5,5,2,
     *8,2,6,5,7,1,9,6,0,8,2,0,1,8,3,0,2,3,0,3,5,2,0,7,
     *7,2,1,3,1,0,3,0,4,8,0,0,8,3,0,0,2,6,0,3,9,6,5,6,
     *6,9,7,9,3,7,0,0,9,0,5,0,7,0,6,4,2,8,2,1,7,4,3,7,
     *5,9,0,2,0,0,4,6,8,7,9,4,2,2,6,1,1,9,0,6,7,2,7,9,
     *1,1,0,9,6,2,1,3,8,0,9,7,7,1,9,6,0,6,9,8,0,8,6,2/
      DATA (KRAND(I),I=7753, 8208)
     */0,3,6,6,3,0,3,2,4,9,6,6,9,0,8,9,7,1,9,6,3,6,8,1,
     *2,7,4,0,2,1,8,3,8,2,2,0,8,3,9,6,6,6,2,5,3,6,1,1,
     *2,9,8,2,5,4,6,8,8,7,2,5,6,4,5,5,8,7,2,9,4,9,7,8,
     *4,5,9,5,4,6,5,9,5,8,0,7,8,6,6,7,2,3,9,1,7,3,0,6,
     *3,8,0,1,5,2,0,1,4,5,5,8,3,0,1,2,6,1,1,3,0,9,9,5,
     *9,3,4,5,0,9,5,3,1,4,5,6,8,3,7,9,2,8,4,1,9,8,1,5,
     *0,7,8,6,1,0,7,2,7,9,5,5,2,6,9,9,8,1,5,7,5,0,0,8,
     *3,3,8,5,3,6,7,6,0,6,9,1,2,5,2,8,2,6,9,4,6,9,2,9,
     *8,9,3,1,8,6,1,4,3,5,8,5,3,1,4,3,4,9,8,0,3,3,2,0,
     *4,6,0,0,2,9,4,8,7,7,2,8,6,1,9,7,7,0,3,1,5,2,2,7,
     *7,6,6,4,2,1,5,8,0,2,0,9,2,6,2,0,8,0,0,9,5,8,4,8,
     *9,5,6,8,5,3,9,4,2,8,9,6,7,9,0,4,6,3,6,8,8,3,5,8,
     *0,6,5,8,5,1,9,5,2,5,9,8,2,8,8,6,3,3,8,2,4,6,7,0,
     *7,7,7,8,1,4,4,8,2,2,3,1,0,1,5,0,3,7,4,8,2,7,6,9,
     *3,3,2,0,3,2,4,1,8,5,4,7,6,6,1,1,8,1,6,4,2,3,1,2,
     *2,7,8,5,2,7,6,9,6,1,7,6,2,4,8,0,2,1,9,5,9,0,0,4,
     *4,8,2,2,8,2,8,0,8,0,6,8,5,7,2,2,2,5,6,8,3,7,6,0,
     *0,4,4,0,8,4,9,4,8,1,8,0,3,8,8,6,7,5,2,0,0,3,9,1,
     *0,2,4,3,5,7,9,5,6,5,3,0,0,9,0,1,0,0,9,6,6,4,2,7/
      DATA (KRAND(I),I=8209, 8664)
     */7,6,6,5,8,1,0,3,5,6,4,3,5,9,9,7,6,8,8,8,5,9,1,1,
     *8,9,2,1,9,7,6,7,6,0,7,0,0,4,9,5,2,5,2,1,4,0,8,8,
     *5,2,4,6,9,4,7,6,3,5,4,4,2,5,0,6,1,8,7,8,9,5,9,8,
     *2,1,7,6,9,8,7,7,9,3,5,3,2,9,9,9,0,5,2,3,0,9,2,1,
     *3,3,0,1,4,3,0,4,1,0,3,3,4,9,6,3,6,7,1,4,3,9,6,9,
     *3,3,2,1,8,2,9,6,2,8,8,1,9,9,8,9,5,9,3,3,3,3,7,4,
     *9,3,3,4,4,9,0,9,7,1,1,2,5,0,0,2,4,0,7,2,9,8,4,3,
     *3,2,3,7,9,8,3,2,7,8,1,9,0,0,7,7,6,3,7,0,6,2,5,9,
     *4,9,7,0,2,0,1,3,1,8,4,0,7,2,7,3,8,8,7,2,3,4,3,1,
     *4,4,0,1,4,1,9,5,0,8,3,5,9,8,4,5,3,7,3,8,0,2,8,1,
     *0,4,4,1,4,5,4,9,2,5,2,4,6,1,5,3,5,3,0,9,1,4,6,4,
     *8,6,5,0,1,5,0,5,0,2,0,7,3,6,2,4,6,6,5,3,2,4,9,4,
     *6,2,2,0,6,8,2,4,4,8,5,4,7,9,5,6,3,8,8,4,5,0,1,8,
     *3,7,2,6,4,6,9,4,6,0,9,0,6,8,4,8,9,9,0,0,2,1,1,2,
     *3,4,3,4,1,5,3,1,9,4,6,9,3,0,4,2,2,8,0,3,0,1,2,3,
     *6,5,4,8,9,6,1,6,2,8,2,7,1,1,9,8,3,0,9,1,4,9,5,7,
     *4,1,4,4,3,5,3,1,9,3,0,8,4,3,4,7,6,9,8,5,1,1,1,4,
     *0,3,1,2,3,6,1,7,2,8,1,2,3,1,7,2,2,6,6,6,9,9,7,3,
     *4,6,8,9,5,9,0,5,8,6,5,2,7,6,6,4,0,5,5,4,2,4,1,4/
      DATA (KRAND(I),I=8665, 8784)
     */6,9,2,2,8,0,7,0,7,8,6,0,0,6,0,4,9,5,8,2,0,0,8,2,
     *5,6,5,5,2,5,1,4,5,6,7,4,0,1,8,0,2,5,0,0,2,5,0,0,
     *1,4,1,8,6,1,0,3,3,8,8,0,2,9,5,6,5,2,2,4,1,5,2,2,
     *1,6,9,5,2,8,7,8,7,3,7,4,7,5,9,8,4,1,7,1,3,6,8,2,
     *9,6,1,7,4,2,6,9,1,6,6,3,8,4,2,7,6,5,8,7,2,5,5,2/

      DATA ERRCOD(1)/'1001'/,
     &ERRMSG(1)/'ERROR OPENING THE INPUT FILE:'/

      DATA ERRCOD(2)/'1002'/,
     &ERRMSG(2)/'ERROR OPENING FILE FOR SCREEN INPUT RESPONSES'/

      DATA ERRCOD(3)/'1003'/,
     &ERRMSG(3)/'ERROR READING THE INPUT FILE: DEPOSITION TYPE'/

      DATA ERRCOD(4)/'1004'/,
     &ERRMSG(4)/'INCORRECT DEPOSITION TYPE SPECIFIED:'/

C     The error code 1005 has been removed

      DATA ERRCOD(6)/'1006'/,
     &ERRMSG(6)/'ERROR READING THE INPUT FILE: OUTPUT FILE TYPE'/

      DATA ERRCOD(7)/'1007'/,
     &ERRMSG(7)/'ERROR READING THE INPUT FILE: SURFACE FILE NAME'/

      DATA ERRCOD(8)/'1008'/,
     &ERRMSG(8)/'ERROR READING THE INPUT FILE: SURFACE FILE TYPE'/

      DATA ERRCOD(9)/'1009'/,
     &ERRMSG(9)/'ERROR READING THE INPUT FILE: MIXING HEIGHT FILE NAME'/

      DATA ERRCOD(10)/'1010'/,
     &ERRMSG(10)/'ERROR READING THE INPUT FILE: PRECIPITATION YES|NO'/

      DATA ERRCOD(11)/'1011'/,
     &ERRMSG(11)/'ERROR READING THE INPUT FILE: PRECIP FILE NAME'/

      DATA ERRCOD(12)/'1012'/,
     &ERRMSG(12)/'ERROR READING THE INPUT FILE: PRECIP FILE TYPE'/

      DATA ERRCOD(13)/'1013'/,
     &ERRMSG(13)/'ERROR READING THE INPUT FILE: SITE LATITUDE'/

      DATA ERRCOD(14)/'1014'/,
     &ERRMSG(14)/'ERROR READING THE INPUT FILE: SITE LONGITUDE'/

      DATA ERRCOD(15)/'1015'/,
     &ERRMSG(15)/'ERROR READING THE INPUT FILE: MIN OBUKHOV LENGTH'/

      DATA ERRCOD(16)/'1016'/,
     &ERRMSG(16)/'ERROR READING THE INPUT FILE: ANEMOMETER HGT'/

      DATA ERRCOD(23)/'1023'/,
     &ERRMSG(23)/'ERROR OPENING THE SURFACE FILE:'/

      DATA ERRCOD(24)/'1024'/,
     &ERRMSG(24)/'ERROR OPENING THE MIXING HEIGHT FILE:'/

      DATA ERRCOD(25)/'1025'/,
     &ERRMSG(25)/'ERROR OPENING THE PRECIPITATION FILE:'/

      DATA ERRCOD(26)/'1026'/,
     &ERRMSG(26)/'PRECIP FILE NOT VARIABLE FORMAT AS SPECIFIED BY USER'/

      DATA ERRCOD(27)/'1027'/,
     &ERRMSG(27)/'ERROR OPENING TEMPORARY PRECIPITATION FILE'/

      DATA ERRCOD(28)/'1028'/,
     &ERRMSG(28)/'PRECIP FILE NOT FIXED FORMAT AS SPECIFIED BY USER'/

      DATA ERRCOD(29)/'1029'/,
     &ERRMSG(29)/'ERROR OPENING PRECIPITATION FILE:'/

      DATA ERRCOD(30)/'1030'/,
     &ERRMSG(30)/'ERROR OPENING OUTPUT FILE (UNFORM):'/

      DATA ERRCOD(31)/'1031'/,
     &ERRMSG(31)/'ERROR OPENING OUTPUT FILE (ASCII):'/

      DATA ERRCOD(32)/'1032'/,
     &ERRMSG(32)/'SURFACE FILE NOT OF TYPE CD-144 AS SPECIFIED BY USER'/

      DATA ERRCOD(33)/'1033'/,
     &ERRMSG(33)/'SURFACE FILE NOT OF TYPE SAMSON AS SPECIFIED BY USER'/

      DATA ERRCOD(34)/'1034'/,
     &ERRMSG(34)/'SURFACE FILE NOT SCRAM FORMAT AS SPECIFIED BY USER'/

      DATA ERRCOD(35)/'1035'/,
     &ERRMSG(35)/'ERROR READING HEADER RECORD OF THE SURFACE FILE:'/

      DATA ERRCOD(36)/'1036'/,
     &ERRMSG(36)/'ILLEGAL VARIABLE ID IN HEADER RECORD:'/

      DATA ERRCOD(37)/'1037'/,
     &ERRMSG(37)/'ERROR READING SAMSON SURFACE DATA:'/

      DATA ERRCOD(38)/'1038'/,
     &ERRMSG(38)/'ERROR READING CD144 SURFACE DATA:'/

      DATA ERRCOD(39)/'1039'/,
     &ERRMSG(39)/'ERROR READING MIXING HEIGHT DATA:'/

      DATA ERRCOD(40)/'1040'/,
     &ERRMSG(40)/'SURFACE AND MIXING HEIGHT TIME STAMP MISMATCH:'/

      DATA ERRCOD(41)/'1041'/,
     &ERRMSG(41)/'SURFACE AND PRECIP DATA TIME STAMP MISMATCH:'/

      DATA ERRCOD(42)/'1042'/,
     &ERRMSG(42)/'STABILITY CALCULATED TO BE LESS THAN 1:'/

      DATA ERRCOD(43)/'1043'/,
     &ERRMSG(43)/'SCRAM DATA and WET DEPOSITION NOT COMPATIBLE'/

      DATA ERRCOD(44)/'1101'/,
     &ERRMSG(44)/'ERROR READING TIME ZONE FROM THE INPUT FILE'/

      DATA ERRCOD(45)/'1110'/,
     &ERRMSG(45)/'ANEMOMETER HT < ROUGHNESS LENGTH       '/

      DATA ERRCOD(46)/'1115'/,
     &ERRMSG(46)/'ANEMOMETER HT < 0.0                    '/

      DATA ERRCOD(47)/'1120'/,
     & ERRMSG(47)/'INSUFFICIENT DATA IN SAMSON FILE FOR PROCESSING'/

      DATA ERRCOD(48)/'1125'/,
     & ERRMSG(48)/'INSUFFICIENT DATA IN SAMSON FILE FOR WET DEPOSITION'/

      DATA ERRCOD(49)/'1130'/,
     & ERRMSG(49)/'ALBEDO OUT OF RANGE                               '/

      DATA ERRCOD(50)/'1135'/,
     & ERRMSG(50)/'NET RADIATION ABSORBED BY GROUND OUT OF RANGE     '/

      DATA ERRCOD(51)/'1140'/,
     & ERRMSG(51)/'INSUFFICIENT DATA IN HUSWO FILE FOR PROCESSING'/

      DATA ERRCOD(52)/'1145'/,
     & ERRMSG(52)/'INSUFFICIENT DATA IN HUSWO FILE FOR WET DEPOSITION'/

      DATA ERRCOD(53)/'1044'/,
     &ERRMSG(53)/'SURFACE FILE NOT OF TYPE HUSWO AS SPECIFIED BY USER'/

      DATA ERRCOD(54)/'1045'/,
     &ERRMSG(54)/'ERROR READING HUSWO SURFACE DATA:'/

      DATA ERRCOD(55)/'1126'/,
     &ERRMSG(55)( 1:43)/'GLOBAL RAD''N OR REL HUMIDITY NOT EXTRACTED '/,
     &ERRMSG(55)(44:61)/'FOR DEPOSITION RUN'/

C     The following 'error message' is a place holder for messages that
C     do not conform to the standard PCRAMMET format (e.g., in SUBR.PREAD)

      DATA ERRCOD(60)/'0000'/,
     & ERRMSG(60)/'                                                  '/

      END
      SUBROUTINE STABIT(IDAY,IHR)
C***********************************************************************
C*    STABIT      Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE: This Routine Calculates Hourly Stability Classes
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*                Adapted From MPRM
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:     Wind speed, ciling height, cloud cover,
C*                sunris/sunset/solar elevation angle
C*
C*    OUTPUTS:    Pasquill-Gifford Stability Category
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      REAL ANGL(3)
      INTEGER LSTAB(12,7)
      DATA ANGL /60.0, 35.0, 15.0/
      DATA LSTAB /7,7,7,6,6,6,5,5,5,5,4,4,
     &            6,6,6,5,5,5,4,4,4,4,4,4,
     &            4,4,4,4,4,4,4,4,4,4,4,4,
     &            3,3,3,4,4,4,4,4,4,4,4,4,
     &            2,2,2,3,3,3,3,3,3,4,4,4,
     &            1,2,2,2,2,2,2,3,3,3,3,4,
     &            1,1,1,1,1,2,2,2,2,3,3,3/

C*    INITIALIZE STABILITY BEFORE IT IS CALCULATED
      KST(IHR)  = 9
      ISPD      = 0
      IRADX     = 0

C*    Determine the wind speed index based on the wind speed (in knots)
      ISPD = KSPEED(IHR)
      IF( ISPD .LT. 1 )  ISPD = 1
      IF( ISPD .GT. 12 ) ISPD = 12

      ICLHT = ICEIL(IHR)
      ISKY  = ICOVER(IHR)

      IF( ISKY .LE. 10  .AND.  KSPEED(IHR) .LT. 90 )THEN

C*       DETERMINE RADIATION INDEX.
         IF (ISKY .EQ. 10 .AND. ICLHT .LT. 70) THEN

C*          LOW/MIDDLE OVERCAST CLOUD LAYER: NEUTRAL CONDITIONS
            IRADX = 3

         ELSE IF (IHR .GT. SUNRIS .AND. IHR .LT. SUNSET) THEN

C*          Daytime conditions
            DO 220 I=1,3
               IF (SOLANG(IHR) .GT. ANGL(I)) GO TO 230
220         CONTINUE
            I=4
230         ICN=5-I
            IF (ISKY .GT. 5) GO TO 240
            IRADX=ICN+3
            GO TO 280
240         IRADX=ICN-1
            IF (ICLHT .LT. 70) GO TO 250
            IF (ICLHT .LT. 160) GO TO 260
            IF (ISKY .EQ. 10) GO TO 270
            IRADX=ICN
            GO TO 270
250         IRADX=ICN-2
            GO TO 270
260         IF (ISKY .EQ. 10) IRADX=IRADX-1
270         IF (IRADX .LT. 1) IRADX=1
            IRADX=IRADX+3

         ELSE
C*          Nighttime conditions
            IRADX=2
            IF (ISKY .LE. 4) IRADX=1

         END IF
C
C*       Determine stability.
280      KST(IHR) = LSTAB(ISPD,IRADX)

C*       Do not allow stability to vary by more than one category hour-to-hour
         IF (IDAY.EQ.1.AND.IHR.EQ.1) LST=KST(IHR)
         IF ((KST(IHR)-LST).GT.1) KST(IHR)=LST+1
         IF ((LST-KST(IHR)).GT.1) KST(IHR)=LST-1
         LST=KST(IHR)

      ENDIF

      IF (KST(IHR) .LT. 1) THEN
1042     CALL ERRHDL('E','1042',1)
      END IF

      RETURN
      END


      SUBROUTINE MIXHIT(IHR)
C***********************************************************************
C*    MIXHIT      Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE: This Routine Calculates Hourly Mixing Heights
C*             By Interpolating Between the Twice Daily Values
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:     Twice-daily mixing heights, sunrise and sunset,
C*                P-G category
C*
C*    OUTPUTS:    Mixing height for the hour
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

C*    Fill Local Variables With Common Block Data, Change Integer
C*    to Real ...

      XHR=IHR

C*    ...Previous Day - AM and PM
      XMNM1 = AMMIX(1)
      XAFM1 = PMMIX(1)

C*    ...Current Day  - AM and PM
      XMN = AMMIX(2)
      XAF = PMMIX(2)

C*    ...Next Day     - AM and PM
      XMNP1 = AMMIX(3)
      XAFP1 = PMMIX(3)

C*    CALCULATE MIXING HEIGHT
      IF ( IHR.GT.14  .AND.  XHR.LE.SUNSET ) GO TO 300
      IND=2
      IF (XHR.LE.SUNSET) GO TO 310
      IF (KST(IHR).EQ.4) GO TO 290
      HLH(2,IHR) = XAF + (XMNP1-XAF) * ((XHR-SUNSET)/(24.0-SUNSET))
      IND=1
290   HLH(IND,IHR) = XAF + (XAFP1-XAF) * ((XHR-SUNSET)/(38.0-SUNSET))
      IF (IND.EQ.2) HLH(1,IHR)=HLH(2,IHR)
      GO TO 360

300   HLH(1,IHR) = XAF
      HLH(2,IHR) = XAF
      GO TO 360

310   IF (XHR.GT.SUNRIS) GO TO 330
      KSTSP=KST(IHR)
      IF (KST(IHR).EQ.4) GO TO 320
      HLH(2,IHR)=XMN
      IND=1

320   HLH(IND,IHR) = XAFM1 + (XAF-XAFM1) *
     &               ((24.0-SUNSET+XHR) / (24.0-SUNSET+14.0))
      IF (IND.EQ.2) HLH(1,IHR)=HLH(2,IHR)
      GO TO 360

330   IF (KSTSP.EQ.4) GO TO 350
      HLH(2,IHR) = XMN + (XAF-XMN) * ((XHR-SUNRIS)/(14.0-SUNRIS))
      HLH(1,IHR) = XAF * (XHR-SUNRIS)/(14.0-SUNRIS)
      GO TO 360

350   HLH(1,IHR) = XAFM1 + (XAF-XAFM1) *
     &             ((24.0-SUNSET+XHR)/(24.0-SUNSET+14.0))
      HLH(2,IHR) = HLH(1,IHR)

360   IF( HLH(1,IHR) .LT. 10.0 )THEN
         WRITE( IDIAG,550 ) HLH(1,IHR), ISYR(1),ISMO(1),ISDY(1),IHR
         NOWARN = .FALSE.
      ENDIF

      IF( HLH(2,IHR) .LT. 10.0 )THEN
         WRITE( IDIAG,555 ) HLH(2,IHR), ISYR(1),ISMO(1),ISDY(1),IHR
         NOWARN = .FALSE.
      ENDIF

      RETURN

  550 FORMAT( ' Rural Mixing Height < 10 m (', F5.1,
     &        ') for (yy/mm/dd/hh) ', 4(i2.2:,'/') )
  555 FORMAT( ' Urban Mixing Height < 10 m (', F5.1,
     &        ') for (yy/mm/dd/hh) ', 4(i2.2:,'/') )

      END


      SUBROUTINE USTARL(IHR)
C***********************************************************************
C*    USTARL      Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE: This Routine Calculates Surface Friction Velocity
C*             And Monin Obukhov Length for he Current Hour
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*                Adapted from DEPMET
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:     Hourly temperature, wind speed, cloud cover,
C*                roughness length (measurement and applied site),
C*                sunrise and sunset
C*
C*    OUTPUTS:    Friction velocity and Obukhov length at the
C*                application site
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'


C*    Trap ISCST2 "calms" and return u*=L=0.0
      IF(CALM(IHR)) THEN
         USTAR(IHR)  = 0.0
         XMONIN(IHR) = 0.0

      ELSEIF( TEMP(IHR) .LT. 900.0   .AND.  WSPEED(IHR) .LT. 90.0 .AND.
     &        FCOVER(IHR) .LE. 1.0 )THEN
C*       Sufficient data to make computations

C*       Adjust anemometer height for the displacement height at the
C        site of the measurements and the application site
C        NOTE: ZDMEAS is the displacement height,
C              which is inactive (defaulted to 0.0) at this time
         ANEMDM = ANEMHT-ZDMEAS

C*       Compute incomming solar radiation flux (W/m**2) at the
C        measurement site
         CALL RADSOL(IHR)

C*       Estimate net radiation at the measurement site
         CALL NETRAD(IHR)

C*       Estimate the sensible heat flux (SHF) in W/m**2 at the
C        measurement site
         CALL HEAT(IHR)
c
C*       Make sure that net radiation and surface heat flux are
C*       positive during the main part of the day.
         IF(FLOAT(IHR) .GT. SUNRIS+2.0  .AND.
     &      FLOAT(IHR) .LT. SUNSET-2.0) THEN
            IF(RNET(IHR) .LE. 0.) RNET(IHR) = 0.001
            IF(SHF(IHR)  .LE. 0.) SHF(IHR)  = 0.001
         ENDIF

C*       Compute nighttime u* and L using the Weil - Brower method; or
C*       compute daytime u* and L using the Holtslag - van Ulden method.
         IF(FLOAT(IHR) .GT .SUNSET+1.0  .OR.
     2      FLOAT(IHR) .LT. SUNRIS      .OR. RNET(IHR) .LE. 0.) THEN
            CALL USLNIT( IHR,ANEMDM,Z0MEAS,USTARM,XMOLM )

         ELSE
            IF(SHF(IHR) .GT. 0.0) THEN
               CALL USLDAY( IHR,ANEMDM,Z0MEAS,USTARM,XMOLM )
            ELSE
               CALL USLNIT( IHR,ANEMDM,Z0MEAS,USTARM,XMOLM )
            END IF
         END IF

C*       Compute the friction velocity (USTARA) and Monin-Obukhov
C        length (XMOLA) at the application site
         CALL USTAR2( ANEMDM,WSPEED(IHR), USTARM,XMOLM,
     &                Z0APPL,USTARA,XMOLA,ELMINM )

C*       Save these values for writing to the output file
         USTAR(IHR)  = USTARA
         XMONIN(IHR) = XMOLA

C*       Make sure that |L| < 999.0 to comply with format of output
         IF(XMONIN(IHR) .LT. -999.0) THEN
            XMONIN(IHR) = -999.0
         ELSEIF(XMONIN(IHR) .GT. 9999.0) THEN
            XMONIN(IHR) = 9999.0
         ENDIF

      ELSE
C*       Not a calm hour but there is insufficient data to make the
C*       computations - set USTAR and XMONIN to missing
         USTAR(IHR)  = -9.0
         XMONIN(IHR) = -999.0

      ENDIF

      RETURN
      END


      SUBROUTINE RADSOL(IHR)
C======================================================================
C     Purpose:  To compute the hourly total incoming solar radiation
C
C     Input:    Solar elevation, cloud cover
C
C     Output:   Insolation
C
C     Called by: USTARL
C-----------------------------------------------------------------------
      INTEGER IHR
      REAL AVGANG
      INCLUDE 'PCRAM.INC'
C
C---- First, compute the average of solar elevation angles at the
C     beginning and end of the hour.  Check for the 24th hour.
C     Note that the angles are in DEGREES - convert to radians
C
      IF (IHR.LE.23) THEN
         AVGANG = (SOLANG(IHR)+SOLANG(IHR+1))/2.0
      ELSE
         AVGANG = (SOLANG(24)+SOLANG(1))/2.0
      ENDIF

      AVGANG = AVGANG / CONST
C
C---- AVGANG can be no less than 10 degrees to avoid problems at low
C     latitudes
C     141.91 W/M**2 = incoming radiation for a 10 degree elevation angle
C     10 degrees = 0.174533 radians
C
      IF ( (AVGANG .LT. 0.174533)  .AND.  (AVGANG .GT. 0.0) )THEN
         RSOL(IHR) = 141.91 * AVGANG * 5.729578

      ELSEIF( AVGANG .LT. 0.0 )THEN
         RSOL(IHR) = 0.0

      ELSE
         RSOL(IHR)=  990.0*SIN(AVGANG) - 30.0

      ENDIF
C
C---- Finally, include a correction for the cloud cover fraction (CCVR).
C     Note that if cloud cover is missing, then CCVR is set to 0.0
C
      IF( FCOVER(IHR) .GT. 0.0 )THEN
         RSOL(IHR) = RSOL(IHR) * (1.0 - 0.75*(FCOVER(IHR)**3.4))
      ENDIF

      RETURN
      END


      SUBROUTINE NETRAD(IHR)
C====================================================================**
C     Purpose: To compute the net radiation (RN) from solar elevation
C              angle, incoming solar radiation, albedo, and temperature.
C              The equation for the net radiation is:
C
C     Input:   Insolation, solar elevation, albedo, temperature,
C              cloud cover, anthropogenic heat flux
C
C     Output:  Net radiation
C
C     Called from: USTARL
C-----------------------------------------------------------------------
      INTEGER IHR
      REAL C1,STEFB
      INCLUDE 'pcram.inc'
C
C---- Constants used in the computation
C
      C1 = 5.31E-13
      C2 = 60.0
      C3 = 1.12
      STEFB = 5.67E-08
C
C---- Adjust the albedo for the hour of the day; sun's elevation is
C     expressed in degrees in PCRAMMET
      ANGLE = SOLANG(IHR)
      B1 = 1.0 - ALBEDO
      IF( SOLANG(IHR) .LE. 0.0 )THEN
         ALBADJ = 1.0
      ELSE
         ALBADJ = ALBEDO + B1 * EXP(-0.1 * ANGLE + (-0.5 * B1 * B1) )
      ENDIF


C---- Compute the net radiation from solar insolation, albedo adjusted
C     for the hour of day, surface temperature and cloud fraction.
C
      RNET(IHR) = ( (1.0-ALBADJ)*RSOL(IHR) + C1*(TEMP(IHR)**6) -
     &               STEFB*(TEMP(IHR)**4)  + C2*FCOVER(IHR) ) / C3

C---- Add the anthropogenioc heat flux
      RNET(IHR) = RNET(IHR) + QF

      RETURN
      END


      SUBROUTINE HEAT(IHR)
C======================================================================
C     Purpose:  To calculate the heat flux, SHF, from the net radiation
C               and Bowen ratio.  The equation for the heat flux is:
C               (CONSTANT*NET RADIATION) / (1+(1/BOWEN RATIO))
C
C     Input:    Net radiation, Bowen ratio, fraction of net radiation
C               absorbed at the ground (CSUBG)
C
C     Output:   Sensible heat flux
C
C     Called from: USTARL
C-----------------------------------------------------------------------
      INTEGER IHR
      INCLUDE 'PCRAM.INC'

C---- Calculate the HEAT FLUX
C
      SHF(IHR) = ( (1.0-CSUBG) * RNET(IHR) ) / ( 1.0 + (1.0/BOWEN) )

      RETURN
      END


      SUBROUTINE USLDAY( IHR,ANEMZD,Z0M,USTARM,XMOLM )
C=======================================================================
C          USLDAY Module of PCRAMMET Meteorological Preprocessor
C
C   Purpose:  This subroutine will set up the iteration needed to
C             calculate the friction velocity during unstable
C             atmospheric cases (SHF > 0.0.) at the measurement site
C
C   Input:
C        IHR       Hour of day
C        ANEMZD    The anemometer height (adjusted for the
C                     displacement height, which is set to 0.0)
C        Z0M       Roughness length at the measurement site
C        SHF       Sensible heat flux
C        WSPEED    Wind speed
C        RHO       Density
C
C   Output:
C        USTARM    Friction velocity at the measurement site
C        XMOLM     Monin-Obukhov length at the measurement site

C
C   Programmed by:   J. Paumier, PES, Inc.               28 April 1995
C      (based on source code for AERMET, the meteorological preprocessor
C       for AERMOD and METPRO, the preprocessor for CTDMPLUS)
C
C   Called from:  USTARL
C-----------------------------------------------------------------------
C
C---- Local variables
C
      INTEGER IHR, ITER
      REAL    PSIZL, PSIZOL, EPS, VONK, LASTL
      REAL    ANEMZD, MU, MU0, CP, PI, GRAV
      REAL    Z0M, USTARM, XMOLM
C
C---- Include common blocks & declarations
C
      INCLUDE 'PCRAM.INC'
C
C---- Define constants.
C       CP   = specific volume of dry air at constant pressure
C       PI   = 3.14159
C       G    = gravitational acceleration (m/sec*sec)
C       VONK = von Karman constant
C       EPS  = convergence criterion (1% here) for Monin-Obukov length
C
      DATA CP/1004.0/, PI/3.14159/, GRAV/9.80655/, VONK/0.4/
      DATA EPS/0.01/
C
C.......................................................................
C---- Make first guess for iteration initialize PSIZL and PSIZOL to zero.
C     Initialize other parameters:
C        ITER = iteration number
C        LASTL =  previous value of the Monin-Obukhov length
C
      LASTL = 0.0
      PSIZL = 0.0
      PSIZOL = 0.0
      ITER = 1
C
C---- Calm conditions for convective boundary layer -
C     checked in the calling routine SUBR.USTARL

C---- Set up iteration loop over Monin-Obukhov length (XMONIN) and
C     friction velocity (USTAR).  NOTE: The first guess for the
C     iteration is with PSIZL and PSIZOL set equal to zero.
C     The next guess is made by reevaluating PSIZL and PSIZOL.

  100 USTARM = VONK * WSPEED(IHR) /
     &           ( ALOG( ANEMZD / Z0M ) - PSIZL + PSIZOL)
      XMOLM = -RHO(IHR) * CP * TEMP(IHR) * (USTARM**3) /
     &              ( VONK * GRAV * SHF(IHR) )

C---- Check loop and stop loop only if within 1% (EPS) of previous value.

      IF( ITER .GT. 1  .AND.
     &    (ABS(XMOLM-LASTL) .LT. ABS(EPS*XMOLM )) ) THEN
         GO TO 200
      ENDIF
C
C---- Save computed M-O length and calculate new MU, MU0,
C     PSIZL and PSIZOL values.
C
      LASTL=XMOLM

      MU = (1.0 - 16.0 * ( ANEMZD / XMOLM ) ) **0.25
      MU0 = (1.0 - 16.0  * ( Z0MEAS / XMOLM ) ) **0.25

      PSIZL = 2.0 * ALOG( (1.0 + MU) / 2.0) +
     &              ALOG( (1.0 + MU * MU) / 2.0) -
     &              2.0 * ATAN(MU) + PI/2.0

      PSIZOL = 2.0 * ALOG( (1.0 + MU0) / 2.0) +
     &               ALOG( (1.0 + MU0 * MU0) / 2.0 ) -
     &               2.0 * ATAN(MU0) + PI/2.0

      ITER = ITER+1
      GO TO 100

200   CONTINUE

      RETURN
      END
C

      SUBROUTINE USLNIT( IHR,ANEMZD,Z0M,USTARM,XMOLM )
C=====================================================================**
C          USLNIT Module of the PCRAMMET Meteorological Preprocessor
C
C   Purpose:  To calculate the friction velocity and surface heat flux
C             for the SBL at the measurement site.  THETA-STAR is first
C             estimated as a temperature scale for profiling.
C             A check is used to see if
C             if the solution to USTAR will be either real or complex.
C             If the solution is real, the computation for USTAR follows
C             Venkatram's solution.  If the solution is complex, a
C             linear interpolation, used by van Ulden and Holtslag, is
C             applied to find the solution.  To find SHF, USTAR is
C             back-substituted  to find the heat flux.  For strong winds
C             there is a limit for hflux (-64 W/M**2).
C
C             These computations are for the measurement site
C
C   Input:
C        IHR       Hour of day
C        ANEMZD    Anemometer height (adjusted for the
C                  displacement height, which is 0.0)
C        WSPEED    Wind speed
C        Z0M       Roughness length at the measurement site
C        FCOVER    Fractional cloud cover
C        TEMP      Temperature
C        RHO       Density
C        ELMINM    Minimum Obukhov length for stable conditions
C
C   Output:
C        USTARM    Friction velocity
C        XMOLM     Obukhov length
C
C   Called from: USTARL
C-----------------------------------------------------------------------

      INTEGER  IHR
      REAL     THSTAR,AA,BB,CC,CDN,UNOT,VONK,GRAV
      REAL     CP, CHEK, UCR, USTCR, BETAM, HLIM, XLIMIT
      REAL     Z0M, USTARM, XMOLM

      INCLUDE 'PCRAM.INC'

C     VONK = von Karman's constant
C     GRAV = Acceleration due to gravity

      DATA VONK/0.4/,  GRAV/9.80655/

C---- Here, BETAM is a constant used for profile relationships in the
C     SBL, CP is the specific heat capacity of dry air
      CP = 1004.0
      BETAM = 4.7

C---- Calm wind conditions for stable boundary layer.
C     already checked in the calling routine SUBR.USTARL

C---- Equations for CDN, THSTAR, UNOT

      CDN = VONK /( ALOG(ANEMZD/Z0M) )
      THSTAR = 0.09 * (1.0 - 0.5*( FCOVER(IHR)**2) )
      UNOT = SQRT( (BETAM * ANEMHT * GRAV * THSTAR)/TEMP(IHR) )

C---- Check to see if USTAR has a real or complex solution by
C     checking the square root part (CHEK) of the USTAR equation.

      CHEK = ( ( 2.0 * UNOT) / (SQRT( CDN ) * WSPEED(IHR) ) )**2

C---- If there is a complex part to USTAR, a subroutine is called that
C     uses VAN ULDEN AND HOLTSLAG linear interpolation method to
C     compute USTAR.  This interpolation is used when the variable
C     CHEK is greater than 1.0.

C---- If the solution to USTAR is real, go ahead with computation
C     of USTAR, SHF and XMONIN.  The solution is real if "CHEK" is
C     less than or equal to 1.0.  For the case of strong winds,
C     the product USTAR*THSTAR, and thus SHF, may become
C     unrealistically large.  To avoid this, a limit of -64 W/m**2
C     is set for heat flux (SHF)

      IF( CHEK.LE.1.0 )THEN
         USTARM = (CDN * WSPEED(IHR) / 2.0) * (1.0 + SQRT(1.0-CHEK))
      ELSE
         UCR = (2.0 * UNOT) / SQRT(CDN)
         USTCR = CDN * UCR / 2.0
         USTARM = USTCR * WSPEED(IHR) / UCR
      ENDIF

C---- Compare the heat flux (SHF) to the limiting value; if it is too
C     large, then recompute the friction velocity.

      HLIM = -64.0
      XLIMIT = -HLIM / (RHO(IHR)*CP)
      IF( (USTARM * THSTAR) .GT. XLIMIT )THEN
         AA = -CDN * WSPEED(IHR)
         BB = 0.0
         CC = BETAM * ANEMHT * GRAV * XLIMIT * CDN / TEMP(IHR)
         CALL CUBIC ( AA, BB, CC, USTARM )
         THSTAR = XLIMIT / USTARM
      ENDIF
C
C---- Calculate the M-O length and friction velocity;
C     check the value of the Monin-Obukov length against the user-
C     specified minimum, setting XMONIN to the minimum and recomputing
C     USTAR if necessary; compute the heat flux.
C
      XMOLM   = TEMP(IHR) * USTARM * USTARM / ( VONK * GRAV * THSTAR )
      IF( XMOLM .LT. ELMINM )THEN
         XMOLM = ELMINM
         USTARM = SQRT(XMOLM * VONK * GRAV * THSTAR / TEMP(IHR) )
      ENDIF

      SHF(IHR) = -THSTAR * RHO(IHR) * CP * USTARM

      RETURN
      END
C

      SUBROUTINE CUBIC( A,B,C,Z )
C=====================================================================**
C
C   Purpose:  To solve a cubic equation to get the proper solution for
C             the friction velocity (UST), where the cubic equation in
C             this case looks similar to: Z**3 + A*Z**2 + B*Z + C = 0.
C
C   Input:
C        A         Real      Coefficient of UST**2
C        B         Real      Coefficient of UST**1
C        C         Real      Coefficient of UST**0
C
C   Output:
C        Z         Real      New friction velocity
C
C   Called from: USLNIT
C-----------------------------------------------------------------------
C
C---- Solve for one root of the cubic equation.
C
       DOUBLE PRECISION CM,SGN,A3,AP,BP,AP3,TROOT,BP2,APP,BSV
       DOUBLE PRECISION ALPHA,TR,BPP,ONE
       REAL A,B,C,Z
       DATA ONE/1.0/

       A3=A/3.
       AP=B-A*A3
       BP=2.*A3**3-A3*B+C
       AP3=AP/3.
       BP2=BP/2.
       TROOT=BP2*BP2+AP3*AP3*AP3
       IF(TROOT.LE.0.0) GO TO 50
       TR=DSQRT(TROOT)
       APP=(-BP2+TR)**0.333333
       BSV=-BP2-TR
       IF(BSV.EQ.0.0)GO TO 45
       SGN=DSIGN(ONE,BSV)
       BPP=SGN*(DABS(BSV))**0.333333
       Z=APP+BPP-A3
       RETURN

45     CONTINUE
       Z=APP-A3
       RETURN
C
50     CM=2.*DSQRT(-AP3)
       ALPHA=DACOS(BP/(AP3*CM))/3.
       Z=CM*DCOS(ALPHA)-A3
       RETURN
       END


      SUBROUTINE USTAR2(ZREF,UREF, US1,OBL1, Z02,US2,OBL2,ELMINM)
C=====================================================================**
C
C     Purpose:  To compute the friction velocity and Monin-Obukhov length
C               at the application site using equations from Atmospheric
C               Environment Tech Note, 1979 P 191-194
C
C               An iterative process is used and the solution is found
C               when the two consecutive values of the friction velocity
C               are within 1% of each other
C
C      Input:   Wind speed, anemometer height, friction velocity and
C               Obukhov length for the meadurement site, minimum Obukhov
C               length for stable conditions, roughness length at the
C               application site
C
C      Output:  Friction velocity and Obukhov length at the application
C               site
C
C      Called from: USTARL
C-----------------------------------------------------------------------
C
      DATA VONKAR/0.4/

      ITER = 0
      USOLD = US1
      OBL = OBL1
10    CONTINUE

C     Check the number of interations; use the last guess if the
C     number exceeds 100
      ITER = ITER + 1
      IF( ITER .GT. 100) THEN
         US2 = USNEW
         OBL2 = OBL
         RETURN
      ENDIF

C     Compute the stability correction for similarity laws
      IF( OBL .GE. 0.0 )THEN
         PSI = 4.7*ZREF/OBL

      ELSE
         GM1 = (1.0 - 16.0*ZREF/OBL)**0.25
         GM2 = (1.0 - 16.0*Z02/OBL)**0.25
         TA  = (GM2*GM2+1)*(GM2+1)*(GM2+1)
         TB  = (GM1*GM1+1)*(GM1+1)*(GM1+1)
         T2  = TA/TB
         ARG2 = ALOG(T2)
         ARG3 = 2.0 * (ATAN(GM1) - ATAN(GM2) )
         PSI  = ARG2 + ARG3

      END IF

      USNEW  = SQRT( (VONKAR * UREF * US1 ) /( ALOG(ZREF/Z02) + PSI ) )

C     Iterate on friction velocity to account for new roughness length
      IF( ABS( (USNEW-USOLD)/USNEW) .LT. 0.01 )THEN
         US2 = USNEW
         OBL2 = OBL
         GO TO 100

      ELSE
         OBL = OBL*(USNEW*USNEW*USNEW)/(USOLD*USOLD*USOLD)
         IF( OBL .GT. 0.0  .AND.  OBL .LT. ELMINM )THEN
            OBL = ELMINM
         ELSEIF( OBL .LT. 0.0  .AND.  OBL .GT. -1.0 )THEN
            OBL = -1.0
         ENDIF
         USOLD = USNEW
         GO TO 10
      END IF

100   RETURN
      END
      SUBROUTINE READSF (NREAD)
C***********************************************************************
C*    READSF  :   A Subroutine for the PCRAMMET Meteorological
C*                Preprocessor for EPA Dispersion Models
C*
C*    PURPOSE:    Reads 24 hours of Surface Data from either CD144 or
C*                SAMSON format.
C*
C*    PROGRAMMER: Jayant Hardikar, Jim Paumier
C*                PES Inc.
C*
C*    DATE:       August 15, 1995
C*
C*    INPUTS:     File Types and Unit Numbers
C*
C*
C*    OUTPUTS:    Character array of hourly weather data
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

      IF (SFCTYP .EQ. 'CD144' ) THEN
C*       DATA TYPE IS CD144
         CALL CD144(INSFC, ACD144, EOFSFC, LWD, NREAD)

      ELSEIF (SFCTYP .EQ. 'SCRAM' ) THEN
C*       DATA TYPE IS SCRAM
         CALL SCRAM(INSFC, ACD144, EOFSFC, LWD, NREAD)

      ELSEIF (SFCTYP .EQ. 'SAMSON') THEN
C*       DATA TYPE IS SAMSON
         CALL SAMSON (NVARS,IDVAR,JSAMSN,MAXSAM,IDSFC,CITY,STATE,
     &           ITZONE,XLAT,XLON,ISELEV,INSFC,EOFSFC,NREAD,
     &           WETFLG,DRYFLG,PPTFLG)

      ELSEIF( SFCTYP .EQ. 'HUSWO' )THEN

C*       Data type is HUSWO (with assumed ENGLISH units)
         CALL HUSWO (NVARS,IFC,IDVAR,JHUSWO,MAXHUS,INSFC,EOFSFC,
     &                  NREAD,WETFLG,DRYFLG,PPTFLG)

      ENDIF

      RETURN
      END


      SUBROUTINE SAMSON (NVARS,IDVAR,JVALUE,MAXVAR,IWBAN,CITY,STATE,
     &                   ITZONE,XLAT,XLON,ISELEV,IUNIT,EOFSFC,NREAD,
     &                   WETFLG,DRYFLG,PPTFLG)
C***********************************************************************
C*    SAMSON Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Reads met data from a SAMSON met file
C*                in 24 hour blocks.  Returns a character array of up
C*                to 21 variables for 24 hours
C*
C*    PROGRAMMER: Jayant Hardikar, Jim Paumier
C*                PES Inc.
C*
C*    DATE:       August 15, 1995
C*
C*    INPUTS:     SAMSON met data file
C*
C*    OUTPUTS:    Arrays of Various Met Parameters for 24 hours
C*
C*    Modifications:
C*       7/31/98 - Added code to retrieve relative humidity and global
C*                 radiation from CD if dry or wet deposition selected
C*
C*    CALLED FROM: READSF
C***********************************************************************
C*
C*    Variable Declarations
C*    (ILINE is Used to Check If This Routine is Being Called
C*     for the Very First Time or Not)
      SAVE          ILINE

      CHARACTER*40  FIELD(26)          ! dimensioned MAXSAM
      CHARACTER*256 ALINE
      CHARACTER     CITY*22, STATE*2, NS*1, EW*1
      INTEGER       LATDEG,LATMIN,IWBAN,ITZONE
      INTEGER       LONDEG,LONMIN,ISELEV
      INTEGER       IDVAR(MAXVAR-5)
      CHARACTER*9   JVALUE(MAXVAR,24)
      CHARACTER*9   JTEMP(26)          ! dimensioned MAXSAM
      CHARACTER*2   VFMT(21)           ! dimensioned MAXSAM-5
      CHARACTER*180 PFMT
      REAL          XLAT,XLON
      LOGICAL       GOTCLD, GOTDBT, GOTWD, GOTWS, GOTCHT, GOTPWX, GOTPPT
      LOGICAL       EOFSFC, PPTFLG, WETFLG, DRYFLG, GOTGRAD, GOTRELH

      DATA ILINE /0/
      DATA VFMT /'A4','A4','A7','A7','A7','A2','A2','A5','A5','A3',
     &           'A4','A3','A5','A6','A6','A9','A4','A6','A4','A3',
     &           'A6'/


C*    First time reading the file, read the file headers
      IF (ILINE .EQ. 0) THEN
C*       READ STATION HEADER RECORD
         READ (IUNIT,6000,ERR=1035) IWBAN,CITY,STATE,ITZONE,NS,LATDEG,
     &                              LATMIN,EW,LONDEG,LONMIN,ISELEV

C*       CONVERT DEG/MIN TO DECIMAL DEGREES
C*       North and West = positive, South and East = negative
         XLAT = LATDEG + LATMIN/60.0
         IF(NS.EQ. 'S') XLAT=XLAT*(-1.0)
         XLON=LONDEG+LONMIN/60.0
         IF(EW.EQ. 'E') XLON=XLON*(-1.0)

C*       Convert the time zone so POSITIVE represents west longitudes
         ITZONE = -ITZONE

C*       READ EXTRACTED-VARIABLES HEADER RECORD
         READ (IUNIT,'(A)') ALINE

C*       PARSE THE LINE TO DETERMINE THE INDIVIDUAL VARIABLES
         CALL PARSER (ALINE,FIELD,IFC)

C*       DECREMENT THE COUNTER FOR NUMBER OF VARIABLES TO EXCLUDE
C*       YR,MO,DY,HR,I FIELDS
         NVARS = IFC-5

C*       DETERMINE THE FORMAT AND THE VARIABLES TO READ
         WRITE (PFMT(1:19),'(A)') "(4(1X,A2),1X,A1,1X,"
         IPOS = 19

         DO 200 IVAR = 1,NVARS

C*          GET THE VARIABLE ID
            CALL STONUM(FIELD(IVAR+5),40,FNUM,IMIT)
            IF (IMIT .NE. 1) THEN
               CALL ERRHDL('E','1036',1)

            ELSE
               IDVAR(IVAR) = INT(FNUM)
            END IF

            IPOS = IPOS+1
            WRITE (PFMT(IPOS:IPOS+1),'(A2)') VFMT(IDVAR(IVAR))
            IPOS = IPOS +2
            IF (IVAR .NE. NVARS) THEN
               WRITE (PFMT(IPOS:IPOS+3),'(A4)') ",1X,"
               IPOS = IPOS +3
            ELSE
              WRITE (PFMT(IPOS:IPOS),'(A1)') ")"
            ENDIF

200      CONTINUE

C        SET FLAG FOR FIRST-TIME READ
         ILINE = ILINE + 1

C        Check the ID numbers of the variables to be sure there
C        is sufficient data to perform the calculations
C        NVARS = # of variables retrieved from SAMSON CD
C        IDVAR = ID number

         DO 250 IV = 1,NVARS
            IF( IDVAR(IV) .EQ. 7 )THEN
               GOTCLD = .TRUE.
            ELSEIF( IDVAR(IV) .EQ. 8 )THEN
               GOTDBT = .TRUE.
            ELSEIF( IDVAR(IV) .EQ. 12 )THEN
               GOTWD = .TRUE.
            ELSEIF( IDVAR(IV) .EQ. 13 )THEN
               GOTWS = .TRUE.
            ELSEIF( IDVAR(IV) .EQ. 15 )THEN
               GOTCHT = .TRUE.
            ELSEIF( WETFLG  .AND.  .NOT. PPTFLG )THEN
               IF( IDVAR(IV) .EQ. 16 )THEN
                  GOTPWX = .TRUE.
               ELSEIF( IDVAR(IV) .EQ. 21 )THEN
                  GOTPPT = .TRUE.
               ENDIF
            ENDIF

C           Additional fields are required for deposition processing
C           3 = global radition; 10 = relative humidity
            IF( DRYFLG .OR. WETFLG )THEN
               IF( IDVAR(IV) .EQ. 3 )THEN
                  GOTGRAD = .TRUE.
               ELSEIF( IDVAR(IV) .EQ. 10 )THEN
                  GOTRELH = .TRUE.
               ENDIF
            ENDIF

  250    CONTINUE

C        Now check the logical variables to determine if there is
C        sufficient data to continue processing
         IF( .NOT. GOTCLD  .OR.  .NOT. GOTDBT  .OR.  .NOT. GOTCHT
     &       .OR.  .NOT. GOTWD   .OR.  .NOT. GOTWS) THEN
            CALL ERRHDL( 'E', '1120', 0 )
         ENDIF

         IF( WETFLG  .AND.  .NOT. PPTFLG )THEN
            IF( .NOT. GOTPWX  .OR.  .NOT. GOTPPT )THEN
               CALL ERRHDL( 'E', '1125', 0 )
            ENDIF
         ENDIF

         IF( WETFLG .OR. DRYFLG )THEN
            IF( .NOT. GOTGRAD  .OR.  .NOT. GOTRELH )THEN
               CALL ERRHDL( 'E', '1126', 0 )
            ENDIF
         ENDIF

C     Endif first time reading the file
      ENDIF

6000  FORMAT (T2,I5,T8,A22,T31,A2,T34,I3,T39,A1,T40,I2,T43,I2,
     &        T47,A1,T48,I3,T52,I2,T56,I4)

C*    For all calls to this routine, read data
C*    NREAD keeps track of the number of data records read

      NREAD = 0

C*    LOOP HOURS
      DO 700 IHR = 1,24

C*       READ DATA
         READ (IUNIT,PFMT,END=999,ERR=1037) (JTEMP(IV),IV=1,NVARS+5)
         NREAD = NREAD + 1

C*       ASSIGN THE DATA TO THE APPROPRIATE VARIABLE INDICES
         DO 300 II = 1,5
            JVALUE(II,IHR) = JTEMP(II)
300      CONTINUE

         DO 600 IALL = 6,MAXVAR
            DO 500 IV = 1,NVARS
               IF (IALL-5 .EQ. IDVAR(IV)) THEN
                  JVALUE(IALL,IHR) = JTEMP(IV+5)
                  GO TO 600
               ELSE
                  IF (IV .EQ. NVARS) THEN
                     JVALUE(IALL,IHR) = '   '
                  ENDIF
               ENDIF
500         CONTINUE
600      CONTINUE

700   CONTINUE
      GO TO 1000

C*    Error Handling
1035  CALL ERRHDL('E','1035',1)
1037  CALL ERRHDL('E','1037',1)
C     WRITE (*,*) 'ERROR GETTING VARIABLE ID', FIELD(IVAR+5)


999   EOFSFC = .TRUE.
1000  RETURN
      END


      SUBROUTINE HUSWO (NVARS,IFC,IDVAR,JVALUE,MAXVAR,IUNIT,EOFSFC,
     &                  NREAD,WETFLG,DRYFLG,PPTFLG)
C***********************************************************************
C*    HUSWO Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Reads met data from a HUSWO met file in 24-hour
C*                blocks.  Returns a character array of up to 20
C*                variables for 24 hours
C*
C*    PROGRAMMER: PES Inc.
C*
C*    DATE:       May 1998
C*
C*    INPUTS:     HUSWO met data file (data retrieved from CD)
C*
C*    OUTPUTS:    Arrays of Various Met Parameters for 24 hours
C*
C*    Modifications:
C*       7/31/98 - Added code to retrieve relative humidity and global
C*                 radiation from CD if dry or wet deposition selected
C*
C*
C*    CALLED FROM: READSF
C***********************************************************************

C*    Variable Declarations
C*    (ILINE is Used to Check If This Routine is Being Called
C*     for the Very First Time or Not)

      SAVE          ILINE

      CHARACTER*40  FIELD(24)          ! dimensioned MAXHUS
      CHARACTER*256 ALINE
      INTEGER       IDVAR(MAXVAR)
      CHARACTER*9   JVALUE(MAXVAR,24)
c     CHARACTER*9   JTEMP(24)          ! dimensioned MAXHUS
      CHARACTER*2   VFMT(24)           ! dimensioned MAXHUS
      CHARACTER*120  PFMT
      LOGICAL       GOTCLD, GOTDBT, GOTWD, GOTWS, GOTCHT, GOTPWX, GOTPPT
      LOGICAL       EOFSFC, PPTFLG, WETFLG, DRYFLG, GOTGRAD, GOTRELH

      DATA ILINE /0/
      DATA VFMT /'A5','A1','A4','A2','A2','A2','A4','A4','A2','A2',
     &           'A5','A6','A3','A4','A4','A4','A6','A5','A8','A5',
     &           'A5','A5','A4','A4'/
      DATA PFMT/' '/


C*    First time reading the file, read the file header
      IF (ILINE .EQ. 0) THEN
C*       READ EXTRACTED-VARIABLES HEADER RECORD
         READ (IUNIT,'(A)') ALINE

C*       PARSE THE LINE TO DETERMINE THE INDIVIDUAL VARIABLES
         CALL PARSER (ALINE,FIELD,IFC)

C*       DECREMENT THE COUNTER OF THE NUMBER OF FIELDS READ TO DETERMINE
C8       THE NUMBER OF WEATHER VARIABLES READ
C*       (STATION AND YEAR FIELDS ARE EXCLUDED)
         NVARS = IFC-2

C*       DETERMINE THE FORMAT AND THE VARIABLES TO READ
C        Format of the first 6 variables, which are always in the file;
C        Variables are: station, ASOS flag, 4-digit year, month, day, hour
         WRITE (PFMT(1:17),'(A)') "(A5,A1,A4,3A2,1X,"
         IPOS = 17

         DO 200 IVAR = 3,IFC

C*          GET THE VARIABLE ID FROM THE FIELD NUMBERS READ ON THE 1ST RECORD
            CALL STONUM(FIELD(IVAR),40,FNUM,IMIT)
            IF (IMIT .NE. 1) THEN
               CALL ERRHDL('E','1036',1)
            ELSE
               IDVAR(IVAR) = INT(FNUM)
            END IF

C           Set the pointer to the position just past the last comma
C           and write the format of the variable to the format statement
C           that will be used to read the data; since there are 4
C           unnumbered fields, the correct position in the VFMT array is
C           4 past the field number being processed
            IPOS = IPOS+1
            WRITE (PFMT(IPOS:IPOS+1),'(A2)') VFMT(IDVAR(IVAR)+4)
            IPOS = IPOS +2
            IF (IVAR .NE. IFC) THEN
               IF( IDVAR(IVAR) .NE. 5 )THEN
                  WRITE (PFMT(IPOS:IPOS+3),'(A4)') ",1X,"
                  IPOS = IPOS +3
               ELSE
                  WRITE(PFMT(IPOS:IPOS),'(A1)')  ","
               ENDIF
            ELSE
              WRITE (PFMT(IPOS:IPOS),'(A1)') ")"
            ENDIF

200      CONTINUE


C*       SET FLAG FOR FIRST-TIME READ
         ILINE = ILINE + 1

C*       Check the ID numbers of the variables to be sure there
C*       is sufficient data to perform the calculations

         DO 250 IV = 1,IFC
            IF( IDVAR(IV) .EQ. 5 .OR. IDVAR(IV) .EQ. 16 )THEN
C              Either total/opaque cloud cover is in the file and/or the
C              ASOS cloud layers are in the file
               GOTCLD = .TRUE.
            ELSEIF( IDVAR(IV) .EQ. 7 )THEN
C              Dry bulb temperature
               GOTDBT = .TRUE.
            ELSEIF( IDVAR(IV) .EQ. 11 )THEN
C              Wind direction
               GOTWD = .TRUE.
            ELSEIF( IDVAR(IV) .EQ. 12 )THEN
C              Wind speed
               GOTWS = .TRUE.
            ELSEIF( IDVAR(IV) .EQ. 14 )THEN
C              Ceiling height
               GOTCHT = .TRUE.
            ELSEIF( WETFLG  .AND.  .NOT. PPTFLG )THEN
               IF( IDVAR(IV) .EQ. 15 )THEN
C                 Present weather
                  GOTPWX = .TRUE.
               ELSEIF( IDVAR(IV) .EQ. 19 )THEN
C                 Precipitation
                  GOTPPT = .TRUE.
               ENDIF
            ENDIF

C           Additional fields are required for deposition processing
C           3 = global radition; 9 = relative humidity
            IF( DRYFLG .OR. WETFLG )THEN
               IF( IDVAR(IV) .EQ. 3 )THEN
                  GOTGRAD = .TRUE.
               ELSEIF( IDVAR(IV) .EQ. 9 )THEN
                  GOTRELH = .TRUE.
               ENDIF
            ENDIF

  250    CONTINUE

         IF( .NOT. GOTCLD  .OR.  .NOT. GOTDBT  .OR.  .NOT. GOTCHT
     &       .OR.  .NOT. GOTWD   .OR.  .NOT. GOTWS) THEN
            CALL ERRHDL( 'E', '1140', 0 )
         ENDIF
         IF( WETFLG  .AND.  .NOT. PPTFLG )THEN
            IF( .NOT. GOTPWX  .OR.  .NOT. GOTPPT )THEN
               CALL ERRHDL( 'E', '1145', 0 )
            ENDIF
         ENDIF

         IF( WETFLG .OR. DRYFLG )THEN
            IF( .NOT. GOTGRAD  .OR.  .NOT. GOTRELH )THEN
               CALL ERRHDL( 'E', '1126', 0 )
            ENDIF
         ENDIF

C     Endif first time reading the file
      ENDIF

C*    For all calls to this routine, read data
C*    NREAD keeps track of the number of data records read

      NREAD = 0

C*    LOOP HOURS
      DO 700 IHR = 1,24

C*       READ DATA; NOTE THAT THERE ARE 4 'UNNAMED' FIELDS ON THE 1ST
C        RECORD - ASOS FLAG, MONTH, DAY, AND HOUR - IN ADDITION TO THE
C        'IFC' NUMBERED FIELDS
         READ (IUNIT,PFMT,END=999,ERR=1045) (JVALUE(IV,IHR),IV=1,IFC+4)
         NREAD = NREAD + 1

700   CONTINUE

      GO TO 1000

C*    Error Handling (program stops in the error handler)

1045  CALL ERRHDL('E','1045',1)


999   EOFSFC = .TRUE.
1000  RETURN
      END


      SUBROUTINE CD144 (IUNIT,ADATA,EOFSFC,LWD,NREAD)
C***********************************************************************
C*    CD144 Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Reads met data from a CD144 met file
C*                in 24 hour blocks.  Returns a character array of
C*                13 variables for 24 hours
C*           ---  dgs (12 variables for 24 hours)
C*
C*    PROGRAMMER: Jayant Hardikar, Jim Paumier
C*                PES Inc.
C*
C*    DATE:       August 15, 1995
C*
C*    INPUTS:     CD144 met data file
C*
C*    OUTPUTS:    Arrays of Various Met Parameters for 24 hours
C*
C*    CALLED FROM: READSF
C***********************************************************************
c dgs ---  Add RH to data array (this shifts Opaque Sky Cover from
c          position 12 to 13)
      parameter(mx144=13)
C*
C*    Variable Declarations
C*    (ISF is Used to Check If This Routine is Being Called
C*    for the Very First Time or Not)
      SAVE ISF
      INTEGER IUNIT

c dgs CHARACTER*5 ADATA(12,24)
      CHARACTER*5 ADATA(mx144,24)

      LOGICAL EOFSFC

      DATA ISF /0/

C*    Skip the Very First Hour-00 (Belongs to Previous Day)
C*    However, keep the station ID and wind direction for calms processing
      ISF = ISF + 1
      IF (ISF .EQ. 1) THEN
         READ (IUNIT,410,ERR=1038) ADATA(1,1),LWD
      ENDIF
  410 FORMAT( A5,T39,I2 )

C     1 = Station ID
C     2 = Year
C     3 = Month
C     4 = Day
C     5 = Hour
C     6 = Ceiling ht
C     7 = Present weather
C     8 = Wind direction
C     9 = Wind speed
C     10 = Station pressure
C     11 = Dry bulb temperature
C     12 = Relative Humidity
C     13 = Opaque sky cover

      NREAD = 0

C*    Begin Processing With Hour 01
      DO 10 IHR = 1,24
c dgs    READ (IUNIT,420,END=999) (ADATA (I,IHR), I=1,12)
         READ (IUNIT,420,END=999) (ADATA (I,IHR), I=1,mx144)

         NREAD = NREAD + 1
   10 CONTINUE
c dgs 420 FORMAT (A5,4A2,A3,8X,A5,9X,2A2,A4,A3,T79,A1)
  420 FORMAT (A5,4A2,A3,8X,A5,9X,2A2,A4,A3,3x,a3,T79,A1)

      GO TO 1000

1038  CALL ERRHDL('E','1038',1)

999   EOFSFC = .TRUE.
      IF( IHR .EQ. 24 )THEN
C*       Duplicate last hour's data for hr=24
c dgs    DO 20 IVBL = 1,12
         DO 20 IVBL = 1,mx144
            ADATA(IVBL,24) = ADATA(IVBL,23)
   20    CONTINUE
         ADATA(5,24) = '24'
         NREAD = NREAD + 1
      ENDIF

1000  RETURN
      END


      SUBROUTINE SCRAM (IUNIT,ADATA,EOFSFC,LWD,NREAD)
C***********************************************************************
C*    SCRAM Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Reads met data from a SCRAM met file
C*                in 24 hour blocks.  Returns a character array of
C*                13 variables for 24 hours (with station pressure
C*                RH, and present weather as missing)
C*      ---  dgs  12 variables for 24 hours (with station pressure
C*      ---  dgs  and present weather as missing)
C*
C*    PROGRAMMER: Jayant Hardikar, Jim Paumier
C*                PES Inc.
C*
C*    DATE:       June 30, 1995
C*
C*    INPUTS:     SCRAM met data file
C*
C*    OUTPUTS:    Arrays of Weather Varaibles for 24 hours
C*
C*    CALLED FROM: READSF
C***********************************************************************
c dgs ---  Add RH to data array (this shifts Opaque Sky Cover from
c          position 12 to 13)
      parameter(mx144=13)
C*
C*    Variable Declarations
C*    (ISF is Used to Check If This Routine is Being Called
C*    for the Very First Time or Not)
      SAVE ISF
      INTEGER IUNIT

c dgs CHARACTER*5 ADATA(12,24)
      CHARACTER*5 ADATA(mx144,24)

      CHARACTER  SPEED*3, TOTAL*2, OPAQ*2
      LOGICAL EOFSFC

      DATA ISF /0/

C*    Skip the Very First Hour-00 (Belongs to Previous Day)
C*    However, keep the station ID and wind direction for calms processing
      ISF = ISF + 1
      IF (ISF .EQ. 1) THEN
         READ (IUNIT,410,ERR=1038) ADATA(1,1),LWD
      ENDIF
  410 FORMAT( A5,T17,I2 )

C     1 = Station ID
C     2 = Year
C     3 = Month
C     4 = Day
C     5 = Hour
C     6 = Ceiling ht
C     7 = Present weather (not available)
C     8 = Wind direction
C     9 = Wind speed
C     10 = Station pressure (not available)
C     11 = Dry bulb temperature
C     12 = Relative Humidity
C     13 = Opaque sky cover

      NREAD = 0

C*    Begin Processing With Hour 01
      DO 10 IHR = 1,24
         READ (IUNIT,420,END=999,ERR=1038) ADATA(1,IHR),ADATA(2,IHR),
     &         ADATA(3,IHR),ADATA(4,IHR),ADATA(5,IHR),ADATA(6,IHR),
     &         ADATA(8,IHR),SPEED,ADATA(11,IHR),TOTAL,OPAQ
         NREAD = NREAD + 1
C------- Set present weather (vbl 7) and stn pressure (vbl 10) to missing
         ADATA(7,IHR)  = '     '
         ADATA(10,IHR) = '    '

C dgs    Set Relative Humidity (vbl 12) to missing
         ADATA(12,IHR)  = '     '

C------- The SCRAM data are processed through the CD-144 routines;
C        therefore, the data first must be formatted after CD-144.
C        The program MET144 was used as a guide to developing the
C        conversion.

C------- Ceiling height, temperature and wind direction do not need
C        converting

C------- Wind Speed
         IF( SPEED .EQ. '   ' )THEN
            ADATA(9,IHR) = '     '

         ELSE
            READ( SPEED,'(I3)') IWS
            IF( IWS .LE. 99 )THEN
               WRITE( ADATA(9,IHR),'(A2)' ) SPEED(2:3)

            ELSE
               ADATA(9,IHR)(2:2) = SPEED(3:3)
               IF( IWS .LT. 200 )THEN
                  ADATA(9,IHR)(1:1) = 'R'

               ELSEIF( IWS .LT. 190 )THEN
                  ADATA(9,IHR)(1:1) = 'Q'

               ELSEIF( IWS .LT. 180 )THEN
                  ADATA(9,IHR)(1:1) = 'P'

               ELSEIF( IWS .LT. 170 )THEN
                  ADATA(9,IHR)(1:1) = 'O'

               ELSEIF( IWS .LT. 160 )THEN
                  ADATA(9,IHR)(1:1) = 'N'

               ELSEIF( IWS .LT. 150 )THEN
                  ADATA(9,IHR)(1:1) = 'M'

               ELSEIF( IWS .LT. 140 )THEN
                  ADATA(9,IHR)(1:1) = 'L'

               ELSEIF( IWS .LT. 130 )THEN
                  ADATA(9,IHR)(1:1) = 'K'

               ELSEIF( IWS .LT. 120 )THEN
                  ADATA(9,IHR)(1:1) = 'J'

               ELSEIF( IWS .LT. 110 )THEN
                  ADATA(9,IHR)(1:1) = '}'

               ENDIF
            ENDIF
         ENDIF

C----    Opaque cloud cover
         IF( OPAQ .NE. '  ') THEN
            IF( OPAQ .EQ. '10' )THEN
c dgs          ADATA(12,IHR) = '-'
               ADATA(13,IHR) = '-'
            ELSE
c dgs          WRITE( ADATA(12,IHR),'(A1)' ) OPAQ(2:2)
               WRITE( ADATA(13,IHR),'(A1)' ) OPAQ(2:2)
            ENDIF

         ELSE
            IF( TOTAL .NE. '  ' )THEN
               IF( TOTAL .EQ. '10' )THEN
c dgs             ADATA(12,IHR) = '-'
                  ADATA(13,IHR) = '-'
               ELSE
c dgs             WRITE( ADATA(12,IHR),'(A1)' ) TOTAL(2:2)
                  WRITE( ADATA(13,IHR),'(A1)' ) TOTAL(2:2)
               ENDIF

            ELSE
               IF( ADATA(6,IHR) .EQ. '   ' )THEN
c dgs             ADATA(12,IHR) = ' '
                  ADATA(13,IHR) = ' '

               ELSEIF( ADATA(6,IHR) .EQ. '---' )THEN
c dgs             ADATA(12,IHR) = '0'
                  ADATA(13,IHR) = '0'

               ELSEIF( ADATA(6,IHR) .LT. '070' )THEN
c dgs             ADATA(12,IHR) = '7'
                  ADATA(13,IHR) = '7'

               ELSE
c dgs             ADATA(12,IHR) = '0'
                  ADATA(13,IHR) = '0'

               ENDIF
            ENDIF
         ENDIF

   10 CONTINUE

  420 FORMAT (A5,4A2,A3,A2,A3,A3,a2,A2)

      GO TO 1000

1038  CALL ERRHDL('E','1038',1)

999   EOFSFC = .TRUE.
      IF( IHR .EQ. 24 )THEN
C*       Duplicate last hour's data for hr=24
c dgs    DO 20 IVBL = 1,12
         DO 20 IVBL = 1,mx144
            ADATA(IVBL,24) = ADATA(IVBL,23)
   20    CONTINUE
         ADATA(5,24) = '24'
         NREAD = NREAD + 1
      ENDIF

1000  RETURN
      END


      SUBROUTINE FILMET (NREAD)
C***********************************************************************
C*    FILMET Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Fills in the Appropriate Weather Variable Arrays,
C*                Translating Character to Numeric
C*
C*    PROGRAMMER: Jayant Hardikar, Jim Paumier
C*                PES Inc.
C*
C*    DATE:       August 15, 1995
C*
C*    INPUTS:     Surface and Precip Raw Data
C*                NREAD = # of hourly sfc obs read for the day
C*
C*    OUTPUTS:    24 Hour Arrays of Numeric Weather Data
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      CHARACTER*1  ASOSYN
      LOGICAL      ASOSNOW

      DATA ASOSNOW/.FALSE./

      DO 100 IH = 1,24

C*       Fill in Surface Variables
         IF (SFCTYP .EQ. 'SAMSON') THEN
C*          From SAMSON ...
            CALL FILSAM(IH)

         ELSEIF( SFCTYP .EQ. 'HUSWO' )THEN
C*          From HUSWO ...
            CALL FILHUS(IH, NREAD, ASOSYN, ASOS1, ASOS2, ASOS3)

C*          Since HUSWO will be the first to have ASOS obs. in a data
C           base, ASOS cloud reports need to be converted to the standard
C           format if the ASOS flag (ASOSYN) is equal to 'A'
            IF (ASOSYN .EQ. 'A' )THEN

C              Let the user know that ASOS data are now being processed
               IF( .NOT. ASOSNOW ) THEN
                  ASOSNOW = .TRUE.
                  WRITE( IDIAG,520 ) ISYR(IH),ISMO(IH),ISDY(IH),IH
               ENDIF
 
               CALL DOCLDS(ISYR(IH),ISMO(IH),ISDY(IH),IH, IDIAG,
     &                     ASOS1,ASOS2,ASOS3, ICC, ICHT)
               ICOVER(IH) = ICC
               IF( ICEIL(IH) .EQ. 9999 )THEN
                  ICEIL(IH)  = ICHT
               ENDIF
            ENDIF

         ELSEIF (SFCTYP .EQ. 'CD144'  .OR.  SFCTYP .EQ. 'SCRAM') THEN
C*          Or From CD144 or SCRAM
            CALL FIL144(IH)

         ENDIF


C*       Check the wind direction:
         IF( IDIR(IH) .NE. 0  .AND. IDIR(IH) .NE. 999 )THEN
C*          It is not a calm hour and not missing - save the direction
            LWD = IDIR(IH)
         ELSEIF( IDIR(IH) .EQ.  0 )THEN
C*          The direction is zero (calm hour), set the direction to the
C           previous valid wind direction
            IDIR(IH) = LWD
         ENDIF

C*       Fill in Hourly Precipitation Variable If Needed
         IF (WETFLG) THEN

C*          If the Surface Data is SAMSON
            IF (SFCTYP .EQ. 'SAMSON' .OR. SFCTYP .EQ. 'HUSWO' )THEN

C*             SAMSON/HUSWO Precip (in PRECIP(IHR)) May have been Read
C*             with surface variables

C*             If Supplementing with TD3240 Data (PPTFLG = .T.), Then
C*             Check to See if SAMSON Precip Was Missing.  If So, '
C*             Substitute With TD3240 If Non-Zero, Else Use SAMSON/HUSWO
               IF (PPTFLG) THEN
                  IF (PRECIP(IH) .GT. 9999.0) THEN
                     IF (P3240(IH) .GE. 0.0 )THEN
                        PRECIP(IH) = P3240(IH)
                        WRITE( IDIAG,500 ) ISYR(IH),ISMO(IH),ISDY(IH),IH
                        NOWARN = .FALSE.
                     ENDIF
                  ENDIF
               ENDIF

C*          Otherwise, if it is CD144 or SCRAM
            ELSE IF (SFCTYP .EQ. 'CD144' .OR. SFCTYP .EQ. 'SCRAM') THEN

C*             Get Precip Variable From 3240 Data
               PRECIP(IH) = P3240(IH)
            ENDIF

         ENDIF


100   CONTINUE

      RETURN

  500 FORMAT( ' FILMET: TD-3240 precip substituted for SAMSON/HUSWO',
     &        ' on (yy/mm/dd/hh): ', 4(i2.2:,'/') )
C 510 FORMAT( ' FILMET: All precip data missing for (yy/mm/dd/hh): ',
C    &         4(i2.2:,'/'),'; set to 0.0' )
  520 FORMAT( /,' ATTENTION: ASOS-derived observations begin',
     &        ' on (yy/mm/dd/hh): ', 3(i2.2,'/'),i2.2, / )
      END


      SUBROUTINE FILSAM(IHR)
C***********************************************************************
C*    FILSAM Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Translates the Weather Variables from Character to
C*                Numeric for SAMSON data
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 14, 1995
C*
C*    INPUTS:     Surface Data from SAMSON
C*
C*    OUTPUTS:    24 Hour Arrays of Numeric Weather Data
C*
C*    Modifications:
C*       7/31/98 - Added code to process relative humidity and global
C*                 radiation from CD if dry or wet deposition selected
C*
C*    CALLED FROM: FILMET
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      CHARACTER*6 WFMT(MAXSAM)
      CHARACTER*9 ATEMP

      data ircount/0/

      DATA WFMT /'(I2)','(I2)','(I2)','(I2)','(I1)',
     &           '(I4)','(I4)','(A7)','(A7)','(A7)','(I2)','(I2)',
     &           '(F5.0)','(F5.0)','(I3)','(I4)','(I3)','(F5.0)',
     &           '(F6.0)',
     &           '(I6)','(A9)','(I4)','(F6.0)','(I4)','(I3)','(I6)'/

C*    Obtain Date and Time Info
      READ (JSAMSN(1,IHR),WFMT(1)) ISYR(IHR)
      READ (JSAMSN(2,IHR),WFMT(2)) ISMO(IHR)
      READ (JSAMSN(3,IHR),WFMT(3)) ISDY(IHR)
      READ (JSAMSN(4,IHR),WFMT(4)) ISHR(IHR)


C*    Loop Over All the SAMSON Variables
      DO 600 IALL = 6,MAXSAM
         DO 300 IV = 1,NVARS

C*          If the Current Variable is Present, Then Process It
C*            As with CD-144 data, if the last hour of the last day
C*            of the year is missing, then substitute with data from
C*            hour 23

            IF (IALL-5 .NE. IDVAR(IV)) THEN
               IF (IV .EQ. NVARS) GO TO 600

            ELSE
C*------------ Opaque Sky Cover (tenths)
               IF (IDVAR(IV) .EQ. 7) THEN
                  READ (JSAMSN(IALL,IHR),WFMT(7+5)) ICOVER(IHR)
                  IF( ICOVER(IHR) .EQ. 99 ) THEN
                     IF( IHR .EQ. 24  .AND. ISMO(IHR) .EQ. 12  .AND.
     &                   ISDY(IHR).EQ.31 )THEN
                         ICOVER(IHR)  =  ICOVER(IHR-1)
                     ELSE
                         WRITE( IDIAG,540 ) ISYR(IHR), ISMO(IHR),
     &                                      ISDY(IHR), ISHR(IHR)
                         NOWARN = .FALSE.
                     ENDIF
                  ENDIF
               ENDIF

C*------------ Ceiling Height (meters)
               IF (IDVAR(IV) .EQ. 15) THEN
                  READ (JSAMSN(IALL,IHR),WFMT(15+5)) ICEIL(IHR)
                  IF( ICEIL(IHR) .GT. 999990 )THEN
                     IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                   ISDY(IHR).EQ.31 )THEN
                         ICEIL(IHR)  =  ICEIL(IHR-1)
                     ELSE
                        ICEIL(IHR) = 77777
                        WRITE( IDIAG,550 ) ISYR(IHR), ISMO(IHR),
     &                                     ISDY(IHR), ISHR(IHR)
                        NOWARN = .FALSE.
                     ENDIF
                  ENDIF
               ENDIF

C*------------ Pressure (millibars)
               IF (IDVAR(IV) .EQ. 11) THEN
                  READ (JSAMSN(IALL,IHR),WFMT(11+5)) IPRESS
                  PRESS(IHR) = IPRESS
                  IF( PRESS(IHR) .GT. 9990.0 )THEN
                     IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                   ISDY(IHR).EQ.31 )THEN
                         PRESS(IHR) =  PRESS(IHR-1)
                     ELSE
                        WRITE( IDIAG,520 ) ISYR(IHR), ISMO(IHR),
     &                                     ISDY(IHR), ISHR(IHR)
                        NOWARN = .FALSE.
                     ENDIF
                  ENDIF
               ENDIF

C*------------ Wind Direction (degrees)
               IF (IDVAR(IV) .EQ. 12) THEN
                  READ (JSAMSN(IALL,IHR),WFMT(12+5)) IDIR(IHR)
                  IF( IDIR(IHR) .EQ. 999 ) THEN
                     IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                   ISDY(IHR).EQ.31 )THEN
                         IDIR(IHR) =  IDIR(IHR-1)
                     ELSE
                        WRITE( IDIAG,500 ) ISYR(IHR), ISMO(IHR),
     &                                     ISDY(IHR), ISHR(IHR)
                        NOWARN = .FALSE.
                     ENDIF
                  ENDIF
               ENDIF

C*------------ Wind Speed (meters/second)
C*             (wind speed has two possible missing codes:
C*                         99 and 9999 - use 99 as the missing code)
               IF (IDVAR(IV) .EQ. 13) THEN
                  READ (JSAMSN(IALL,IHR),WFMT(13+5)) WSPEED(IHR)
                  IF( WSPEED(IHR) .GE. 99.0 ) THEN
                     IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                   ISDY(IHR).EQ.31 )THEN
                         WSPEED(IHR) =  WSPEED(IHR-1)
                     ELSE
                        WSPEED(IHR) = 99.0
                        WRITE( IDIAG,510 ) ISYR(IHR), ISMO(IHR),
     &                                     ISDY(IHR), ISHR(IHR)
                        NOWARN = .FALSE.
                     ENDIF
                  ENDIF
               ENDIF

C*------------ Temperature (degrees Celsius)
               IF (IDVAR(IV) .EQ. 8) THEN
                  READ (JSAMSN(IALL,IHR),WFMT(8+5)) TEMP(IHR)
                  IF( TEMP(IHR) .GT. 9990. )THEN
                     IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                   ISDY(IHR).EQ.31 )THEN
                         TEMP(IHR) =  TEMP(IHR-1)
                     ELSE
                        TEMP(IHR) = 999.9
                        WRITE( IDIAG,530 ) ISYR(IHR), ISMO(IHR),
     &                                     ISDY(IHR), ISHR(IHR)
                        NOWARN = .FALSE.
                     ENDIF
                  ENDIF
               ENDIF

C*------------ Present Weather Code
               IF (IDVAR(IV) .EQ. 16) THEN
                  READ (JSAMSN(IALL,IHR),WFMT(16+5)) ATEMP
                  DO 200 JJ = 1, 9
                     READ (ATEMP(JJ:JJ),'(I1)') IWXSAM(JJ,IHR)
200               CONTINUE
               ENDIF

C*             Hourly Precip (inches and hundredths)
C*             Convert precip to millimeters here rather than in S.UNITS
               IF (IDVAR(IV) .EQ. 21) THEN
                  READ (JSAMSN(IALL,IHR),WFMT(21+5)) IPCP
                  PRECIP(IHR) = IPCP
                  IF( PRECIP(IHR) .GT. 99990.0 ) THEN
                     IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                   ISDY(IHR).EQ.31 )THEN
                         PRECIP(IHR) =  PRECIP(IHR-1)
                     ELSE
                        WRITE( IDIAG,560 ) ISYR(IHR), ISMO(IHR),
     &                                     ISDY(IHR), ISHR(IHR)
                        NOWARN = .FALSE.
                     ENDIF
                  ENDIF
                  IF( PRECIP(IHR) .LE. 99990.0 ) THEN
                     PRECIP(IHR) = ( PRECIP(IHR) /100.0 ) * 25.4
                  ENDIF
               ENDIF

C              For dry and wet deposition, both relative humidity and 
C              GLOBAL radiation are required in the output file

               IF( DRYFLG  .OR.  WETFLG )THEN
C                 First, the relative humidity, variable 10 in SAMSON
                  IF( IDVAR(IV) .EQ. 10 )THEN
                     READ (JSAMSN(IALL,IHR),WFMT(10+5)) IRHUM(IHR)
                     IF( IRHUM(IHR) .GE. 990 ) THEN
                        IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                      ISDY(IHR).EQ.31 )THEN
                            IRHUM(IHR) =  IRHUM(IHR-1)
                        ELSE
                           IRHUM(IHR) = -99
                           WRITE( IDIAG,570 ) ISYR(IHR), ISMO(IHR),
     &                                        ISDY(IHR), ISHR(IHR)
                           NOWARN = .FALSE.
                        ENDIF
                     ENDIF
                  ENDIF
    
C                 Next, the global radiation, variable 3 in SAMSON
C                 Note that it is a 7-character field, but only the
C                   first 4 characters (all integers) are needed; the
C                   last 3 characters are source and uncertainty flags

                  IF( IDVAR(IV) .EQ. 3 )THEN
c                    READ (JSAMSN(IALL,IHR)(1:4),WFMT(3+5)) IGRAD(IHR)
                     READ (JSAMSN(IALL,IHR)(1:4),'(I4)') IGRAD(IHR)
                     IF( IGRAD(IHR) .GE. 9990 ) THEN
                        IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                      ISDY(IHR).EQ.31 )THEN
                            IGRAD(IHR) =  IGRAD(IHR-1)
                        ELSE
                           IGRAD(IHR) = 9999
                           WRITE( IDIAG,580 ) ISYR(IHR), ISMO(IHR),
     &                                        ISDY(IHR), ISHR(IHR)
                           NOWARN = .FALSE.
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDIF

C*       End Loops Over Variables
300      CONTINUE
600   CONTINUE

      RETURN

  500 FORMAT( ' FILSAM: Wind direction missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  510 FORMAT( ' FILSAM: Wind speed missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  520 FORMAT( ' FILSAM: Station pressure missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  530 FORMAT( ' FILSAM: Temperature missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  540 FORMAT( ' FILSAM: Cloud cover missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  550 FORMAT( ' FILSAM: Ceiling height missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  560 FORMAT( ' FILSAM: Precipitation missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  570 FORMAT( ' FILSAM: Dry/Wet dep: SAMSON rel. humidity missing for',
     &        ' (yy/mm/dd/hh) ', 4(i2.2:,'/') )
c dgs  580 FORMAT( ' FILSAM: Dry/Wet dep: SAMSON global rad''n missing for',
c dgs     &        ' (yy/mm/dd/hh) ', 4(i2.2:,'/') )
  580 FORMAT( ' FILSAM: Dry/Wet dep: SAMSON global radn estimated for',
     &        ' (yy/mm/dd/hh) ', 4(i2.2:,'/') )

      END

      SUBROUTINE FILHUS(IHR, NREAD, ASOSFL,ASOS1,ASOS2,ASOS3)
C***********************************************************************
C*    FILSAM Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Translates the Weather Variables from Character to
C*                Numeric for HUSWO data
C*
C*    PROGRAMMER: PES Inc.
C*
C*    DATE:       May 1998
C*
C*    INPUTS:     Surface Data from HUSWO
C*
C*    OUTPUTS:    24 Hour Arrays of Numeric Weather Data
C*
C*    Modifications:
C*       7/31/98 - Added code to process relative humidity and global
C*                 radiation from CD if dry or wet deposition selected
C*
C*    CALLED FROM: FILMET
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      INTEGER     ASOS1, ASOS2, ASOS3
      CHARACTER*8 ATEMP
      CHARACTER*6 WFMT(MAXHUS)
      CHARACTER*1 ASOSFL

      DATA WFMT /'(I5)','(A1)','(I4)','(I2)',  '(I2)',  '(I2)','(I4)',
     &           '(I4)','(I2)','(I2)','(F5.0)','(F6.0)','(I3)','(I4)',
     &           '(I4)','(F4.0)','(F6.0)','(I5)','(A8)','(I5)',
     &           '(I5)','(I5)','(I4)','(I4)'/

C*    Initialize the ASOS flag to a blank (indicating NO ASOS data) and
C     the ASOS variables
      ASOSFL = ' '
      ASOS1 = 99999
      ASOS2 = 99999
      ASOS3 = 99999

C*    Obtain Station ID, ASOS Flag, Date and Time Info

      READ (JHUSWO(1,IHR),WFMT(1)) IDSFC          !STNID
      READ (JHUSWO(2,IHR),WFMT(2)) ASOSFL
      READ (JHUSWO(3,IHR),WFMT(3)) ISYR(IHR)
      READ (JHUSWO(4,IHR),WFMT(4)) ISMO(IHR)
      READ (JHUSWO(5,IHR),WFMT(5)) ISDY(IHR)
      READ (JHUSWO(6,IHR),WFMT(6)) ISHR(IHR)

C*    HUSWO format uses 4-digit year; convert to 2-digit year
      IF( ISYR(IHR) .GE. 1900  .AND.  ISYR(IHR) .LT. 2000 )THEN
         ISYR(IHR) = ISYR(IHR) - 1900
      ELSEIF( ISYR(IHR) .GE. 2000 )THEN
         ISYR(IHR) = ISYR(IHR) - 2000
      ENDIF


C*    Loop Over All the HUSWO variables
      DO 300 IV = 1,IFC
C*       If the Current Variable is Present, Then Process It

C*------ Opaque Sky Cover (tenths)
         IF (IDVAR(IV) .EQ. 6) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) ICOVER(IHR)
            IF( ICOVER(IHR) .EQ. 99 ) THEN
               IF( ASOSFL .EQ. ' ' )THEN
                  IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                ISDY(IHR).EQ.31 )THEN
                      ICOVER(IHR)  =  ICOVER(IHR-1)
                  ELSE
                      WRITE( IDIAG,540 ) ISYR(IHR), ISMO(IHR),
     &                                   ISDY(IHR), ISHR(IHR)
                      NOWARN = .FALSE.
                  ENDIF
               ENDIF
            ENDIF
         ENDIF

C*------ ASOS Cloud Cover and Ceiling (tenths and hundreds of feet)
C        IF field 16 is present, then fields 17 and 18 are also present

         IF (IDVAR(IV) .EQ. 16) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) ASOS1
            READ (JHUSWO(IV+5,IHR),WFMT(IDVAR(IV)+5)) ASOS2
            READ (JHUSWO(IV+6,IHR),WFMT(IDVAR(IV)+6)) ASOS3

            IF( ASOSFL .EQ. 'A'  )THEN
C*             Save the ASOS data for use as last hour of year
               SAVAS1 = ASOS1
               SAVAS2 = ASOS2
               SAVAS3 = ASOS3
               IF( ASOS1 .EQ. 99999 )THEN
                  IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                ISDY(IHR).EQ.31 )THEN
                      ASOS1  =  SAVAS1
                      ASOS2  =  SAVAS2
                      ASOS3  =  SAVAS3
                  ELSE
C                     The next message has been commented out since the
C                     subroutine DOCLDS, called from FILMET, will
C                     issue an identical message
c                     WRITE( IDIAG,545 ) ISYR(IHR), ISMO(IHR),
c    &                                   ISDY(IHR), ISHR(IHR)
                      NOWARN = .FALSE.
                  ENDIF
               ENDIF
            ENDIF
         ENDIF

C*------ Ceiling Height (feet)
         IF (IDVAR(IV) .EQ. 14) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) ICEIL(IHR)
            IF( ICEIL(IHR) .GT. 99990 )THEN
               IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &             ISDY(IHR).EQ.31 )THEN
                   ICEIL(IHR)  =  ICEIL(IHR-1)
               ELSE
                  ICEIL(IHR) = 77777
                  WRITE( IDIAG,550 ) ISYR(IHR), ISMO(IHR),
     &                               ISDY(IHR), ISHR(IHR)
                  NOWARN = .FALSE.
               ENDIF
            ENDIF
         ENDIF

C*------ Pressure (hundredths of inches)
         IF (IDVAR(IV) .EQ. 10) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) IPRESS
            PRESS(IHR) = IPRESS
            IF( PRESS(IHR) .GT. 9990.0 )THEN
               IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &             ISDY(IHR).EQ.31 )THEN
                   PRESS(IHR) =  PRESS(IHR-1)
               ELSE
                  WRITE( IDIAG,520 ) ISYR(IHR), ISMO(IHR),
     &                               ISDY(IHR), ISHR(IHR)
                  NOWARN = .FALSE.
               ENDIF
            ENDIF
         ENDIF

C*------ Wind Direction (degrees)
         IF (IDVAR(IV) .EQ. 11) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) IDIR(IHR)
            IF( IDIR(IHR) .EQ. 999 ) THEN
               IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &             ISDY(IHR).EQ.31 )THEN
                   IDIR(IHR) =  IDIR(IHR-1)
               ELSE
                  WRITE( IDIAG,500 ) ISYR(IHR), ISMO(IHR),
     &                               ISDY(IHR), ISHR(IHR)
                  NOWARN = .FALSE.
               ENDIF
            ENDIF
         ENDIF

C*------ Wind Speed (miles per hour)
         IF (IDVAR(IV) .EQ. 12) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) WSPEED(IHR)
            IF( WSPEED(IHR) .GT. 99.0 ) THEN
               IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &             ISDY(IHR).EQ.31 )THEN
                   WSPEED(IHR) =  WSPEED(IHR-1)
               ELSE
                  WSPEED(IHR) = 99.0
                  WRITE( IDIAG,510 ) ISYR(IHR), ISMO(IHR),
     &                               ISDY(IHR), ISHR(IHR)
                  NOWARN = .FALSE.
               ENDIF
            ENDIF
         ENDIF

C*------ Temperature (degrees Fahrenheit)
         IF (IDVAR(IV) .EQ. 7) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) TEMP(IHR)
            IF( TEMP(IHR) .GT. 990. )THEN
               IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &             ISDY(IHR).EQ.31 )THEN
                   TEMP(IHR) =  TEMP(IHR-1)
               ELSE
                  TEMP(IHR) = 999.9
                  WRITE( IDIAG,530 ) ISYR(IHR), ISMO(IHR),
     &                               ISDY(IHR), ISHR(IHR)
                  NOWARN = .FALSE.
               ENDIF
            ENDIF
         ENDIF

C*------ Present Weather Code
         IF (IDVAR(IV) .EQ. 15) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) ATEMP
            DO 200 JJ = 1, 4
               READ (ATEMP(2*JJ-1:2*JJ),'(I2)') IWXHUS(JJ,IHR)
200         CONTINUE
         ENDIF

C*       Hourly Precip (inches and hundredths)
C*       Convert precip to millimeters here rather than in S.UNITS
         IF (IDVAR(IV) .EQ. 19) THEN
            READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) IPCP
            IF( IPCP .GT. 990.0 ) THEN
               IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &             ISDY(IHR).EQ.31 )THEN
                   PRECIP(IHR) =  PRECIP(IHR-1)
               ELSE
                  WRITE( IDIAG,560 ) ISYR(IHR), ISMO(IHR),
     &                               ISDY(IHR), ISHR(IHR)
                  NOWARN = .FALSE.
               ENDIF
            ENDIF 

            IF( IPCP .LE. 990.0 ) THEN
               PRECIP(IHR) = ( IPCP /100.0 ) * 25.4
            ENDIF
         ENDIF

C        For dry and wet deposition, both relative humidity and 
C        GLOBAL radiation are required in the output file

         IF( DRYFLG  .OR.  WETFLG )THEN
C           First, the relative humidity, variable 10 in SAMSON
            IF( IDVAR(IV) .EQ. 9 )THEN
              READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) IRHUM(IHR)
               IF( IRHUM(IHR) .GE. 990 ) THEN
                  IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                ISDY(IHR).EQ.31 )THEN
                      IRHUM(IHR) =  IRHUM(IHR-1)
                  ELSE
                     IRHUM(IHR) = -99
                     WRITE( IDIAG,570 ) ISYR(IHR), ISMO(IHR),
     &                                  ISDY(IHR), ISHR(IHR)
                     NOWARN = .FALSE.
                  ENDIF
               ENDIF
            ENDIF
    
C           Next, the global radiation, variable 3 in SAMSON

            IF( IDVAR(IV) .EQ. 3 )THEN
              READ (JHUSWO(IV+4,IHR),WFMT(IDVAR(IV)+4)) IGRAD(IHR)
               IF( IGRAD(IHR) .GE. 9990 ) THEN
                  IF( IHR .EQ. 24  .AND. ISMO(IHR).EQ.12  .AND.
     &                ISDY(IHR).EQ.31 )THEN
                      IGRAD(IHR) =  IGRAD(IHR-1)
                  ELSE
                     IGRAD(IHR) = 9999
                     WRITE( IDIAG,580 ) ISYR(IHR), ISMO(IHR),
     &                                  ISDY(IHR), ISHR(IHR)
                     NOWARN = .FALSE.
                  ENDIF
               ENDIF
            ENDIF
         ENDIF

C*    End Loops Over Variables
300   CONTINUE

      RETURN

  500 FORMAT( ' FILHUS: Wind direction missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  510 FORMAT( ' FILHUS: Wind speed missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  520 FORMAT( ' FILHUS: Station pressure missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  530 FORMAT( ' FILHUS: Temperature missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  540 FORMAT( ' FILHUS: Cloud cover missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  545 FORMAT( ' FILHUS: ASOS cloud data missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  550 FORMAT( ' FILHUS: Ceiling height missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  560 FORMAT( ' FILHUS: Precipitation missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  570 FORMAT( ' FILSHUS: Dry/Wet dep: HUSWO rel. humidity missing for',
     &        ' (yy/mm/dd/hh) ', 4(i2.2:,'/') )
c dgs  580 FORMAT( ' FILSHUS: Dry/Wet dep: HUSWO global rad''n missing for',
c dgs     &        ' (yy/mm/dd/hh) ', 4(i2.2:,'/') )
  580 FORMAT( ' FILSHUS: Dry/Wet dep: HUSWO global radn estimated for',
     &        ' (yy/mm/dd/hh) ', 4(i2.2:,'/') )

      END


      SUBROUTINE FIL144(IHR)
C***********************************************************************
C*    FIL144 Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Translates the Character Weather Variables to
C*                Numeric for CD-144 or SCRAM
c        dgs ---  Add Relative Humidity
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 14, 1995
C*
C*    INPUTS:     Surface Data from CD144
C*
C*    OUTPUTS:    1 Hour of Numeric Weather Data
C*
C*    CALLED FROM: FILMET
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      CHARACTER*5 CCC,DDD
      INTEGER     PCOUNT

      DATA PCOUNT /0/

C     1 = Station ID
C     2 = Year
C     3 = Month
C     4 = Day
C     5 = Hour
C     6 = Ceiling ht
C     7 = Present weather (missing in SCRAM)
C     8 = Wind direction
C     9 = Wind speed
C     10 = Station pressure (missing in SCRAM)
C     11 = Dry bulb temperature
c dgs ---
C     12 = Relative Humidity
C     13 = Opaque sky cover

C*    Obtain Station ID, Date and Time Info
      READ (ACD144(1,IHR),'(I5)') IDSFC
      READ (ACD144(2,IHR),'(I2)') IY
      READ (ACD144(3,IHR),'(I2)') IM
      READ (ACD144(4,IHR),'(I2)') ID
      READ (ACD144(5,IHR),'(I2)') IH

C*    Change Hour Zero to Hour 24 of Previous Day
      IF (IH .EQ. 0) THEN
         CALL JULIAN (IY,IM,ID,JD,0)
         CALL HR0024(IY,JD,IH)
         CALL JULIAN (IY,IM,ID,JD,1)
      ENDIF

C*    Save Date/Time Info In Appropriate Arrays
      ISYR(IHR) = IY
      ISMO(IHR) = IM
      ISDY(IHR) = ID
      ISHR(IHR) = IH

C*--- Decode Ceiling Height - Standard RAMMET Procedures
      CCC = ACD144(6,IHR)
c dgs DDD = ACD144(12,IHR)
      DDD = ACD144(13,IHR)
      CALL CCCODE(IDIAG,CCC,DDD,ICOVER(IHR),ICEIL(IHR),NOWARN)

C*    Save Other Variables in Appropriate Arrays

C*--- Present weather: only the 5 fields, in cols. 25-29 for CD-144,
C                      for liquid and frozen precipitation;
C                      for SCRAM data, these fields are blanks

      DO 100 J=1,5
         READ(ACD144(7,IHR)(J:J), '(I1)' ) IPREC(J,IHR)
  100 CONTINUE

C*    Missing value codes correspond to the codes used in SAMSON data

C*--- Wind direction
      IF( ACD144(8,IHR) .NE. '  ' )THEN
         READ (ACD144(8,IHR),'(I2)')   IDIR(IHR)
      ELSE
         WRITE( IDIAG, 500 ) IY, IM, ID, IH
         IDIR(IHR) = 999
         NOWARN = .FALSE.
      ENDIF

C*--- Wind speed
      IF( ACD144(9,IHR) .NE. '  ' )THEN
         READ (ACD144(9,IHR),'(f2.0)') WSPEED(IHR)
      ELSE
         WRITE( IDIAG, 510 ) IY, IM, ID, IH
         WSPEED(IHR) = 99.0
         NOWARN = .FALSE.
      ENDIF

C*--- Station pressure: needed only for dry or wet deposition
      IF( DRYFLG  .OR.  WETFLG )THEN
         IF( ACD144(10,IHR) .NE. '    ' )THEN
            READ (ACD144(10,IHR),'(f4.2)')PRESS(IHR)
         ELSE
C*          In SUBR.UNITS, the station pressure will be set to 1000mb
C           when the pressure is missing; the message is written only
C           for the first 500 occurrences of missing pressure
            PCOUNT = PCOUNT + 1
            IF( PCOUNT .LE. 500 )THEN
               WRITE( IDIAG, 520 ) IY, IM, ID, IH
            ENDIF
            PRESS(IHR) = 9999.0
            NOWARN = .FALSE.
         ENDIF
      ENDIF

C*--- Temperature
      IF( ACD144(11,IHR) .NE. '   ' )THEN
         READ (ACD144(11,IHR),'(F3.0)')  TEMP(IHR)
      ELSE
         WRITE( IDIAG, 530 ) IY, IM, ID, IH
         TEMP(IHR) = 999.9
         NOWARN = .FALSE.
      ENDIF

c dgs ----------------------
C*--- Relative Humidity: signal output by dry or wet deposition flag
      IF( DRYFLG  .OR.  WETFLG )THEN
         IF( ACD144(12,IHR) .NE. '    ' )THEN
            READ (ACD144(12,IHR),'(i3)')IRHUM(IHR)
         ELSE
            WRITE( IDIAG, 540 ) IY, IM, ID, IH
            IRHUM(IHR) = -99
            NOWARN = .FALSE.
         ENDIF
      ENDIF

      RETURN

  500 FORMAT( ' FIL144: Wind direction missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  510 FORMAT( ' FIL144: Wind speed missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
  520 FORMAT( ' FIL144: Station pressure missing for (yy/mm/dd/hh) ',
     &         3(i2.2:,'/'),i2.2, ', using 1000 mb')
  530 FORMAT( ' FIL144: Temperature missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )

c dgs ---------------
  540 FORMAT( ' FIL144: Relative Humidity missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )

      END


      SUBROUTINE CCCODE(IDIAG,CCC,DDD,ISKY,IROOF,NOWARN)
C***********************************************************************
C*    CCCODE Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Decodes the Ceiling Height and Cloud Cover from the
C*                CD144 or SCRAM Data
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:     Surface Data from CD144 or SCRAM
C*
C*    OUTPUTS:    1 Hour Numeric Value for Ceiling and Cloud Cover
C*
C*    CALLED FROM: FIL144
C***********************************************************************
C*
C*    Variable Declarations
      CHARACTER*1 DIG(11), CTMP(3), CC
      CHARACTER*5 CCC,DDD
      INTEGER IDG(3)
      LOGICAL NOWARN
      DATA DIG /'0','1','2','3','4','5','6','7','8','9','-'/

C*    Decode Cloud Cover - if it is missing (i.e., blank) or any invalid
C*    character, set to 10/10
      READ (DDD(1:1),'(A1)') CC

80    DO 90 JK=1,11
         IF (CC .EQ. DIG(JK)) GO TO 100
90    CONTINUE
      JK=11
      WRITE (IDIAG,530) CC
530   FORMAT (' CCCODE: The character (',A1,') is not allowable.',
     &        ' Cloud cover defaults to 10.')
      NOWARN = .FALSE.
100   ISKY=JK-1


C*    Decode Ceiling Height
      IF( CCC(1:1) .EQ. '-'  .OR.  CCC(1:3) .EQ. '   ' )THEN
         IDG(1)=9
         IDG(2)=9
         IDG(3)=8

      ELSE
         DO 20 JJ= 1,3
            READ (CCC(JJ:JJ),'(A1)') CTMP(JJ)
20       CONTINUE

110      DO 140 JI=1,3
            DO 120 JK=1,10
               IF (CTMP(JI).EQ.DIG(JK)) GO TO 130
120         CONTINUE
130         IDG(JI)=JK-1
140      CONTINUE
      ENDIF

C*    Compute ceiling height in hundreds of feet.
150   IROOF = IDG(1)*100 + IDG(2)*10 + IDG(3)

      RETURN
      END


      SUBROUTINE DOCLDS(IY,IM,ID,IH,IDIAG,ASOS1,ASOS2,ASOS3,ICCVR,ICEIL)
C***********************************************************************
C*    DOCLDS Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Decodes the ASOS Cloud Data
C*
C*    PROGRAMMER: PES Inc.
C*
C*    DATE:       May 1998
C*
C*    INPUTS:     3 levels of ASOS cloud data of the form XXYYY
C*                where XX = cloud coverage, YYY = ceiling in hundreds
C*                of feet
C*
C*    OUTPUTS:    1 Hour Numeric Value for Ceiling and Cloud Cover
C*
C*    CALLED FROM: FILMET
C***********************************************************************
C*
C*    Variable Declarations
      INTEGER  ACC1,ACC2,ACC3,ACHT1,ACHT2,ACHT3,ASOS1,ASOS2,ASOS3
      INTEGER  MAXCLD, ICCVR, ICEIL

      MAXCLD = 99
      ICCVR  = 99
      ICEIL  = 77777


      IF( ASOS1 .NE. 99999 )THEN
         ACC1  = INT(ASOS1/1000.0)
         ACHT1 = ASOS1 - ACC1*1000
         MAXCLD = ACC1
         IF( MAXCLD .GT. 02 )THEN
            ICEIL = ACHT1
         ENDIF
      ENDIF

      IF( ASOS2 .NE. 99999 )THEN
         ACC2  = INT(ASOS2/1000.0)
         ACHT2 = ASOS2 - ACC2*1000
         IF( ACC2 .NE. 99  .AND.  ACC2 .GT. MAXCLD )THEN
            MAXCLD = ACC2
            IF( MAXCLD .GT. 02 .AND. ICEIL .NE. 77777 )THEN
               ICEIL = ACHT2
            ENDIF
         ENDIF

      ENDIF

      IF( ASOS3 .NE. 99999 )THEN
         ACC3  = INT(ASOS3/1000.0)
         ACHT3 = ASOS3 - ACC3*1000
         IF( ACC3 .NE. 99  .AND.  ACC3 .GT. MAXCLD )THEN
            MAXCLD = ACC3
            IF( MAXCLD .GT. 02 .AND. ICEIL .NE. 77777 )THEN
               ICEIL = ACHT3
            ENDIF
         ENDIF
      ENDIF


C---- Convert the ASOS sky condition codes to  fractional cloud cover
C     (tenths).  Note that the VALID codes are 0, 2, 4, 6, 7, 9, and 99.
C     The remaining codes (1, 3, 5, 8, and 10) are not valid according to
C     the HUSWO documentation, but are allowed to pass through as 1/10,
C     3/10, etc. (per conversation with Dennis Atkinson, USEPA, 10/30/98).
C     These codes _may_ be augmented by human observers or errors, but 
C     there is no way to determine this.

      IF( MAXCLD .EQ. 0 )THEN
C*       A valid ASOS Sky Condition Value
         ICCVR = 0
      ELSEIF( MAXCLD .EQ. 01 )THEN
C*       Not a valid ASOS Sky Condition Value
         ICCVR = 1
      ELSEIF( MAXCLD .EQ. 02 )THEN
C*       A valid ASOS Sky Condition Value
         ICCVR = 3
      ELSEIF( MAXCLD .EQ. 03 )THEN
C*       Not a valid ASOS Sky Condition Value
         ICCVR = 3
      ELSEIF( MAXCLD .EQ. 04 )THEN
C*       A valid ASOS Sky Condition Value
         ICCVR = 7
      ELSEIF( MAXCLD .EQ. 05 )THEN
C*       Not a valid ASOS Sky Condition Value
         ICCVR = 5
      ELSEIF( MAXCLD .EQ. 06 )THEN
C*       A valid ASOS Sky Condition Value
         ICCVR = 10
      ELSEIF( MAXCLD .EQ. 07 )THEN
C*       A valid ASOS Sky Condition Value
         ICCVR = 10
      ELSEIF( MAXCLD .EQ. 08 )THEN
C*       Not a valid ASOS Sky Condition Value
         ICCVR = 8
      ELSEIF( MAXCLD .EQ. 09 )THEN
C*       A valid ASOS Sky Condition Value
         ICCVR = 99
         WRITE( IDIAG, 500 ) IY, IM, ID, IH
      ELSEIF( MAXCLD .EQ. 10 )THEN
C*       A valid ASOS Sky Condition Value
         ICCVR = 10
      ELSEIF( MAXCLD .EQ. 99 )THEN
         ICCVR = 99
         WRITE( IDIAG, 510 ) IY, IM, ID, IH

      ENDIF

500   FORMAT( ' DOCLDS: ASOS cloud cover unknown for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )
510   FORMAT( ' DOCLDS: ASOS cloud data missing for (yy/mm/dd/hh) ',
     &         4(i2.2:,'/') )


      RETURN
      END

      SUBROUTINE PCODES(IDIAG,IDATE,IPREC,IPCODE,NOWARN,SFTYPE)
C***********************************************************************
C*    PCODES Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Convert CD144 precipitation type data to two-digit
C*                precipitation codes; SCRAM has blanks (zeroes) for
C*                these fields
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*                Adapted From DEPMET Version: 1.0 Level: 931108
C*                                           : J. Chang, SRC
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:
C*            IDATE - integer    - Date for this hour YYJJJHH
C*            IPREC - int. array - Precipitation TYPE data
C*
C*    OUTPUTS:
C*           IPCODE - integer    - Two-digit precipitation CODE
C*
C*    REFERENCE: Press W.H. et al., 1986: Numerical Recipes, Cambridge
C*               Univ. Press, 818pp.
C*
C*    CALLED FROM: MAIN program
C***********************************************************************

C*    VARIABLE DECLARATIONS
      INTEGER      IDATE,IPREC(5),IPCODE
      CHARACTER*6  SFTYPE
      LOGICAL      NOWARN

      KT = IPREC(1) + IPREC(2) + IPREC(3) + IPREC(4) + IPREC(5)
      IF(KT.EQ.0) THEN
        IPCODE = 0
        GO TO 99
      ELSE
        ILIQ = 0
        IFROZ = 0
        DO 5 I = 5,1,-1
          IP = IPREC(I)
          IF(IP.EQ.0) GO TO 5
C*---     IPCODE IS THE PRECIPITATION CODE (0-45)
          IPCODE = (I-1) * 9 + IP
          IF(IPCODE.LT.19)THEN
            ILIQ = 1
          ELSE
            IFROZ = 1
          ENDIF
5       CONTINUE
      ENDIF

C*--- CHECK FOR MORE THAN ONE TYPE OF PRECIPITATION REPORTED FOR THE SAME HOUR
      IF(ILIQ.EQ.1 .AND. IFROZ.EQ.1)THEN
         WRITE(IDIAG,1000) IDATE,SFTYPE,IPREC,IPCODE
1000     FORMAT(' Warning -- more than one type of precipitation ',
     1          'reported on (YYDDDHH):(',I7,')',/,15X,
     2          A6,' TYPE = ',5I1,3X,'precip. code used = ',I2)
         NOWARN = .FALSE.
      ENDIF

99    RETURN
      END


      SUBROUTINE SAMWX(IWETH,IP5)
C***********************************************************************
C*    SAMWX Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Converts SAMSON present weather codes into
C*                CD144 weather observations (columns 25-29)
C*                as best as possible
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*                Adapted From DEPMET Version: 1.0 Level: 931108
C*                                           : J. Chang, SRC
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:
C*       IWETH - int. array - Precipitation TYPE data
C*
C*    OUTPUTS:
C*       IP5    - integer    - Two-digit precipitation CODE
C*
C*    CALLED FROM: MAIN program
C***********************************************************************

C*    Variable Declarations
C*    IPREC    = COLUMNS 25-29 OF NOAA WEATHER CODE(IN CHAR FORM)
C*    IWETH    = SAMSON PRESENT WEATHER CODE ARRAY
      INTEGER IWETH(9),IP5(5)

C*    CLASSIFY LIQUID PRECIPITATION
      IF (IWETH(2) .EQ. 0) IP5(1) = 1
      IF (IWETH(2) .EQ. 1) IP5(1) = 2
      IF (IWETH(2) .EQ. 2) IP5(1) = 3
      IF (IWETH(2) .EQ. 3) IP5(1) = 4
      IF (IWETH(2) .EQ. 4) IP5(1) = 5
      IF (IWETH(2) .EQ. 5) IP5(1) = 6
      IF (IWETH(2) .EQ. 6) IP5(1) = 7
      IF (IWETH(2) .EQ. 7) IP5(1) = 8
      IF (IWETH(2) .EQ. 8) IP5(1) = 9
      IF (IWETH(2) .EQ. 9) IP5(1) = 0

      IF (IWETH(3) .EQ. 0) IP5(2) = 1
      IF (IWETH(3) .EQ. 1) IP5(2) = 2
      IF (IWETH(3) .EQ. 3) IP5(2) = 4
      IF (IWETH(3) .EQ. 4) IP5(2) = 5
      IF (IWETH(3) .EQ. 5) IP5(2) = 6
      IF (IWETH(3) .EQ. 6) IP5(2) = 7
      IF (IWETH(3) .EQ. 7) IP5(2) = 8
      IF (IWETH(3) .EQ. 8) IP5(2) = 9
      IF (IWETH(3) .EQ. 9) IP5(2) = 0

C*    CLASSIFY SNOW
      IF (IWETH(4) .EQ. 0) IP5(3) = 1
      IF (IWETH(4) .EQ. 1) IP5(3) = 2
      IF (IWETH(4) .EQ. 2) IP5(3) = 3
      IF (IWETH(4) .EQ. 3) IP5(3) = 4
      IF (IWETH(4) .EQ. 4) IP5(3) = 5
      IF (IWETH(4) .EQ. 5) IP5(3) = 6
      IF (IWETH(4) .EQ. 6) IP5(3) = 7
      IF (IWETH(4) .EQ. 7) IP5(3) = 8
      IF (IWETH(4) .EQ. 8) IP5(3) = 9
      IF (IWETH(4) .EQ. 9) IP5(3) = 0

      IF (IWETH(5) .EQ. 0) IP5(4) = 1
      IF (IWETH(5) .EQ. 1) IP5(4) = 2
      IF (IWETH(5) .EQ. 2) IP5(4) = 3
      IF (IWETH(5) .EQ. 3) IP5(4) = 4
      IF (IWETH(5) .EQ. 4) IP5(4) = 5
      IF (IWETH(5) .EQ. 5) IP5(4) = 6
      IF (IWETH(5) .EQ. 9) IP5(4) = 0

      IF (IWETH(6) .EQ. 0) IP5(5) = 1
      IF (IWETH(6) .EQ. 1) IP5(5) = 2
      IF (IWETH(6) .EQ. 2) IP5(5) = 3
      IF (IWETH(6) .EQ. 4) IP5(5) = 5
      IF (IWETH(6) .EQ. 9) IP5(5) = 0

      RETURN
      END


      SUBROUTINE HUSWX(IWETH,IP5)
C***********************************************************************
C*    HUSWX Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Converts SAMSON present weather codes into
C*                CD144 weather observations (columns 25-29)
C*                as best as possible
C*
C*    PROGRAMMER: PES Inc.
C*
C*    DATE:       May 1998
C*
C*    INPUTS:
C*       IWETH - integer array - Precipitation TYPE data
C*
C*    OUTPUTS:
C*       IP5   - integer array - Two-digit precipitation CODE
C*
C*    CALLED FROM: MAIN program
C***********************************************************************

C*    Variable Declarations
C*    IPREC    = COLUMNS 25-29 OF NOAA WEATHER CODE(IN CHAR FORM)
C*    IWETH    = HUSWO PRESENT WEATHER ARRAY
      INTEGER     IWETH(4),IP5(5)


      IP5(1) = 0
      IP5(2) = 0
      IP5(3) = 0
      IP5(4) = 0
      IP5(5) = 0

C     If the entire present weather filed is filled with 9's, or possibly
C     00009999, then there was no weather; make a quick check and return
C     if this condition exists
      IF( (IWETH(1) .eq. 99  .or.  IWETH(1) .eq. 00) .and.
     &    (IWETH(2) .eq. 99  .or.  IWETH(2) .eq. 00) .and.
     &    (IWETH(3) .eq. 99  .or.  IWETH(3) .eq. 00) .and.
     &    (IWETH(4) .eq. 99  .or.  IWETH(4) .eq. 00) ) THEN

         RETURN

      ELSE
C*       Only 4 weather groups are in the HUSWO data; to use subr.PCODES
C        to determine the precip codes, the 5th element in the IP5 array
C        is retained and set to zero

         IP5(5) = 0

         DO I=1,4
C*          CLASSIFY LIQUID PRECIPITATION
            IF (IWETH(i) .EQ. 20) IP5(i) = 1
            IF (IWETH(i) .EQ. 21) IP5(i) = 2
            IF (IWETH(i) .EQ. 22) IP5(i) = 3
            IF (IWETH(i) .EQ. 23) IP5(i) = 4
            IF (IWETH(i) .EQ. 24) IP5(i) = 5
            IF (IWETH(i) .EQ. 25) IP5(i) = 6
            IF (IWETH(i) .EQ. 26) IP5(i) = 7
            IF (IWETH(i) .EQ. 27) IP5(i) = 8
            IF (IWETH(i) .EQ. 28) IP5(i) = 9
            IF (IWETH(i) .EQ. 29) IP5(i) = 0

            IF (IWETH(i) .EQ. 30) IP5(i) = 1
            IF (IWETH(i) .EQ. 31) IP5(i) = 2
            IF (IWETH(i) .EQ. 33) IP5(i) = 4
            IF (IWETH(i) .EQ. 34) IP5(i) = 5
            IF (IWETH(i) .EQ. 35) IP5(i) = 6
            IF (IWETH(i) .EQ. 36) IP5(i) = 7
            IF (IWETH(i) .EQ. 37) IP5(i) = 8
            IF (IWETH(i) .EQ. 38) IP5(i) = 9
            IF (IWETH(i) .EQ. 39) IP5(i) = 0

C*          CLASSIFY FROZEN PRECIPITATION
            IF (IWETH(i) .EQ. 40) IP5(i) = 1
            IF (IWETH(i) .EQ. 41) IP5(i) = 2
            IF (IWETH(i) .EQ. 42) IP5(i) = 3
            IF (IWETH(i) .EQ. 43) IP5(i) = 4
            IF (IWETH(i) .EQ. 44) IP5(i) = 5
            IF (IWETH(i) .EQ. 45) IP5(i) = 6
            IF (IWETH(i) .EQ. 46) IP5(i) = 7
            IF (IWETH(i) .EQ. 47) IP5(i) = 8
            IF (IWETH(i) .EQ. 48) IP5(i) = 9
            IF (IWETH(i) .EQ. 49) IP5(i) = 0

            IF (IWETH(i) .EQ. 50) IP5(i) = 1
            IF (IWETH(i) .EQ. 51) IP5(i) = 2
            IF (IWETH(i) .EQ. 52) IP5(i) = 3
            IF (IWETH(i) .EQ. 53) IP5(i) = 4
            IF (IWETH(i) .EQ. 54) IP5(i) = 5
            IF (IWETH(i) .EQ. 55) IP5(i) = 6
            IF (IWETH(i) .EQ. 59) IP5(i) = 0

            IF (IWETH(i) .EQ. 60) IP5(i) = 1
            IF (IWETH(i) .EQ. 61) IP5(i) = 2
            IF (IWETH(i) .EQ. 62) IP5(i) = 3
            IF (IWETH(i) .EQ. 64) IP5(i) = 5
            IF (IWETH(i) .EQ. 69) IP5(i) = 0

            IF (IWETH(i) .EQ. 90) IP5(i) = 1
            IF (IWETH(i) .EQ. 91) IP5(i) = 2
            IF (IWETH(i) .EQ. 92) IP5(i) = 3
            IF (IWETH(i) .EQ. 99) IP5(i) = 0
         ENDDO
      ENDIF
      RETURN
      END


      SUBROUTINE READMX
C***********************************************************************
C*    READMIX Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Reads Mixing Height Data - AM and PM
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 14, 1995
C*
C*    INPUTS:     SCRAM Mixing Height File
C*
C*    OUTPUTS:    Morning and Afternoon Mixing Heights
C*
C*    CALLED FROM: MAIN program
C***********************************************************************

C*    Variable Declarations
C*    (IMX is Used to Check If This Routine is Being Called
C*    for the Very First Time or Not)
      SAVE IMX
      INCLUDE 'PCRAM.INC'
      CHARACTER*35 DUMMY

      DATA IMX/0/

C*    For The Very First Day Read Previous Days And Next
C*    Days Mixing Heights As Well
      IMX = IMX + 1
      IF (IMX .EQ. 1) THEN

C*       Look To See If a Header Is Present
         READ(INMIX,'(A35)') DUMMY

C*       If It Is, Then Skip It and Read Another Line, Otherwise,
C*       Start Reading from This Line.
         IF (DUMMY(8:11)  .EQ. '    '   .AND.
     &       DUMMY(12:12) .EQ. ' '      .AND.
     &       DUMMY(31:35) .EQ. '     ' ) THEN

            READ (INMIX,440,ERR=1039) IDMIX(1),IYRMIX(1),IMOMIX(1),
     &                           IDYMIX(1),AMMIX(1),PMMIX(1)
440         FORMAT (I5,3I2,1X,F5.0,13X,F5.0)
         ELSE
            READ (DUMMY,440,ERR=1039) IDMIX(1),IYRMIX(1),IMOMIX(1),
     &                           IDYMIX(1),AMMIX(1),PMMIX(1)

         ENDIF

C*       Read Current Day and Next Day
         READ (INMIX,440,ERR=1039) IDMIX(2),IYRMIX(2),IMOMIX(2),
     &                           IDYMIX(2),AMMIX(2),PMMIX(2)
         READ (INMIX,440,ERR=1039) IDMIX(3),IYRMIX(3),IMOMIX(3),
     &                           IDYMIX(3),AMMIX(3),PMMIX(3)

      ELSE

C*       Make this Previous Day = Last Current Day
C*       Make this Current  Day = Last Next Day
         AMMIX(1) = AMMIX(2)
         PMMIX(1) = PMMIX(2)
         IDMIX(1) = IDMIX(2)
         IYRMIX(1) = IYRMIX(2)
         IMOMIX(1) = IMOMIX(2)
         IDYMIX(1) = IDYMIX(2)
         AMMIX(2) = AMMIX(3)
         PMMIX(2) = PMMIX(3)
         IDMIX(2) = IDMIX(3)
         IYRMIX(2) = IYRMIX(3)
         IMOMIX(2) = IMOMIX(3)
         IDYMIX(2) = IDYMIX(3)

C*       Read the Next Day
         READ (INMIX,440,ERR=1039) IDMIX(3),IYRMIX(3),IMOMIX(3),
     &                        IDYMIX(3),AMMIX(3),PMMIX(3)

      ENDIF
      GO TO 999

C*    Error Handling
1039  CALL ERRHDL('E','1039',0)

999   RETURN
      END


      SUBROUTINE READPP
C***********************************************************************
C*    READPPT Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Reads Hourly Preciptitation Data from the
C*                TD3240 File
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 14, 1995
C*
C*    INPUTS:     TD3240 Precip
C*
C*    OUTPUTS:    24 Hours of Precip Data
C*
C*    CALLED FROM: MAIN program
C***********************************************************************

C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

C*    If PPTFLG Is TRUE, Read From TD3240
      IF (PPTFLG) THEN

C*       Loop Over 24 Hours for This Day
         DO 100 IHR = 1,24

C*          Determine The Period For Which Data Is Needed
C*          Using Surface Data
            IF (SFCTYP .EQ. 'CD144') THEN

               READ (ACD144(2,IHR),'(I5)') IY
               READ (ACD144(3,IHR),'(I5)') IM
               READ (ACD144(4,IHR),'(I5)') ID
               READ (ACD144(5,IHR),'(I5)') IH

               IF (IH .EQ. 0) THEN
                  CALL JULIAN (IY,IM,ID,JD,0)
                  CALL HR0024(IY,JD,IH)
                  CALL JULIAN (IY,IM,ID,JD,1)
               ENDIF

            ELSE IF (SFCTYP .EQ. 'SAMSON') THEN
               READ (JSAMSN(1,IHR),'(I2)') IY
               READ (JSAMSN(2,IHR),'(I2)') IM
               READ (JSAMSN(3,IHR),'(I2)') ID
               READ (JSAMSN(4,IHR),'(I2)') IH

            ENDIF

C*          Code Date Variable
            CALL JULIAN (IY,IM,ID,JD,0)
            NDATE = IY*100000+JD*100+IH

C*          Get an Hour of Precip Data
            CALL UNCDP(INPPT,NDATE,24,1,PMM,ICODE,IDIAG)
            P3240(IHR) = PMM
            IY3240(IHR) = IY
            IM3240(IHR) = IM
            ID3240(IHR) = ID

100      CONTINUE
      ENDIF
      RETURN
      END


      SUBROUTINE CHKCLM(IHR)
C***********************************************************************
C*    CHKCLM Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    This routine checks if the current hour is calm
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:     Wind speed
C*
C*    OUTPUTS:    Logical (T or F) indicating calm conditions
C*
C*    CALLED FROM: MAIN program
C***********************************************************************

C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

C*    If The Wind Speed Is Zero, Then
      IF (WSPEED(IHR) .EQ. 0.0) THEN
         CALM(IHR) = .TRUE.
      ELSE
         CALM(IHR) = .FALSE.
      ENDIF

      RETURN
      END


      SUBROUTINE CHKDAT(IHR)
C***********************************************************************
C*    CHKDAT      Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    Checks the Date and Time Stamps Between All Surface
C*                Mixing Height (and Precip Data)
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 16, 1995
C*
C*    INPUTS:     Various Date amd Time Stamps
C*
C*    OUTPUTS:    none - program terminates on error
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

C*    Check To See If The Surface Data Time Stamp Matches
C*    The Upper Air Data Time Stamp
      IF (ISYR(IHR) .NE. IYRMIX(2) .OR.
     &    ISMO(IHR) .NE. IMOMIX(2) .OR.
     &    ISDY(IHR) .NE. IDYMIX(2)) THEN
          GO TO 1040
      ENDIF

C*    If Using Precip Data...
C*    A check on dates for precipitation is not necessary because
C*    the year, month, day and hour come from the CD144 or SAMSON data
C*    which is then provided to the routines that fetch the precip for
C*    the requested hour.

      GO TO 999

C*    Error Handling
1040  CALL ERRHDL('E','1040',1)
1041  CALL ERRHDL('E','1041',1)

999   RETURN
      END


      SUBROUTINE FLOVEC(IHR,JDAY)
C***********************************************************************
C*    FLOVEC Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE: This Routine Converts The Meteorological Wind Direction
C*             To Flow Vector and Also Calculates Random Flow Vector
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:     Wind direction, array of random numbers
C*
C*    OUTPUTS:    Flow vector for the hour
C*
C*    CALLED FROM: MAIN program
C***********************************************************************

C*    Variable Declarations
      INCLUDE 'PCRAM.INC'
      INTEGER KRAND
      COMMON /BDATA/ IRAND(24,366)

C*    Calculate flow vector and random flow vector only if the
C*    wind direciton is missing.

C*    Only winds from CD144 format need to be multiplied by 10;
C*    the SAMSON data are reported to the nearest 10 degrees, but
C*    in whole degrees.

      IF( IDIR(IHR) .LT. 900 )THEN

         IF( SFCTYP .EQ. 'CD144'  .OR.  SFCTYP .EQ. 'SCRAM' )THEN
            XDIR = IDIR(IHR)*10.0
         ELSEIF( SFCTYP .EQ. 'SAMSON'  .OR.  SFCTYP .EQ. 'HUSWO' )THEN
            XDIR = IDIR(IHR)
         ENDIF

         IF (XDIR.GT.180.0) THEN
            FV=XDIR-180.0
         ELSE
            FV=XDIR+180.0
         ENDIF
         AFV(IHR)=FV

C*       IHR AND JDAY, THE LOOP CONTROL VARIABLES, ARE BEING USED AS
C*       INDICES INTO THE RANDOM NUMBER ARRAY KRAND(24,366).
C*       IRAND IS SINGLE DIGIT FROM 0 TO 9.

         KRAND = IRAND(IHR,JDAY)
         FVR(IHR) = FV + KRAND - 4.0
         IF (FVR(IHR).GT.360.0) FVR(IHR)=FVR(IHR)-360.0

      ELSE
         FVR(IHR) = 999.0

      ENDIF

      RETURN
      END


      SUBROUTINE UNITS(IHR)
C***********************************************************************
C*    UNITS Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE:    This Routine Converts All Met Variables to Appropriate
C*                Units; it also converts any missing values to a
C*                nonmissing value for selected variables
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*
C*    DATE:       April 28, 1995
C*
C*    INPUTS:     Various Parameters
C*
C*    OUTPUTS:    Same parameters converted to required units
C*
C*    CALLED FROM: MAIN program
C***********************************************************************

C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

      IF( SFCTYP .EQ. 'CD144'  .OR.  SFCTYP .EQ. 'SCRAM')THEN

C*       CONVERT TEMP FROM FAHRENHEIT TO KELVIN
         IF( TEMP(IHR) .LT. 9000.0 )THEN
            TEMP(IHR) = 0.5556 * (TEMP(IHR)-32.0) + 273.15
         ENDIF

C*       CONVERT WIND SPEED FROM KNOTS TO METERS/SEC,
C*       but save the wind speed in knots (integer) for use in
C*       computing stability category
         KSPEED(IHR) = WSPEED(IHR) + 0.01
         IF( KSPEED(IHR) .LT. 98 )THEN
            WSPEED(IHR) = WSPEED(IHR)*0.51444
C*          IF Wind Speed < 1.0 m/s, Wind Speed Is Set To 1 m/s
            IF (WSPEED(IHR) .LT. 1.0) WSPEED(IHR)=1.0
         ENDIF

C*       Convert cloud cover from tenths to fraction
C*       Note that a missing cloud cover is assumed to be 10/10
C        as performed in SUBR.CCCODE
         FCOVER(IHR) = 0.1 * FLOAT(ICOVER(IHR))

C*       Convert surface pressure from inches hg to mb
C*       (33.864 mb = 1 inch hg)
         IF( PRESS(IHR) .LT. 9000.0 )THEN
            PRESS(IHR) = PRESS(IHR) * 33.864
         ELSE
            PRESS(IHR) = 1000.0
         ENDIF

      ELSEIF( SFCTYP .EQ. 'SAMSON' )THEN

C*       Ceiling height must be converted back to hundreds of feet;
C*       77777 = unlimited, 88888 = ciroform, 999999 = missing, 
C*       but the range of valid values is 0 - 30450 meters (99900 feet)
         IF( ICEIL(IHR) .EQ. 77777  .OR.  ICEIL(IHR) .GT. 30450 )THEN
C           Unlimited ceiling
            ICEIL(IHR) = 998
         ELSE
            ICEIL(IHR) = NINT( ( ICEIL(IHR) / 0.3048 ) / 100.0 )
         ENDIF

C*       Convert temp from celsius to Kelvin
C*       For the SAMSON results to be consistent with the CD144 results,
C*       temperature is converted from celsius to fahrenheit then back
C*       to celsius
         IF( TEMP(IHR) .LT. 9000.0 )THEN
            TFAHR     = NINT(TEMP(IHR)/0.5556 + 32.0)
            TEMP(IHR) = 0.5556*(TFAHR-32.0) + 273.15
         ENDIF

C*       Wind speed (WSPEED) is already in meters/sec, but we need knots
C*       for stability class (KSPEED);
C*       Convert the KSPEED back to m/s so the SAMSON results are
C*       consistent with the CD-144 results
         IF( WSPEED(IHR) .LT. 99.0 )THEN
            KSPEED(IHR) = NINT(WSPEED(IHR)/0.51444)
            WSPEED(IHR) = KSPEED(IHR) * 0.51444

C*          IF Wind Speed < 1.0 m/s, Wind Speed Is Set To 1 m/s
            IF (WSPEED(IHR) .LT. 1.0) WSPEED(IHR)=1.0
         ENDIF

C*       Convert cloud cover from tenths to fraction
         IF( ICOVER(IHR) .LT. 90 )THEN
            FCOVER(IHR) = 0.1*FLOAT(ICOVER(IHR))
         ELSE
            FCOVER(IHR) = 1.0
         ENDIF

C*       Surface pressure is already in millibars, but check for missing
C        and set it to 1000 mb
         IF( PRESS(IHR) .GT. 9000.0 )THEN
            PRESS(IHR) = 1000.0
         ENDIF

C*       Precipitation - already converted to millimeters (in S.FILSAM),
C*       but check for missing
         IF( PRECIP(IHR) .GT. 9999.0 )THEN
            PRECIP(IHR) = 0.0
         ENDIF

      ELSEIF( SFCTYP .EQ. 'HUSWO' )THEN

C*       Ceiling height must be converted frpm feet to hundreds of feet
C*       77777 = unlimited, 88888 = ciroform, 99999 = missing, 
C*       but the valid range of values is 00000 - 99900 feet
         IF( ICEIL(IHR) .EQ. 77777  .OR.  ICEIL(IHR) .GT. 99900 )THEN
C           Unlimited ceiling
            ICEIL(IHR) = 998
         ELSE
            ICEIL(IHR) = NINT( ( ICEIL(IHR) ) / 100.0 )
         ENDIF

C*       Convert temp from Fahrenheit to Kelvin
         IF( TEMP(IHR) .LT. 900.0 )THEN
            TEMP(IHR) = 0.5556*(TEMP(IHR)-32.0) + 273.15
         ENDIF

C*       Wind speed is in miles per hour, but we need knots for
C*       stability class and meters/second for all other calculations
         IF( WSPEED(IHR) .LT. 99.0 )THEN
            KSPEED(IHR) = NINT(WSPEED(IHR)/1.15)
            WSPEED(IHR) = WSPEED(IHR) / 2.237

C*          IF Wind Speed < 1.0 m/s, Wind Speed Is Set To 1 m/s
            IF (WSPEED(IHR) .LT. 1.0) WSPEED(IHR)=1.0
         ELSE
            KSPEED(IHR) = 99
         ENDIF

C*       Convert cloud cover from tenths to fraction
         IF( ICOVER(IHR) .LT. 99 )THEN
            FCOVER(IHR) = 0.1*FLOAT(ICOVER(IHR))
         ELSE
            FCOVER(IHR) = 1.0
         ENDIF

C*       Surface pressure is hundredths of inches
         IF( PRESS(IHR) .LT. 9000.0 )THEN
            PRESS(IHR) = (PRESS(IHR)/100.0) * 33.8639
         ENDIF

C*       Precipitation - already converted to millimeters (in S.FILSAM),
C*       but check for missing
         IF( PRECIP(IHR) .GT. 9999.0 )THEN
            PRECIP(IHR) = 0.0
         ENDIF

      ENDIF

C*    Check the surface pressure - if less than 500 mb or greater than
C*    9000.0, then the pressure could be missing, so set it to 1000 mb
C     (A similar check and substitution may already have been performed)
      IF( PRESS(IHR) .LT. 500.0  .OR.  PRESS(IHR) .GT. 9000.0 )THEN
          PRESS(IHR) = 1000.0
      ENDIF

C*    Compute air density in kg/m**3 -- rho = p/(r * t) --
C*    r=287 m**2/(deg k * sec**2)
C*    0.3484321=(100. kg/(m*sec**2) per mb)/(287 m**2/(deg k * sec**2))
      IF(TEMP(IHR) .LT. 900.0 )THEN
         RHO(IHR) = 0.3484321 * PRESS(IHR) / TEMP(IHR)
      ELSE
         RHO(IHR) = -1.0
      ENDIF

      RETURN
      END


c ------------------------------------------------------------------------------
      subroutine uncdp(io,ndate,maxap,k,pmm,icode,idiag)
c ------------------------------------------------------------------------------

c
c*******sample call *********************************
c            NDATE = IYEAR*100000+JULIAN*100+IHOUR  *
c            CALL UNCDP(INPPT,NDATE,24,1,PMM,ICODE) *
c****************************************************
c
c --- PMERGE   Version: 1.2       Level: 901130                    UNCDP
c ---          J. Scire, SRC
C*
C*
C*    Modified: Jayant A. Hardikar, PES
C*              Made it relatively generic
C*              2/15/95
C*
c
c --- Determine the precipitation rate for a given date/hour by:
c        (a) reading a character variable storing a previously read
c            record
c        (b) reading a precipitation record from a TD-3240 file
c        (c) resolving an accumulation period
c        (d) resolving a missing data period
c
c --- INPUTS:
c                IO - integer    - Fortran unit no. of precip. input
c                                  file
c             NDATE - integer    - Coded date/time field (YYJJJHH) of
c                                  current hour
c             MAXAP - integer    - Maximum allowed length (hrs) of an
c                                  accumulation period
c                 K - integer    - Array index of precip. arrays
c                                  (K = station no.)
c --- OUTPUT:
c               PMM - real       - Precipitation rate (mm/hr)
c                                  (missing value indicator = 9999.)
c             ICODE - integer    - Data status code:
c                                  1 = valid hourly value,
c                                  2 = valid accumulation period,
c                                  3 = missing data flag (labeled missing),
c                                  4 = missing due to excessive length
c                                      of accumulation period,
c                                  5 = missing data before first valid
c                                      record in file
c                                  6 = missing data after last valid
c                                      record in file
c
c --- UNCDP called by:  READPP
c --- UNCDP calls:      PREAD
c
c --- IFLAG -- Flag indicating precip. data status:
c              IFLAG = -99 if this is the first pass for this station
c              IFLAG =   0 if date/hr of first precip. record has not
c                          been reached yet (data is assumed missing
c                          up to date/hr of first valid record)
c              IFLAG =  +1 if current date/hr > date/hr of first record
c                          (precip. rate is assumed = 0.0 between time
c                          of valid records)
c ------------------------------------------------------------------------------
      parameter(maxps=1)
c
      real pmmsav(maxps)
      integer icodsv(maxps),ibdat(maxps),iedat(maxps),iflag(maxps)
      integer iprev(maxps)
      character*42 cdat(maxps)
c
      data icodsv/maxps*0/,ibdat/maxps*0/,iedat/maxps*0/
      data iflag/maxps*-99/,cdat/maxps*' '/
      data iprev/maxps*0/

c     Set the unit log file unit number, io6, to idiag
      io6=idiag
c
c --- determine if current date/hr is within range previously stored
c --- pmmsav array
10    continue
      if(ndate.lt.ibdat(k))then
c
c ---    Date/hr between valid records -- precip. rate = 0.0
         pmm=0.0
         icode=1
         return
      else if(ndate.le.iedat(k))then
c
c ---    Current date/hr is within period of validity of pmmsav
         pmm=pmmsav(k)
         icode=icodsv(k)
         return
      endif
c
c --- Current date/hr is after end of period of validity of pmmsav
25    continue
      call pread(io,ndate,maxap,iflag(k),cdat(k),iprev(k),
     1           icodsv(k),ibdat(k),iedat(k),pmmsav(k),idiag)
      go to 10
      end
c ------------------------------------------------------------------------------
      subroutine pread(io,ndate,maxap,iflag,cdat,iprev,
     1 icode,ibdat,iedat,pmmsav,idiag)
c ------------------------------------------------------------------------------
c
c --- PMERGE   Version: 1.2       Level: 921022                    PREAD
c ---          J. Scire, SRC
c
c --- Read a precipitation record -- if necessary, read a second record
c --- to resolve a missing data or accumulation period
c
c --- INPUTS:
c                IO - integer    - Fortran unit no. of precip. input
c                                  file
c             NDATE - integer    - Coded date/time field (YYJJJHH) of
c                                  current hour
c             MAXAP - integer    - Maximum allowed length (hrs) of an
c                                  accumulation period
c             IFLAG - integer    - Flag indicating precip. data status:
c                                  IFLAG = -99 if this is the first pass
c                                              for this station
c                                  IFLAG =   0 if date/hr of first
c                                              precip. record has not
c                                              been reached yet (data is
c                                              assumed missing up to
c                                              date/hr of first valid
c                                              record)
c                                  IFLAG =  +1 if current date/hr >
c                                              date/hr of first record
c                                              (precip. rate is assumed
c                                              = 0.0 between time of
c                                              valid records)
c              CDAT - char.*42   - A character string to store a TD-3240
c                                  data record (an input only if IFLAG = 0)
c             IPREV - integer    - Coded date/time field (YYJJJHH) of
c                                  previously read TD-3240 record
c
c --- OUTPUT:
c             ICODE - integer    - Data status code:
c                                  1 = valid hourly value,
c                                  2 = valid accumulation period,
c                                  3 = missing data flag (labeled missing)
c                                  4 = missing due to excessive length
c                                      of accumulation period
c                                  5 = missing data before first valid
c                                      record in file
c                                  6 = missing data after last valid
c                                      record in file
c             IBDAT - integer    - Beginning date/time of data (YYJJJHH)
c             IEDAT - integer    - Ending date/time of data (YYJJJHH)
c            PMMSAV - real       - Precipitation rate (mm/hr)
c                                  (missing value indicator = 9999.)
c              CDAT - char.*42   - A character string storing a TD-3240
c                                  data record (an output only if
c                                  IFLAG = -99)
c             IPREV - integer    - Updated coded date/time field of
c                                  last TD-3240 record read
c
c --- PREAD called by:  UNCDP
c --- PREAD calls:   JULIAN
c                    INDECR
c ------------------------------------------------------------------------------
c
      character*42 cdat
      character*1 cflag,cflag2
      cflag=' '
      cflag2=' '
      io6=idiag

      if(iflag.le.0)then
c
c ---    If first time through for this station, read TD-3240 record &
c ---    store in character string (CDAT)
         if(iflag.eq.-99)then
            read(io,18)cdat
18          format(a42)
            iflag=0
c
c ---       extract station id, date/hr
            read(cdat,20)idsta,iyr,imo,iday,ihr
20          format(3x,i6,10x,i2,i2,i4,3x,i2,2x,i6,a1)
            call julian(iyr,imo,iday,ijul,0)
            idate=iyr*100000+ijul*100+ihr
            iprev=idate
c
c
c ---       all data up to time of first record in file is considered
c ---       as zero
            icode=5
            ibdat=0
c ---       subtract one hour from yr/Julian day/hr
            call indecr(iyr,ijul,ihr,-1,1,24,idiag )
            iedat=iyr*100000+ijul*100+ihr
            pmmsav=0.
            return
         endif
c
c ---    extract data from a previously read character string
         read(cdat,20,end=995)idsta,iyr,imo,iday,ihr,ihinch,cflag
         iflag=1
         call julian(iyr,imo,iday,ijul,0)
         idate=iyr*100000+ijul*100+ihr
c

      else
c
c ---    read a new record
         read(io,20,end=995)idsta,iyr,imo,iday,ihr,ihinch,cflag
         call julian(iyr,imo,iday,ijul,0)
         idate=iyr*100000+ijul*100+ihr
c
c
c ---    check if date/hr of record is out of order
         if(idate.gt.iprev)then
            iprev=idate
         else
            go to 1040
         endif
      endif
c
      if(cflag.eq.' ' .or. cflag.eq.'E'  .or. cflag .eq. 'e')then
c
c ---    valid hourly data value -- convert to mm/hr
         icode=1
         ibdat=idate
         iedat=idate
         pmmsav=0.254*float(ihinch)
         return
      else if(cflag.eq.'A' .or.  cflag .eq. 'a')then
c
c ---    beginning of accumulation period -- read next record
c ---    with ending date/time of accumulation period & accum. amount
         read(io,20,end=996)jdsta,jyr,jmo,jday,jhr,jhinch,cflag2
         call julian(jyr,jmo,jday,jjul,0)
         jdate=jyr*100000+jjul*100+jhr
         if(jdate.gt.iprev)then
            iprev=jdate
         else
            go to 1050
         endif
c
         if(cflag2.ne.'A' .and. cflag2 .ne. 'a')then
c
c ---       ERROR -- unpaired accumulation period
            go to 1000
         else
c
c ---       paired accumulation records -- resolve accumulation
c ---       period precip. rate
            call deltt(iyr,ijul,ihr,jyr,jjul,jhr,idelt)
            nhrs=idelt+1
c
c ---       if length of the accumulation period exceeds max.
c ---       allowed, consider data as missing
            if(nhrs.gt.maxap)then
               icode=4
               ibdat=idate
               iedat=jdate
               pmmsav=9999.
               return
            endif
c
c ---       valid accumulation period -- resolve & save results
            icode=2
            ibdat=idate
            iedat=jdate
            pmmsav=0.254*float(jhinch)/float(nhrs)
            return
         endif
c
      else if(cflag.eq.'M' .or. cflag.eq.'D' .or.
     &        cflag.eq.'m' .or. cflag.eq.'d')then
c ---    beginning of missing or deleted data period -- read next record
c ---    with ending date/time of missing or deleted data period
         read(io,20,end=998)jdsta,jyr,jmo,jday,jhr,jhinch,cflag2
         call julian(jyr,jmo,jday,jjul,0)
         jdate=jyr*100000+jjul*100+jhr
         if(jdate.gt.iprev)then
            iprev=jdate
         else
            go to 1050
         endif
c
c **     if(cflag2.ne.'M' .and. cflag2 .ne. 'm')then
         if(cflag2.ne.cflag)then
c
c ---       ERROR -- unpaired missing or deleted data records
            go to 1030
         else
c
c ---       paired missing data records
            icode=3
            ibdat=idate
            iedat=jdate
            pmmsav=9999.
            return
         endif
      endif
c
c --- invalid precipitation flag encountered
      write(io6,990)idsta,ndate,io,iyr,imo,iday,ihr,cflag,cflag2
990   format(/1x,'Error in subr. PREAD -- invalid ',
     1 'precipitation flag encountered'//1x,'ID = ',i6,3x,
     2 //1x,'Requested date/hr (YYJJJHH) = ',i7,3x,'io = ',i5
     3 //1x,'Yr: ',i2,2x,'Month: ',i2,2x,'Day: ',i2,
     4 2x,'Hr: ',i2//1x,'CFLAG = ',a1,3x,'CFLAG2 = ',a1)
      CALL ERRHDL('E','0000',2)
c
c --- end of file encountered
995   continue
      icode=6
      ibdat=0
      iedat=9999999
      pmmsav=0.
      return
c
c --- end of file encountered -- unpaired accumulation period
996   continue
      write(io6,997)idsta,ndate,io,iyr,imo,iday,ihr,cflag
997   format(/1x,'Error in subr. PREAD -- unpaired ',
     1 'accumulation period'//1x,'ID = ',i6,3x,
     2 //1x,'Requested date/hr (YYJJJHH) = ',i7,3x,'io = ',i5
     3 //1x,'1st record: ',2x,'Yr: ',i2,2x,'Month: ',i2,2x,
     4 'Day: ',i2,2x,'Hr: ',i2,2x,' Flag: ',a1
     5 //1x,'2nd record: ',2x,'END OF FILE REACHED')
      CALL ERRHDL('E','0000',2)
c
c --- end of file encountered -- unpaired missing or deleted period
998   continue
      write(io6,999)idsta,ndate,io,iyr,imo,iday,ihr,cflag
999   format(/1x,'Error in subr. PREAD -- unpaired ',
     1 'missing or deleted data period'//1x,'ID = ',i6,3x,
     2 //1x,'Requested date/hr (YYJJJHH) = ',i7,3x,'io = ',i5
     3 //1x,'1st record: ',2x,'Yr: ',i2,2x,'Month: ',i2,2x,
     4 'Day: ',i2,2x,'Hr: ',i2,2x,' Flag: ',a1
     5 //1x,'2nd record: ',2x,'END OF FILE REACHED')
      CALL ERRHDL('E','0000',2)
c
c --- write error message -- unpaired accumulation period
1000  continue
      write(io6,1002)idsta,ndate,io,iyr,imo,iday,ihr,cflag,
     1 jyr,jmo,jday,jhr,cflag2
1002  format(/1x,'Error in subr. PREAD -- unpaired ',
     1 'accumulation period'//1x,'ID = ',i6,3x,
     2 //1x,'Requested date/hr (YYJJJHH) = ',i7,3x,'io = ',i5
     3 //1x,'1st record: ',2x,'Yr: ',i2,2x,'Month: ',i2,2x,
     4 'Day: ',i2,2x,'Hr: ',i2,2x,' Flag: ',a1
     5 //1x,'2nd record: ',2x,'Yr: ',i2,2x,'Month: ',i2,2x,
     6 'Day: ',i2,2x,'Hr: ',i2,2x,' Flag: ',a1)
      CALL ERRHDL('E','0000',2)
c
c --- write error message -- unpaired missing data period
1030  continue
      write(io6,1032)idsta,ndate,io,iyr,imo,iday,ihr,cflag,
     1 jyr,jmo,jday,jhr,cflag2
1032  format(/1x,'Error in subr. PREAD -- unpaired ',
     1 'missing or deleted data period'//1x,3x,
     2 //1x,'Requested date/hr (YYJJJHH) = ',i7,3x,'io = ',i5
     3 //1x,'1st record: ',2x,'Yr: ',i2,2x,'Month: ',i2,2x,
     4 'Day: ',i2,2x,'Hr: ',i2,2x,' Flag: ',a1
     5 //1x,'2nd record: ',2x,'Yr: ',i2,2x,'Month: ',i2,2x,
     6 'Day: ',i2,2x,'Hr: ',i2,2x,' Flag: ',a1)
      CALL ERRHDL('E','0000',2)
c
c --- write error message -- invalid date/hr ( <= previous value)
1040  continue
      write(io6,1042)idsta,ndate,io,iyr,imo,iday,ihr,cflag,
     1 idate,iprev
1042  format(/1x,'Error in subr. PREAD -- invalid date/hr ( <= ',
     1 'previous value)'//1x,'ID = ',i6,3x,
     2 //1x,'Requested date/hr (YYJJJHH) = ',i7,3x,'io = ',i5
     3 //1x,'Yr: ',i2,2x,'Month: ',i2,2x,
     4 'Day: ',i2,2x,'Hr: ',i2,2x,' Flag: ',a1
     5 //1x,'Date/hr (YYJJJHH) = ',i10,'  (Current record)'
     6 //1x,'Date/hr (YYJJJHH) = ',i10,' (Previous record)')
      CALL ERRHDL('E','0000',2)
c
1050  continue
      write(io6,1042)jdsta,ndate,io,jyr,jmo,jday,jhr,cflag,
     1 jdate,iprev
      CALL ERRHDL('E','0000',2)

      end


c ------------------------------------------------------------------------------
      subroutine indecr(iyr,ijul,ihr,idelt,ihrmin,ihrmax,idiag)
c ------------------------------------------------------------------------------
c
c --- PMERGE   Version: 1.2       Level: 901130                    INDECR
c ---          J. Scire, SRC
c
c --- Increment or decrement a date/time by "IDELT" hours
c --- (-24 <= IDELT <= 24)
c --- Allows specification of 0-23 or 1-24 hour clock
c
c --- INPUTS:
c               IYR - integer    - Input Year
c              IJUL - integer    - Input Julian day
c               IHR - integer    - Input hour (ihrmin <= IHR <= ihrmax)
c             IDELT - integer    - Change in time (hours) -- must be
c                                  between -24 to +24, inclusive
c            IHRMIN - integer    - Minimum hour (i.e., either  0 or  1)
c            IHRMAX - integer    - Maximum hour (i.e., either 23 or 24)
c
c --- OUTPUT:
c               IYR - integer    - Year after change of "IDELT" hours
c              IJUL - integer    - Julian day after change of "IDELT" hours
c               IHR - integer    - Hour after change of "IDELT" hours
c
c --- INDECR called by:  PREAD
c --- INDECR calls:      none
c ------------------------------------------------------------------------------
      io6=idiag
c
      if(iabs(idelt).gt.24)then
         write(io6,10)'IDELT',iyr,ijul,ihr,idelt,ihrmin,ihrmax
10       format(/1x,'ERROR in subr. INDECR -- invalid "',a,'" -- ',
     1   ' iyr,ijul,ihr,idelt,ihrmin,ihrmax = ',6i10)
         stop
      endif
      if(ihr.lt.ihrmin.or.ihr.gt.ihrmax)then
         write(io6,10)'IHR',iyr,ijul,ihr,idelt,ihrmin,ihrmax
         stop
      endif
c
      if(idelt.lt.0)then
c ---    idelt is negative
         ihr=ihr+idelt
         if(ihr.lt.ihrmin)then
            ihr=ihr+24
            ijul=ijul-1
            if(ijul.lt.1)then
               iyr=iyr-1
               if(mod(iyr,4).eq.0)then
                  ijul=366
               else
                  ijul=365
               endif
            endif
         endif
      else
c ---    idelt is positive or zero
         ihr=ihr+idelt
         if(ihr.gt.ihrmax)then
            ihr=ihr-24
            ijul=ijul+1
            if(mod(iyr,4).eq.0)then
               ndays=366
            else
               ndays=365
            endif
            if(ijul.gt.ndays)then
               ijul=1
               iyr=iyr+1
            endif
         endif
      endif
c
      return
      end


c-----------------------------------------------------------------------
      subroutine deltt(j1yr,j1jul,j1hr,j2yr,j2jul,j2hr,jleng)
c-----------------------------------------------------------------------
c
c --- PMERGE   Version: 1.2       Level: 901130                    DELTT
c ---          J. Scire, SRC
c
c --- Compute the difference (in hours) between two dates & times
c ---    (time #2 - time #1)
c
c --- INPUTS:
c              J1YR - integer    - Year of date/time #1
c             J1JUL - integer    - Julian day of date/time #1
c              J1HR - integer    - Hour of date/time #1
c              J2YR - integer    - Year of date/time #2
c             J2JUL - integer    - Julian day of date/time #2
c              J2HR - integer    - Hour of date/time #2
c
c --- OUTPUT:
c             JLENG - integer    - Difference (#2 - #1) in hours
c
c --- DELTT called by:  PREAD
c --- DELTT calls:      none
c-----------------------------------------------------------------------
c
      jmin=min0(j1yr,j2yr)
c
c --- find the number of hours between Jan. 1 of the "base" year and
c --- the first date/hour
      if(j1yr.eq.jmin)then
         j1=0
      else
         j1=0
         j1yrm1=j1yr-1
         do 10 i=jmin,j1yrm1
         if(mod(i,4).eq.0)then
            j1=j1+8784
         else
            j1=j1+8760
         endif
10       continue
      endif
      j1=j1+(j1jul-1)*24+j1hr
c
c --- find the number of hours between Jan. 1 of the "base" year and
c --- the second date/hour
      if(j2yr.eq.jmin)then
         j2=0
      else
         j2=0
         j2yrm1=j2yr-1
         do 20 i=jmin,j2yrm1
         if(mod(i,4).eq.0)then
            j2=j2+8784
         else
            j2=j2+8760
         endif
20       continue
      endif
      j2=j2+(j2jul-1)*24+j2hr
c
c --- compute the time difference (in hours)
      jleng=j2-j1
c
      return
      end


      SUBROUTINE VAR2FIX(INPPT)
C*--------------------------------------------------------------------
C*    PURPOSE:    THIS ROUTINE WILL READ IN A VARIABLE LENGTH TD3240
C*                FORMAT HOURLY PRECIPITATION FILE AND WRITE OUT A FIXED
C*                LENGTH PRECIPITATION SCRATCH FILE.
C*
C*    WRITTEN BY: JAYANT A. HARDIKAR
C*                PES, INC
C*
C*    DATE:       2/23/95
C*
C*    INPUT:      Variable-format TD-3240 file
C*
C*    OUTPUT:     Fixed-format temporary file
C*
C*    CALLED BY:  FILES
C----------------------------------------------------------------------

      CHARACTER*27 TEXT1
      CHARACTER*12 TEXT2(100)
      INTEGER NUMTXT

      DO 500 I = 1,999999

         READ (INPPT,6666,END=999) TEXT1,NUMTXT,(TEXT2(J),J=1,NUMTXT)
6666     FORMAT(A27,I3,100(A12))

         DO 400 II = 1,NUMTXT
            WRITE (INPPT+50,7777) TEXT1,TEXT2(II)
7777        FORMAT(A27,3X,A12)
400      CONTINUE

C*    Note: the unit number is changed when returned to the calling routine

500   CONTINUE

999   RETURN
      END
      SUBROUTINE LWRUPR (RUNST1)
C***********************************************************************
C*                LWRUPR Module of ISC2 Model
C*
C*       PURPOSE: Transfer All Characters From Lower Case To
C*                Upper Case (Using INDEX IntrinsiC*Function)
C*                Note that the CHAR*80 RUNST1 Variable Includes
C*                the Original Case for Echoing and for Later Use
C*                To Retrieve Filenames.
C*
C*       PROGRAMMER: Roger Brode, Kevin Stroupe
C*
C*       DATE:    March 2, 1992
C*
C*       MODIFIED:   Jayant Hardikar, PES, Inc.
C*                   February 13, 1995
C*                   Adapted for PCRAMMET - Made Generic
C*
C*       INPUTS:  Input Runstream Card Image (80 Character Array)
C*                Number of Characters in String, PARAMETER ISTRG
C*
C*       OUTPUTS: Input Runstream Card Image (Array) in Uppercase
C*
C*       CALLED FROM:   READIN
C***********************************************************************
C*
C*    Variable Declarations
      PARAMETER (ISTRG = 80)
      CHARACTER UPCASE*26
      CHARACTER LWCASE*26
      CHARACTER*1 RUNST(ISTRG)
      CHARACTER   RUNST1*80

C*    Variable Initializations
      DATA UPCASE/'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      DATA LWCASE/'abcdefghijklmnopqrstuvwxyz'/

      DO 10 I = 1, ISTRG
         READ (RUNST1(I:I),'(A1)') RUNST(I)
10    CONTINUE

      DO 20 I = 1, ISTRG
         IF (RUNST(I) .NE. ' ') THEN
            INDCHK = INDEX(LWCASE,RUNST(I))
            IF (INDCHK .NE. 0) THEN
               RUNST(I) = UPCASE(INDCHK:INDCHK)
            END IF
         END IF
 20   CONTINUE

      DO 30 I = 1, ISTRG
         WRITE (RUNST1(I:I),'(A1)') RUNST(I)
30    CONTINUE


      RETURN
      END


      SUBROUTINE JULIAN(IYR,IMO,IDY,JDAY,ITYPE)
C***********************************************************************
C*    JULIAN Utility Module
C*
C*    PURPOSE:    Converts dates between Julian and mo/day format
C*
C*    INPUTS:     Year, month, day   OR  Julian day and year
C*                ITYPE: 0 - CONVERT IMO/IDY TO JDAY
C*                       1 - CONVERT JDAY TO IMO/IDY
C*
C*    OUTPUTS:    Julian day and year   OR  year, month, day
C*
C*    CALLED FROM: Various subprograms
C***********************************************************************

C*    Variable Declarations
      dimension iofset(13,2)

      data iofset/
     $ 0,31,59,90,120,151,181,212,243,273,304,334,365,
     $ 0,31,60,91,121,152,182,213,244,274,305,335,366/
C
C**** SEE IF WE ARE IN A LEAP YEAR
C
      if( mod(iyr,4).eq.0 ) then
         ileap = 2
      else
         ileap = 1
      endif
C
C**** SEE WHICH DIRECTION WE ARE GOING
C
      if(itype.eq.0) then
         jday = iofset(imo,ileap) + idy
      else
         do 10 j=2,13
            if(jday.le.iofset(j,ileap)) then
               imo = j-1
               idy = jday - iofset(imo,ileap)
               go to 999
            endif
   10    continue
      endif
  999 continue
      return
      end


      SUBROUTINE HR0024(IYR,JULDAY,IHR)
C***********************************************************************
C*    HR0024 Utility Module
C*
C*    PURPOSE:    To convert a date/time group from hour=0 to hour=24
C*                of the previous day
C*
C*    INPUTS:     Julian day and hour
C*
C*    OUTPUTS:    Julian day and hour
C*
C*    CALLED FROM: FIL144, READPP
C***********************************************************************

C*    Variable Declarations
C
      INTEGER IYR, JULDAY, IHR

C*    IYR    = YEAR
C*    JULDAY = JULIAN DAY
C*    IHR    = IHR

C*    Set hour 00 to hour 24
      IHR = 24

      IF(JULDAY .EQ. 1) THEN
C*       The following code will be processed only for January 1
         IYR = IYR - 1

C*       Account for Y2K
         IF( IYR .LT. 0 )THEN
            IYR = 99
         ENDIF

         IF( MOD(IYR,4).EQ.0 )THEN
            JULDAY = 366
         ELSE
            JULDAY = 365
         ENDIF
      ELSE
         JULDAY = JULDAY - 1
      ENDIF
C
      RETURN
      END


      SUBROUTINE SUN(JULDAY)
C***********************************************************************
C*    SUN Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE: This routine calculates the sunrise and sunset times
C*             of the day from the date, latitude, longitude, and time
C*             zone
C*
C*    PROGRAMMER: Jayant Hardikar
C*                PES Inc.
C*                Adapted From MPRM
C*
C*    DATE:       February 23, 1995
C*
C*    INPUTS:     Latitude, longitude, Julian day
C*
C*    OUTPUTS:    Sunrise, sunset, solar elevation for each hour of day
C*
C*    CALLED FROM: MAIN program
C***********************************************************************
C*
C*    Variable Declarations
      INCLUDE 'PCRAM.INC'

C*    ZONE 05 = EASTERN
C*    ZONE 06 = CENTRAL
C*    ZONE 07 = MOUNTAIN
C*    ZONE 08 = PACIFIC
C*
      DUM    = XLON/15.0 - ITZONE
      TEMPZ  = 15.0*ITZONE - XLON
      SINLAT = SIN(XLAT/CONST )
      COSLAT = COS(XLAT/CONST )

C---- Determine the angular(radians) fraction of a year for this date;
C     the constant 0.0172028=360./(365.242*57.29578)
      DAYNO  = (JULDAY-1.0)*0.0172028
      TDAYNO = 2.0*DAYNO
      SIND   = SIN(DAYNO)
      COSD   = COS(DAYNO)
      SINTD  = SIN(TDAYNO)
      COSTD  = COS(TDAYNO)
C
C---- Account for ellipticity of earth's orbit.

      SIGMA = 279.9348 + (DAYNO*CONST) + 1.914827*SIND -
     &        0.079525*COSD + 0.019938*SINTD - 0.00162*COSTD

C---- Find the sine of the solar declination;
C     the constant 0.39785=sin(0.4091720193=23.44383/57.29578)

      DSIN = 0.39785*SIN(SIGMA/CONST)
      DCOS = SQRT(1.0-DSIN*DSIN)

C---- Determine time(hrs) of meridian passage

      AMM  = 12.0 + 0.12357*SIND - 0.004289*COSD + 0.153809*SINTD +
     &                                             0.060783*COSTD
      HCOS = (-SINLAT*DSIN)/(COSLAT*DCOS)

C---- Determine solar hour angle of sunrise-sunset.
      H2 = (ATAN2(SQRT(1.0-HCOS*HCOS),HCOS)/15.0)*CONST

C---- Time of sunrise(SUNRIS) and time of sunset(SUNSET) are expressed
C     in local standard time since the zone correction has already
C     been made; otherwise they would be in Greenwich mean time.
      SUNRIS = AMM - H2 + DUM
      SUNSET = AMM + H2 + DUM


C*    Calculate the solar elevation for all (daytime) hours
      DO 100 IHOUR = 1,24

         IF( IHOUR .LE. SUNRIS ) THEN
            SOLANG(IHOUR) = 0.0
         ELSEIF( IHOUR .GE. SUNSET )THEN
            SOLANG(IHOUR) = 0.0
         ELSE
C----       Determine the angle of elevation and solar hour angle(radians)
            HI    = (15.0*(IHOUR-AMM)+TEMPZ)/CONST
            ALFSN = SINLAT*DSIN + DCOS*COSLAT*COS(HI)

C----       Determine solar elevation angle (in degrees).
            ALF   = ATAN2(ALFSN,SQRT(1.0-ALFSN*ALFSN))*CONST
            SOLANG(IHOUR) = ALF
         ENDIF

  100 CONTINUE

      RETURN
      END


      SUBROUTINE PARSER (RUNSTG,FIELDS,IFCC)
C***********************************************************************
C*    PARSER Module of PCRAMMET Meteorological Pre-processor
C*
C*    PURPOSE: A generic routine for parsing a line of space-delimited
C*             text into individual character variables
C*
C*    PROGRAMMER: Jayant Hardikar
C*
C*    DATE:    February 9, 1995
C*
C*    INPUTS:  Input character string (space delimited)
C*             Number of Characters in String
C*
C*    OUTPUTS: Array of parsed character Variables
C*
C*    CALLED FROM: SAMSON
C***********************************************************************
C
C     Variable Declarations
      PARAMETER (IFMAX = 27)
      PARAMETER (ISTRG = 256)

      CHARACTER*256  RUNSTG
      CHARACTER        FIELDS(IFMAX)* 40

      CHARACTER RUNST(ISTRG)*1
      CHARACTER FIELD(IFMAX)*40

      LOGICAL INFLD

      COMMON /FIELDS/ LOCB(IFMAX), LOCE(IFMAX), IFC
      COMMON /FLDCHR/ FIELD
      COMMON /LOGIN1/ INFLD
      COMMON /RUNSTR/ RUNST


      DO 20 I = 1,ISTRG
         READ (RUNSTG(I:I),'(A1)') RUNST(I)
20    CONTINUE


C*    Convert Lower Case to Upper Case Letters        ---   CALL LWRUPR
      CALL LWRUPR (RUNSTG)

C     Define Fields on Card                           ---   CALL DEFINE
      CALL DEFINE

C     Get the Contents of the Fields as Characters    ---   CALL GETFLD
      CALL GETFLD


      DO 10 I = 1,IFMAX
         FIELDS (I) = FIELD(I)
10    CONTINUE

      IFCC = IFC


      RETURN
      END


      SUBROUTINE DEFINE
C***********************************************************************
C     DEFINE Module of PCRAMMET Meteorological Pre-processor
C
C     PURPOSE: Defines Location of Fields on Runstream Input Image
C
C     PROGRAMMER: Jeff Wang, Roger Brode
C
C     DATE:       March 2, 1992
C
C     MODIFIED:   Jayant Hardikar - 2/9/95 : Made it generic
C
C     INPUTS:     One line of variables separated by spaces
C
C     OUTPUTS:    Number of Fields on Card, IFC
C                 Beginning and Ending Columns of Fields, LOCB and LOCE
C
C     CALLED FROM: PARSER
C***********************************************************************
C
C     Variable Declarations
      PARAMETER (IFMAX = 27)
      PARAMETER (ISTRG = 256)
      CHARACTER*1 RUNST
      CHARACTER*40 FIELD
      LOGICAL INFLD
      COMMON /FIELDS/ LOCB(IFMAX), LOCE(IFMAX), IFC
      COMMON /FLDCHR/ FIELD(IFMAX)
      COMMON /LOGIN1/ INFLD
      COMMON /RUNSTR/ RUNST(ISTRG)

C     Initialize the Blank Line and In-field Status Indicators
      INFLD = .FALSE.

C     Define the Starting Column for the Input Line In Case File Is Shifted.
C     Allow for Shift of Up to 3 Columns

      LOCB(1) = 0
      IF (RUNST(1) .NE. ' ') THEN
         LOCB(1) = 1
      ELSE IF (RUNST(2) .NE. ' ') THEN
         LOCB(1) = 2
      ELSE IF (RUNST(3) .NE. ' ') THEN
         LOCB(1) = 3
      ELSE IF (RUNST(4) .NE. ' ') THEN
         LOCB(1) = 4
      ELSE
         LOCB(1) = 1
      END IF

      IFC = 0

C     Loop through the Data Fields
      DO 20 I = LOCB(1), ISTRG

         IF (.NOT.INFLD .AND. RUNST(I).NE.' ') THEN
C           Location is the Beginning of a Field
C           Set Mark of in a Field
            INFLD = .TRUE.
C           Increment the Field Counter
            IFC = IFC + 1
C           Record the Location of Beginning of the Field
            LOCB(IFC) = I
         ELSE IF (INFLD .AND. RUNST(I).EQ.' ') THEN
C           Location is the End of a Field
C           Set Mark of Not In a field
            INFLD = .FALSE.
C           Record the Location of Ending of the Field
            LOCE(IFC) = I - 1
         END IF

C        Check for End of Input String
         IF (INFLD .AND. I.EQ.ISTRG) THEN
            LOCE(IFC) = ISTRG
         END IF

 20   CONTINUE

      RETURN
      END

      SUBROUTINE GETFLD
C***********************************************************************
C     GETFLD Module of PCRAMMET Meteorological Pre-processor
C
C     PURPOSE: Gets Contents of Fields on Runstream Input Image
C
C     PROGRAMMER: Roger Brode, Jeff Wang
C
C     DATE:    March 2, 1992
C
C     MODIFIED:   Jayant Hardikar - 2/9/95 : Made it generic
C
C     INPUTS:     Fields on a line
C
C     OUTPUTS: Contents of Fields on Card
C
C     CALLED FROM:   PARSER
C***********************************************************************
C
C     Variable Declarations
      PARAMETER (IFMAX = 27)
      PARAMETER (ISTRG = 256)
      CHARACTER*1 RUNST
      CHARACTER*40 FIELD
      LOGICAL INFLD

      COMMON /FIELDS/ LOCB(IFMAX), LOCE(IFMAX), IFC
      COMMON /FLDCHR/ FIELD(IFMAX)
      COMMON /LOGIN1/ INFLD
      COMMON /RUNSTR/ RUNST(ISTRG)


      DO 25 I = 1, IFC
         IF (LOCE(I)-LOCB(I) .LE. 39) THEN
C           Field Satisfies Limit of 40 Characters
            WRITE(FIELD(I),9004) (RUNST(J),J=LOCB(I),LOCE(I))
         ELSE
            WRITE(FIELD(I),9004) (RUNST(J),J=LOCB(I),LOCB(I)+39)
         END IF
 25   CONTINUE

 9004 FORMAT(40(A1:))

      RETURN
      END

      SUBROUTINE STONUM(STRVAR,LENGTH,FNUM,IMUTI)
C***********************************************************************
C     STONUM Module of PCRAMMET Meteorological Pre-processor
C
C     PURPOSE: Gets Number From A String Variable
C
C     PROGRAMMER: Jeff Wang, Roger Brode
C
C     DATE:    March 2, 1992
C
C     INPUTS:  Input String Variable Length of Character String
C
C     OUTPUTS: Numbers
C
C     CALLED FROM: (This Is A Utility Program)
C***********************************************************************
C
C     Variable Declarations
      CHARACTER STRVAR*(*), CHK, NUMS*10
      REAL FNUM, CNUM
      LOGICAL MEND, IN, NMARK, PMARK, DMARK, MMARK, EMARK

C     Variable Initialization
      NUMS = '0123456789'
      I = 1
      MEND = .FALSE.
      IN = .FALSE.
      NMARK = .FALSE.
      PMARK = .FALSE.
      DMARK = .FALSE.
      MMARK = .FALSE.
      EMARK = .FALSE.
      CNUM  = 0.0
      IMUTI = 1
      FDEC  = 1.

C     Beginning the Processing
      DO WHILE (.NOT.MEND .AND. I.LE.LENGTH)
         CHK = STRVAR(I:I)
         IF (CHK .NE. ' ') THEN
            IN = .TRUE.
            IF (CHK.GE.'0' .AND. CHK.LE.'9') THEN
C              CHK is a Number, Assign a Value
               IF (.NOT. DMARK) THEN
                  CNUM = CNUM*10.+FLOAT(INDEX(NUMS,CHK)-1)
               ELSE
                  FDEC = FDEC/10.
                  FDC1 = FDEC*FLOAT(INDEX(NUMS,CHK)-1)
                  CNUM = CNUM+FDC1
               END IF
            ELSE
C              Handle The E-Type Real Number
               IF (.NOT.EMARK .AND. CHK.EQ.'E') THEN
                  EMARK = .TRUE.
                  IF (.NOT.NMARK) THEN
                     HEAD = CNUM
                  ELSE
                     HEAD = -CNUM
                  END IF
                  DMARK = .FALSE.
                  NMARK = .FALSE.
                  CNUM = 0.0
               ELSE IF (.NOT.PMARK .AND. CHK.EQ.'+') THEN
C                 Set Positive Indicator
                  PMARK = .TRUE.
               ELSE IF (.NOT.NMARK .AND. CHK.EQ.'-') THEN
C                 Set Negative Indicator
                  NMARK = .TRUE.
               ELSE IF (.NOT.DMARK .AND. CHK.EQ.'.') THEN
C                 Set Decimal Indicator
                  DMARK = .TRUE.
               ELSE IF (.NOT.MMARK .AND. CHK.EQ.'*' .AND.
     &                  .NOT.NMARK) THEN
C                 Set Repeat Number
                  MMARK = .TRUE.
                  IMUTI = INT(CNUM)
                  CNUM = 0.0
               ELSE
C                 Error Occurs, Set Switch and Exit Out Of The Subroutine
                  GO TO 9999
               END IF
            END IF
         ELSE IF (IN .AND. CHK.EQ.' ') THEN
            MEND = .TRUE.
         END IF
         I = I + 1
      END DO

      FNUM = CNUM

C     In Case Of Negative Field, Value Set to Negative
      IF (NMARK) THEN
         FNUM = -FNUM
      END IF

C     In Case of E-Format, Check for Exponents Out of Range
      IF (EMARK .AND. ABS(FNUM) .LE. 30.) THEN
         FNUM = HEAD*10**(FNUM)
      ELSE IF (EMARK .AND. ABS(FNUM) .GT. 30.) THEN
         IF (FNUM .LT. 0.0) THEN
            FNUM = 0.0
         ELSE IF (FNUM .GT. 0.0) THEN
            FNUM = HEAD * 10**30.
         END IF
         GO TO 9999
      END IF

      GO TO 1000

C     Set Error Switch for Illegal Numerical Field (WRITE Message and Handle
C     Error in Calling Routine)
 9999 IMUTI = -1

 1000 RETURN
      END


      SUBROUTINE ERRHDL(INERTP,INERCD,IWRITE)
C***********************************************************************
C     ERRHDL   Module of PCRAMMET Meteorological Pre-processor
C
C     PURPOSE: A General Error Handling Procedure
C
C     PROGRAMMED BY: PES, Inc.
C
C     DATE:
C
C     INPUTS:  Error Type (INERTP), Error Code (INERCD)
C              Whether to print the month & day (IWRITE=1)
C              or not (IWRITE=0)
C
C     OUTPUTS: Error Message
C
C     CALLED FROM: Various subprograms
C***********************************************************************
C
C     Variable Declarations
      INCLUDE 'PCRAM.INC'
      CHARACTER ERRMG1*72, INERTP*1, INERCD*4, SAVCD*4
      LOGICAL FIND

C     Variable Initializations
      FIND = .FALSE.
      I = 1

C     Match The Error Message
      DO WHILE (.NOT.FIND .AND. I.LE.MAXERR)
         IF (INERCD .EQ. ERRCOD(I)) THEN
            IERRDX = I
            ERRMG1 = ERRMSG(I)
            FIND = .TRUE.
         END IF
         I = I + 1
      END DO

      IF (.NOT. FIND) THEN
         INERTP = 'E'
         SAVCD  = INERCD
         INERCD = '----'
         WRITE(ERRMG1,1001) SAVCD
 1001    FORMAT('SYSTEM ERROR: MESSAGE NOT FOUND FOR NUMBER ',A4)
      END IF

C     Write Out The Error Message
      IF( IWRITE .EQ. 1 )THEN
         WRITE(IDIAG,1111) INERTP,INERCD,ERRMG1,ISMO(1),ISDY(1)
      ELSEIF( IWRITE .EQ. 0 )THEN
         WRITE(IDIAG,1111) INERTP,INERCD,ERRMG1
      ENDIF
 1111 FORMAT(A1,A4,1X,': ',A72: ,/,7X,' on (month,day) ', i3, ',',i3)

      IF (INERTP .EQ. 'E') THEN
         WRITE (*,*) '            ****** FATAL ERROR ******        '
         WRITE (*,*) '               PROGRAM TERMINATED            '
         WRITE (*,*) '    See the Output File PCRAM.LOG for Details'
         STOP
      ENDIF


 999  RETURN
      END
